@@ -2,7 +2,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,62 +13,14 @@ from __future__ import print_function
 import os, sys, time, shutil
 
 def AddDefine(conffile,name,value,prefix='SLEPC_'):
-  conffile.write('#ifndef '+prefix+name+'\n#define '+prefix+name+' "'+value+'"\n#endif\n\n')
-
-def CreateFile(basedir,fname,log):
-  ''' Create file basedir/fname and return path string '''
-  newfile = os.path.join(basedir,fname)
-  try:
-    newfile = open(newfile,'w')
-  except:
-    log.Exit('ERROR: Cannot create '+fname+' file in '+basedir)
-  return newfile
-
-def CreateDir(basedir,dirname,log):
-  ''' Create directory basedir/dirname and return path string '''
-  newdir = os.path.join(basedir,dirname)
-  if not os.path.exists(newdir):
-    try:
-      os.mkdir(newdir)
-    except:
-      log.Exit('ERROR: Cannot create '+dirname+' directory: '+newdir)
-  return newdir
-
-def CreateDirTwo(basedir,dir1,dir2,log):
-  ''' Create directory basedir/dir1/dir2 and return path string '''
-  newbasedir = os.path.join(basedir,dir1)
-  if not os.path.exists(newbasedir):
-    try:
-      os.mkdir(newbasedir)
-    except:
-      log.Exit('ERROR: Cannot create '+dir1+' directory: '+newbasedir)
-  newdir = os.path.join(newbasedir,dir2)
-  if not os.path.exists(newdir):
-    try:
-      os.mkdir(newdir)
-    except:
-      log.Exit('ERROR: Cannot create '+dirname+' directory: '+newdir)
-  return newdir
-
-def CreateDirTest(basedir,dirname,log):
-  ''' Create directory, return path string and flag indicating if already existed '''
-  newdir = os.path.join(basedir,dirname)
-  if not os.path.exists(newdir):
-    existed = False
-    try:
-      os.mkdir(newdir)
-    except:
-      log.Exit('ERROR: Cannot create '+dirname+' directory: '+newdir)
-  else:
-    existed = True
-  return newdir, existed
+  conffile.write('#define '+prefix+name+' "'+value+'"\n')
 
 def WriteModulesFile(modules,version,sdir):
   ''' Write the contents of the Modules file '''
   modules.write('#%Module\n\n')
   modules.write('proc ModulesHelp { } {\n')
   modules.write('    puts stderr "This module sets the path and environment variables for slepc-%s"\n' % version)
-  modules.write('    puts stderr "     see http://slepc.upv.es/ for more information"\n')
+  modules.write('    puts stderr "     see https://slepc.upv.es/ for more information"\n')
   modules.write('    puts stderr ""\n}\n')
   modules.write('module-whatis "SLEPc - Scalable Library for Eigenvalue Problem Computations"\n\n')
   modules.write('module load petsc\n')
@@ -94,22 +46,6 @@ def WritePkgconfigFile(pkgconfig,version,pversion,sdir,isinstall,prefixdir,singl
   else:
     pkgconfig.write(' -L${libdir} -lslepcnep -lslepcpep -lslepcsvd -lslepceps -lslepcmfn -lslepclme -lslepcsys\n')
 
-def WriteCMakeConfigFile(cmakeconf):
-  ''' Write the contents of the CMake configuration file '''
-  cmakeconf.write('''
-set (SLEPC_PACKAGE_LIBS "${ARPACK_LIB}" "${BLZPACK_LIB}" "${TRLAN_LIB}" "${PRIMME_LIB}" "${FEAST_LIB}" "${BLOPEX_LIB}" "${SLICOT_LIB}" )
-set (SLEPC_PACKAGE_INCLUDES "${PRIMME_INCLUDE}")
-find_library (PETSC_LIB petsc HINTS ${PETSc_BINARY_DIR}/lib )
-if (NOT PETSC_LIB) # Interpret missing libpetsc to mean that PETSc was built --with-single-library=0
-  set (PETSC_LIB "")
-  foreach (pkg sys vec mat dm ksp snes ts tao)
-    string (TOUPPER ${pkg} PKG)
-    find_library (PETSC${PKG}_LIB "petsc${pkg}" HINTS ${PETSc_BINARY_DIR}/lib)
-    list (APPEND PETSC_LIB "${PETSC${PKG}_LIB}")
-  endforeach ()
-endif ()
-''')
-
 def WriteReconfigScript(reconfig,slepcdir,usedargs):
   ''' Write the contents of the reconfigure script '''
   reconfig.write('#!/usr/bin/env python\n\n')
@@ -123,10 +59,10 @@ if 'LC_LOCAL' in os.environ and os.environ['LC_LOCAL'] != '' and os.environ['LC_
 if 'LANG' in os.environ and os.environ['LANG'] != '' and os.environ['LANG'] != 'en_US' and os.environ['LANG'] != 'en_US.UTF-8': os.environ['LANG'] = 'en_US.UTF-8'
 
 # Check python version
-if not hasattr(sys, 'version_info') or not sys.version_info[0] == 2 or not sys.version_info[1] >= 6:
+if sys.version_info < (2,6) or (sys.version_info >= (3,0) and sys.version_info < (3,4)):
   print('*******************************************************************************')
-  print('*       Python2 version 2.6 or higher is required to run ./configure          *')
-  print('*          Try: "python2.7 ./configure" or "python2.6 ./configure"            *')
+  print('*        Python version 2.6+ or 3.4+ is required to run ./configure           *')
+  print('*           Try: "python2.7 ./configure" or "python3 ./configure"             *')
   print('*******************************************************************************')
   sys.exit(4)
 
@@ -143,22 +79,22 @@ argdb = argdb.ArgDB(sys.argv)
 log   = log.Log()
 
 # Load classes for packages and process command-line options
-import slepc, petsc, arpack, blzpack, trlan, feast, primme, blopex, sowing, lapack, slicot
+import slepc, petsc, arpack, blzpack, trlan, primme, blopex, sowing, lapack, slicot, hpddm
 slepc   = slepc.SLEPc(argdb,log)
 petsc   = petsc.PETSc(argdb,log)
 arpack  = arpack.Arpack(argdb,log)
 blopex  = blopex.Blopex(argdb,log)
 blzpack = blzpack.Blzpack(argdb,log)
-feast   = feast.Feast(argdb,log)
 primme  = primme.Primme(argdb,log)
 trlan   = trlan.Trlan(argdb,log)
 sowing  = sowing.Sowing(argdb,log)
 lapack  = lapack.Lapack(argdb,log)
 slicot  = slicot.Slicot(argdb,log)
+hpddm   = hpddm.HPDDM(argdb,log)
 
-externalpackages = [arpack, blopex, blzpack, feast, primme, slicot, trlan]
-optionspackages  = [slepc, arpack, blopex, blzpack, feast, primme, slicot, trlan, sowing]
-checkpackages    = [arpack, blopex, blzpack, feast, primme, slicot, trlan, lapack]
+externalpackages = [arpack, blopex, blzpack, hpddm, primme, slicot, trlan]
+optionspackages  = [slepc, sowing] + externalpackages
+checkpackages    = externalpackages + [lapack]
 
 # Print help if requested and check for wrong command-line options
 if argdb.PopHelp():
@@ -167,26 +103,23 @@ if argdb.PopHelp():
   for pkg in optionspackages:
     pkg.ShowHelp()
   sys.exit(0)
+argdb.ErrorPetscOptions()
 argdb.ErrorIfNotEmpty()
 
 # Check enviroment and PETSc version
-print('Checking environment...', end=' ')
+log.Print('Checking environment...')
 petsc.InitDir(slepc.prefixdir)
 slepc.InitDir()
 petsc.LoadVersion()
 slepc.LoadVersion()
-if petsc.version < slepc.version:
-  sys.exit('ERROR: This SLEPc version is not compatible with PETSc version '+petsc.version)
 
-# Check some information about PETSc configuration
+# Load PETSc configuration
 petsc.LoadConf()
-if not petsc.precision in ['double','single','__float128']:
-  sys.exit('ERROR: This SLEPc version does not work with '+petsc.precision+' precision')
 
 # Check for empty PETSC_ARCH
 emptyarch = not ('PETSC_ARCH' in os.environ and os.environ['PETSC_ARCH'])
 if emptyarch:
-  pseudoarch = 'arch-' + sys.platform.replace('cygwin','mswin')+ '-' + petsc.language
+  pseudoarch = 'arch-' + sys.platform.replace('cygwin','mswin')+ '-' + petsc.language.lower().replace('+','x')
   if petsc.debug:
     pseudoarch += '-debug'
   else:
@@ -198,18 +131,26 @@ else:
   archname = petsc.arch
 
 # Create directories for configuration files
-archdir, archdirexisted = CreateDirTest(slepc.dir,archname,log)
-libdir  = CreateDir(archdir,'lib',log)
-confdir = CreateDirTwo(libdir,'slepc','conf',log)
+archdir, archdirexisted = slepc.CreateDirTest(slepc.dir,archname)
+libdir  = slepc.CreateDir(archdir,'lib')
+confdir = slepc.CreateDirTwo(libdir,'slepc','conf')
 
 # Open log file
-log.Open(os.path.join(confdir,'configure.log'))
+log.Open(slepc.dir,confdir,'configure.log')
 log.write('='*80)
 log.write('Starting Configure Run at '+time.ctime(time.time()))
 log.write('Configure Options: '+' '.join(sys.argv[1:]))
 log.write('Working directory: '+os.getcwd())
 log.write('Python version:\n'+sys.version)
 log.write('make: '+petsc.make)
+
+# Some checks related to PETSc configuration
+if petsc.nversion < slepc.nversion:
+  log.Exit('This SLEPc version is not compatible with PETSc version '+petsc.version)
+if not petsc.precision in ['double','single','__float128']:
+  log.Exit('This SLEPc version does not work with '+petsc.precision+' precision')
+
+# Display versions and paths
 log.write('PETSc source directory: '+petsc.dir)
 log.write('PETSc install directory: '+petsc.prefixdir)
 log.write('PETSc version: '+petsc.lversion)
@@ -222,25 +163,18 @@ log.write('SLEPc version: '+slepc.lversion)
 
 # Clean previous configuration if needed
 if archdirexisted:
-  if not slepc.clean:
-    try:
-      f = open(os.path.join(confdir,'slepcvariables'),'r')
-      searchlines = f.readlines()
-      f.close()
-      if any(pkg.packagename.upper() in ''.join(searchlines) for pkg in externalpackages) and not any(pkg.requested for pkg in externalpackages):
-        log.Print('\nWARNING: forcing --with-clean=1 because previous configuration had external packages')
-        slepc.clean = True
-    except: pass
+  if slepc.isinstall and not slepc.clean:
+    log.Exit('You are requesting a prefix install but the arch directory '+archdir+' already exists and may contain files from previous builds; consider adding option --with-clean')
   if slepc.clean:
-    log.Print('\nCleaning arch dir '+archdir+'...')
+    log.Println('\nCleaning arch dir '+archdir+'...')
     try:
       for root, dirs, files in os.walk(archdir,topdown=False):
         for name in files:
           if name!='configure.log':
             os.remove(os.path.join(root,name))
     except:
-      log.Exit('ERROR: Cannot remove existing files in '+archdir)
-    for rdir in ['CMakeFiles','obj','externalpackages']:
+      log.Exit('Cannot remove existing files in '+archdir)
+    for rdir in ['obj','externalpackages']:
       try:
         shutil.rmtree(os.path.join(archdir,rdir))
       except: pass
@@ -248,32 +182,30 @@ if archdirexisted:
 # Create other directories and configuration files
 if not slepc.prefixdir:
   slepc.prefixdir = archdir
-includedir = CreateDir(archdir,'include',log)
-modulesdir = CreateDirTwo(confdir,'modules','slepc',log)
-pkgconfdir = CreateDir(libdir,'pkgconfig',log)
-slepcvars  = CreateFile(confdir,'slepcvariables',log)
-slepcrules = CreateFile(confdir,'slepcrules',log)
-slepcconf  = CreateFile(includedir,'slepcconf.h',log)
-cmakeconf  = CreateFile(confdir,'SLEPcBuildInternal.cmake',log)
-pkgconfig  = CreateFile(pkgconfdir,'SLEPc.pc',log)
+includedir = slepc.CreateDir(archdir,'include')
+modulesdir = slepc.CreateDirTwo(confdir,'modules','slepc')
+pkgconfdir = slepc.CreateDir(libdir,'pkgconfig')
+slepcvars  = slepc.CreateFile(confdir,'slepcvariables')
+slepcconf  = slepc.CreateFile(includedir,'slepcconf.h')
+pkgconfig  = slepc.CreateFile(pkgconfdir,'SLEPc.pc')
 if slepc.isinstall:
-  modules  = CreateFile(modulesdir,slepc.lversion,log)
+  modules  = slepc.CreateFile(modulesdir,slepc.lversion)
 else:
-  modules  = CreateFile(modulesdir,slepc.lversion+'-'+archname,log)
-  reconfig = CreateFile(confdir,'reconfigure-'+archname+'.py',log)
+  modules  = slepc.CreateFile(modulesdir,slepc.lversion+'-'+archname)
+  reconfig = slepc.CreateFile(confdir,'reconfigure-'+archname+'.py')
   reconfigpath = os.path.join(confdir,'reconfigure-'+archname+'.py')
 
 # Write initial part of file slepcvariables
 slepcvars.write('SLEPC_CONFIGURE_OPTIONS = '+argdb.UsedArgs()+'\n')
-slepcvars.write('SLEPC_DESTDIR = '+slepc.prefixdir+'\n')
+slepcvars.write('SLEPC_INSTALLDIR = '+slepc.prefixdir+'\n')
 if emptyarch:
   slepcvars.write('INSTALLED_PETSC = 1\n')
 if slepc.datadir:
   slepcvars.write('DATAFILESPATH = '+slepc.datadir+'\n')
 
 # Write initial part of file slepcconf.h
-slepcconf.write('#if !defined(__SLEPCCONF_H)\n')
-slepcconf.write('#define __SLEPCCONF_H\n\n')
+slepcconf.write('#if !defined(SLEPCCONF_H)\n')
+slepcconf.write('#define SLEPCCONF_H\n\n')
 AddDefine(slepcconf,'PETSC_DIR',petsc.dir)
 AddDefine(slepcconf,'PETSC_ARCH',petsc.arch)
 AddDefine(slepcconf,'DIR',slepc.dir)
@@ -285,25 +217,25 @@ if slepc.isrepo:
 
 # Create global configuration file for the case of empty PETSC_ARCH
 if emptyarch:
-  globconf = CreateFile(os.path.join(slepc.dir,'lib','slepc','conf'),'slepcvariables',log)
+  globconf = slepc.CreateFile(os.path.join(slepc.dir,'lib','slepc','conf'),'slepcvariables')
   globconf.write('SLEPC_DIR = '+slepc.dir+'\n')
   globconf.write('PETSC_ARCH = '+archname+'\n')
   globconf.close()
 
 # Check if PETSc is working
 log.NewSection('Checking PETSc installation...')
-if petsc.version > slepc.version:
-  log.Println('\nWARNING: PETSc version '+petsc.version+' is newer than SLEPc version '+slepc.version)
+if petsc.nversion > slepc.nversion:
+  log.Warn('PETSc version '+petsc.version+' is newer than SLEPc version '+slepc.version)
 if slepc.release=='1' and not petsc.release=='1':
-  log.Exit('ERROR: a release version of SLEPc requires a release version of PETSc, not a development version')
+  log.Exit('A release version of SLEPc requires a release version of PETSc, not a development version')
 if slepc.release=='0' and petsc.release=='1':
-  log.Exit('ERROR: a development version of SLEPc cannot be built with a release version of PETSc')
+  log.Exit('A development version of SLEPc cannot be built with a release version of PETSc')
 if petsc.isinstall:
   if os.path.realpath(petsc.prefixdir) != os.path.realpath(petsc.dir):
-    log.Println('\nWARNING: PETSC_DIR does not point to PETSc installation path')
+    log.Warn('PETSC_DIR does not point to PETSc installation path')
 petsc.Check()
 if not petsc.havepackage:
-  log.Exit('ERROR: Unable to link with PETSc')
+  log.Exit('Unable to link with PETSc')
 
 # Single library installation
 if petsc.singlelib:
@@ -315,29 +247,9 @@ if petsc.singlelib:
 
 # Check for external packages and for missing LAPACK functions
 for pkg in checkpackages:
-  pkg.Process(slepcconf,slepcvars,cmakeconf,petsc,archdir)
+  pkg.Process(slepcconf,slepcvars,slepc,petsc,archdir)
 
-# Determine which tests must be run
-testruns = set(petsc.test_runs.split())
-supported_tests = set(['C','C_Complex','C_NoComplex','C_NotSingle','C_NoComplex_NotSingle','VECCUDA','VECCUDA_Complex','VECCUDA_NoComplex'])
-if petsc.fortran:
-  supported_tests = supported_tests.union(set(['F90','Fortran','F90_Complex','Fortran_Complex','F90_NoComplex','Fortran_NoComplex','F90_NotSingle','Fortran_NotSingle','Fortran_NoComplex_NotSingle']))
-testruns = testruns.intersection(supported_tests)
-if not petsc.mpiuni:
-  if petsc.precision == 'double':
-    testruns = testruns.union(set(['C_Double']))
-  if petsc.precision == 'double' and petsc.scalar == 'real' and not petsc.ind64:
-    testruns = testruns.union(set(['C_DataFile']))
-  if slepc.datadir:
-    if petsc.scalar == 'complex':
-      testruns = testruns.union(set(['DATAFILESPATH_Complex']))
-    else:
-      testruns = testruns.union(set(['DATAFILESPATH']))
-  for pkg in externalpackages:
-    testruns = testruns.union(set(pkg.TestRuns(petsc)))
-slepcvars.write('TEST_RUNS = '+' '.join(testruns)+'\n')
-
-# Write Modules, pkg-config and CMake configuration files
+# Write Modules and pkg-config configuration files
 log.NewSection('Writing various configuration files...')
 log.write('Modules file in '+modulesdir)
 if slepc.isinstall:
@@ -345,11 +257,7 @@ if slepc.isinstall:
 else:
   WriteModulesFile(modules,slepc.lversion,slepc.dir)
 log.write('pkg-config file in '+pkgconfdir)
-slflag = ''
-if petsc.buildsharedlib: slflag = petsc.slflag
 WritePkgconfigFile(pkgconfig,slepc.lversion,petsc.version,slepc.dir,slepc.isinstall,slepc.prefixdir,petsc.singlelib)
-log.write('CMake configure file in '+confdir)
-WriteCMakeConfigFile(cmakeconf)
 
 # Write reconfigure file
 if not slepc.isinstall:
@@ -357,13 +265,11 @@ if not slepc.isinstall:
   try:
     os.chmod(reconfigpath,0o775)
   except OSError as e:
-    log.Exit('ERROR: Unable to make reconfigure script executable:\n'+str(e))
+    log.Exit('Unable to make reconfigure script executable:\n'+str(e))
 
 # Finish with configuration files (except slepcvars)
-slepcrules.close()
-slepcconf.write('#endif\n')
+slepcconf.write('\n#endif\n')
 slepcconf.close()
-cmakeconf.close()
 pkgconfig.close()
 modules.close()
 if not slepc.isinstall: reconfig.close()
@@ -371,14 +277,14 @@ if not slepc.isinstall: reconfig.close()
 # Download sowing if requested and make Fortran stubs if necessary
 bfort = petsc.bfort
 if sowing.downloadpackage:
-  bfort = sowing.Install(archdir,petsc.make)
+  bfort = sowing.DownloadAndInstall(slepc,petsc,archdir)
 
 if slepc.isrepo and petsc.fortran:
   try:
     if not os.path.exists(bfort):
       bfort = os.path.join(archdir,'bin','bfort')
     if not os.path.exists(bfort):
-      bfort = sowing.Install(archdir,petsc.make)
+      bfort = sowing.DownloadAndInstall(slepc,petsc,archdir)
     log.NewSection('Generating Fortran stubs...')
     log.write('Using BFORT='+bfort)
     sys.path.insert(0, os.path.abspath(os.path.join('lib','slepc','bin','maint')))
@@ -386,54 +292,20 @@ if slepc.isrepo and petsc.fortran:
     generatefortranstubs.main(slepc.dir,bfort,os.getcwd(),0)
     generatefortranstubs.processf90interfaces(slepc.dir,0)
   except:
-    log.Exit('ERROR: Try configuring with --download-sowing or use a git version of PETSc')
+    log.Exit('Try configuring with --download-sowing or use a git version of PETSc')
 
 if bfort != petsc.bfort:
   slepcvars.write('BFORT = '+bfort+'\n')
 
-# CMake stuff
-cmakeok = False
-if slepc.cmake:
-  log.NewSection('Configuring CMake builds...')
-  if sys.version_info < (2,5):
-    log.Exit('ERROR: python version should be 2.5 or higher')
-  elif petsc.isinstall:
-    log.Exit('ERROR: CMake builds cannot be used with prefix-installed PETSc')
-  elif not petsc.build_using_cmake:
-    log.Exit('ERROR: CMake builds need a PETSc configured --with-cmake')
-  else:
-    import cmakegen
-    try:
-      cmakegen.main(slepc.dir,petsc.dir,petscprefixdir=petsc.prefixdir)
-    except (OSError) as e:
-      log.Exit('ERROR: Generating CMakeLists.txt failed:\n'+str(e))
-    import cmakeboot
-    try:
-      cmakeok = cmakeboot.main(slepc.dir,petsc.dir,log=log)
-    except (OSError) as e:
-      log.Exit('ERROR: Booting CMake in PETSC_ARCH failed:\n'+str(e))
-    except (ImportError, KeyError) as e:
-      log.Exit('ERROR: Importing cmakeboot failed:\n'+str(e))
-    except (AttributeError) as e:
-      log.Println('\nxxx'+'='*73+'xxx')
-      log.Println('WARNING: CMake builds are not available (initialization failed)')
-      log.Println('You can ignore this warning (use default build), or try reconfiguring PETSc')
-      log.Println('xxx'+'='*73+'xxx')
-    # remove files created by PETSc's script
-    for f in ['build.log','build.log.bkp','RDict.log']:
-      try: os.remove(f)
-      except OSError: pass
-if cmakeok:
-  slepcvars.write('SLEPC_BUILD_USING_CMAKE = 1\n')
-
 # Finally we can close the slepcvariables file
 slepcvars.close()
 
 # Print summary
-log.NewSection('\n')
-log.Println('='*79)
+log.NewSection('')
+log.Println('')
+log.Println('='*80)
 log.Println('SLEPc Configuration')
-log.Println('='*79)
+log.Println('='*80)
 log.Println('\nSLEPc directory:\n '+slepc.dir)
 if slepc.isrepo:
   log.Println('  It is a git repository on branch: '+slepc.branch)
@@ -448,9 +320,7 @@ if petsc.isrepo:
       petscdate = dateutil.parser.parse(petsc.gitdate)
       slepcdate = dateutil.parser.parse(slepc.gitdate)
       if abs(petscdate-slepcdate)>datetime.timedelta(days=30):
-        log.Println('xxx'+'='*73+'xxx')
-        log.Println('WARNING: your PETSc and SLEPc repos may not be in sync (more than 30 days apart)')
-        log.Println('xxx'+'='*73+'xxx')
+        log.Warn('Your PETSc and SLEPc repos may not be in sync (more than 30 days apart)')
     except ImportError: pass
 if emptyarch and slepc.isinstall:
   log.Println('Prefix install with '+petsc.precision+' precision '+petsc.scalar+' numbers')
@@ -459,16 +329,13 @@ else:
 for pkg in checkpackages:
   pkg.ShowInfo()
 log.write('\nFinishing Configure Run at '+time.ctime(time.time()))
-log.write('='*79)
+log.write('='*80)
 print()
-print('xxx'+'='*73+'xxx')
-if petsc.make_is_gnumake: buildtype = 'gnumake'
-elif cmakeok: buildtype = 'cmake'
-else: buildtype = 'legacy'
-print(' Configure stage complete. Now build the SLEPc library with ('+buildtype+' build):')
+print('xxx'+'='*74+'xxx')
+print(' Configure stage complete. Now build the SLEPc library with:')
 if emptyarch:
-  print('   make SLEPC_DIR=$PWD PETSC_DIR='+petsc.dir)
+  print('   make SLEPC_DIR='+slepc.dir+' PETSC_DIR='+petsc.dir)
 else:
-  print('   make SLEPC_DIR=$PWD PETSC_DIR='+petsc.dir+' PETSC_ARCH='+archname)
-print('xxx'+'='*73+'xxx')
+  print('   make SLEPC_DIR='+slepc.dir+' PETSC_DIR='+petsc.dir+' PETSC_ARCH='+archname)
+print('xxx'+'='*74+'xxx')
 print()
