@@ -1,15 +1,14 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 
-import os, commands
-import log, package
+import os, log, package
 
 class Arpack(package.Package):
 
@@ -18,11 +17,12 @@ class Arpack(package.Package):
     self.packagename    = 'arpack'
     self.installable    = True
     self.downloadable   = True
-    self.version        = '3.3.0'
+    self.version        = '3.7.0'
     self.url            = 'https://github.com/opencollab/arpack-ng/archive/'+self.version+'.tar.gz'
     self.archive        = 'arpack-ng-'+self.version+'.tar.gz'
     self.dirname        = 'arpack-ng-'+self.version
     self.supportssingle = True
+    self.supports64bint = True
     self.fortran        = True
     self.ProcessArgs(argdb)
 
@@ -52,7 +52,7 @@ class Arpack(package.Package):
     return functions
 
 
-  def Check(self,conf,vars,cmake,petsc):
+  def Check(self,conf,vars,petsc,archdir):
     functions = self.Functions(petsc)
     if self.packagelibs:
       libs = [self.packagelibs]
@@ -65,31 +65,34 @@ class Arpack(package.Package):
     if self.packagedir:
       dirs = [self.packagedir]
     else:
-      dirs = self.GenerateGuesses('Arpack')
-
-    self.FortranLib(conf,vars,cmake,dirs,libs,functions)
+      dirs = self.GenerateGuesses('Arpack',archdir)
+    self.FortranLib(conf,vars,dirs,libs,functions)
 
 
-  def Install(self,conf,vars,cmake,petsc,archdir):
-    externdir = os.path.join(archdir,'externalpackages')
+  def DownloadAndInstall(self,conf,vars,slepc,petsc,archdir,prefixdir):
+    externdir = slepc.CreateDir(archdir,'externalpackages')
     builddir  = os.path.join(externdir,self.dirname)
-    self.Download(externdir,builddir)
+    self.Download(externdir,builddir,slepc.downloaddir)
 
     # Check for autoreconf
-    result,output = commands.getstatusoutput('autoreconf --help')
+    result,output = self.RunCommand('autoreconf --help')
     if result:
-      self.log.Exit('ERROR: --download-arpack requires that the command autoreconf is available on your PATH.')
+      self.log.Exit('--download-arpack requires that the command autoreconf is available on your PATH')
 
     # Build package
-    confopt = '--prefix='+archdir+' F77="'+petsc.fc+'" FFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'"'
+    confopt = '--prefix='+prefixdir+' CC="'+petsc.cc+'" CFLAGS="'+petsc.cc_flags+'" F77="'+petsc.fc+'" FFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'" FC="'+petsc.fc+'" FCFLAGS="'+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'"'
     if not petsc.mpiuni:
-      confopt = confopt+' --enable-mpi'
+      confopt = confopt+' --enable-mpi MPICC="'+petsc.cc+'" MPIF77="'+petsc.fc+'" MPIFC="'+petsc.fc+'"'
     if not petsc.buildsharedlib:
       confopt = confopt+' --disable-shared'
-    result,output = commands.getstatusoutput('cd '+builddir+'&& sh bootstrap && ./configure '+confopt+' && '+petsc.make+' && '+petsc.make+' install')
+    if petsc.ind64:
+      if not petsc.blaslapackint64:
+        self.log.Exit('To install ARPACK with 64-bit integers you also need a BLAS with 64-bit integers')
+      confopt = confopt+' INTERFACE64=1'
+    result,output = self.RunCommand('cd '+builddir+'&& sh bootstrap && ./configure '+confopt+' && '+petsc.make+' && '+petsc.make+' install')
     self.log.write(output)
     if result:
-      self.log.Exit('ERROR: installation of ARPACK failed.')
+      self.log.Exit('Installation of ARPACK failed')
 
     # Check build
     functions = self.Functions(petsc)
@@ -97,8 +100,7 @@ class Arpack(package.Package):
       libs = [['-larpack']]
     else:
       libs = [['-lparpack','-larpack']]
-    libDir = os.path.join(archdir,'lib')
-    dirs = [libDir]
-    self.FortranLib(conf,vars,cmake,dirs,libs,functions)
-    self.havepackage = True
+    libdir = os.path.join(prefixdir,'lib')
+    dirs = [libdir]
+    self.FortranLib(conf,vars,dirs,libs,functions)
 
