@@ -1,22 +1,22 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#if !defined(_LMEIMPL)
-#define _LMEIMPL
+#if !defined(SLEPCLMEIMPL_H)
+#define SLEPCLMEIMPL_H
 
 #include <slepclme.h>
 #include <slepc/private/slepcimpl.h>
 
-PETSC_EXTERN PetscBool LMERegisterAllCalled;
-PETSC_EXTERN PetscErrorCode LMERegisterAll(void);
-PETSC_EXTERN PetscLogEvent LME_SetUp,LME_Solve,LME_ComputeError;
+SLEPC_EXTERN PetscBool LMERegisterAllCalled;
+SLEPC_EXTERN PetscErrorCode LMERegisterAll(void);
+SLEPC_EXTERN PetscLogEvent LME_SetUp,LME_Solve,LME_ComputeError;
 
 typedef struct _LMEOps *LMEOps;
 
@@ -69,26 +69,39 @@ struct _p_LME {
   LMEConvergedReason reason;
 };
 
-PETSC_INTERN PetscErrorCode LMERankSVD(LME,PetscInt,PetscScalar*,PetscScalar*,PetscInt*);
+SLEPC_INTERN PetscErrorCode LMEDenseRankSVD(LME,PetscInt,PetscScalar*,PetscInt,PetscScalar*,PetscInt,PetscInt*);
+
+/*
+    Macros to test valid LME arguments
+*/
+#if !defined(PETSC_USE_DEBUG)
+
+#define LMECheckCoeff(h,A,mat,eq) do {} while (0)
+
+#else
+
+#define LMECheckCoeff(h,A,mat,eq) \
+  do { \
+    if (!(A)) SETERRQ2(PetscObjectComm((PetscObject)(h)),PETSC_ERR_ARG_WRONGSTATE,"%s matrix equation requires coefficient matrix %s",eq,mat); \
+  } while (0)
+
+#endif
 
 /* functions interfaced from Fortran library SLICOT */
 #if defined(SLEPC_HAVE_SLICOT)
 
 #if defined(SLEPC_SLICOT_HAVE_UNDERSCORE)
 #define SLEPC_SLICOT(lcase,ucase) lcase##_
-#elif defined(SLEPC_SLICOT_HAVE_CAPS) || defined(PETSC_SLICOT_HAVE_STDCALL)
+#elif defined(SLEPC_SLICOT_HAVE_CAPS)
 #define SLEPC_SLICOT(lcase,ucase) ucase
 #else
 #define SLEPC_SLICOT(lcase,ucase) lcase
 #endif
 
-#if !defined(PETSC_BLASLAPACK_STDCALL)
 #define SLICOTsb03od_(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q) SLEPC_SLICOT(sb03od,SB03OD) ((a),(b),(c),(d),(e),(f),(g),(h),(i),(j),(k),(l),(m),(n),(o),(p),(q),1,1,1)
-PETSC_EXTERN void SLEPC_SLICOT(sb03od,SB03OD)(const char*,const char*,const char*,PetscBLASInt*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscReal*,PetscScalar*,PetscScalar*,PetscReal*,PetscBLASInt*,PetscBLASInt*,PetscBLASInt,PetscBLASInt,PetscBLASInt);
-#else
-#define SLICOTsb03od_(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q) SLEPC_SLICOT(sb03od,SB03OD) ((a),1,(b),1,(c),1,(d),(e),(f),(g),(h),(i),(j),(k),(l),(m),(n),(o),(p),(q))
-PETSC_EXTERN void PETSC_STDCALL SLEPC_SLICOT(sb03od,SB03OD)(const char*,PetscBLASInt,const char*,PetscBLASInt,const char*,PetscBLASInt,PetscBLASInt*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscReal*,PetscScalar*,PetscScalar*,PetscReal*,PetscBLASInt*,PetscBLASInt*);
-#endif
+SLEPC_EXTERN void SLEPC_SLICOT(sb03od,SB03OD)(const char*,const char*,const char*,PetscBLASInt*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscReal*,PetscScalar*,PetscScalar*,PetscReal*,PetscBLASInt*,PetscBLASInt*,PetscBLASInt,PetscBLASInt,PetscBLASInt);
+#define SLICOTsb03md_(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t) SLEPC_SLICOT(sb03md,SB03MD) ((a),(b),(c),(d),(e),(f),(g),(h),(i),(j),(k),(l),(m),(n),(o),(p),(q),(r),(s),(t),1,1,1,1)
+SLEPC_EXTERN void SLEPC_SLICOT(sb03md,SB03MD)(const char*,const char*,const char*,const char*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscScalar*,PetscBLASInt*,PetscReal*,PetscReal*,PetscReal*,PetscScalar*,PetscScalar*,PetscBLASInt*,PetscReal*,PetscBLASInt*,PetscBLASInt*,PetscBLASInt,PetscBLASInt,PetscBLASInt,PetscBLASInt);
 
 #endif
 
