@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -17,6 +17,7 @@
 #include <slepcbv.h>
 #include <slepcds.h>
 #include <slepcrg.h>
+#include <slepclme.h>
 #include <petscsnes.h>
 
 SLEPC_EXTERN PetscErrorCode EPSInitializePackage(void);
@@ -49,6 +50,7 @@ typedef const char* EPSType;
 #define EPSRQCG        "rqcg"
 #define EPSLOBPCG      "lobpcg"
 #define EPSCISS        "ciss"
+#define EPSLYAPII      "lyapii"
 #define EPSLAPACK      "lapack"
 #define EPSARPACK      "arpack"
 #define EPSBLZPACK     "blzpack"
@@ -190,7 +192,7 @@ SLEPC_EXTERN PetscErrorCode EPSSetFromOptions(EPS);
 SLEPC_EXTERN PetscErrorCode EPSSetUp(EPS);
 SLEPC_EXTERN PetscErrorCode EPSSolve(EPS);
 SLEPC_EXTERN PetscErrorCode EPSView(EPS,PetscViewer);
-PETSC_STATIC_INLINE PetscErrorCode EPSViewFromOptions(EPS eps,PetscObject obj,const char name[]) {return PetscObjectViewFromOptions((PetscObject)eps,obj,name);}
+SLEPC_EXTERN PetscErrorCode EPSViewFromOptions(EPS,PetscObject,const char[]);
 SLEPC_EXTERN PetscErrorCode EPSErrorView(EPS,EPSErrorType,PetscViewer);
 PETSC_DEPRECATED_FUNCTION("Use EPSErrorView()") PETSC_STATIC_INLINE PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer v) {return EPSErrorView(eps,EPS_ERROR_RELATIVE,v);}
 SLEPC_EXTERN PetscErrorCode EPSErrorViewFromOptions(EPS);
@@ -276,6 +278,7 @@ SLEPC_EXTERN PetscErrorCode EPSGetTrackAll(EPS,PetscBool*);
 
 SLEPC_EXTERN PetscErrorCode EPSSetDeflationSpace(EPS,PetscInt,Vec[]);
 SLEPC_EXTERN PetscErrorCode EPSSetInitialSpace(EPS,PetscInt,Vec[]);
+SLEPC_EXTERN PetscErrorCode EPSSetLeftInitialSpace(EPS,PetscInt,Vec[]);
 
 SLEPC_EXTERN PetscErrorCode EPSSetOptionsPrefix(EPS,const char*);
 SLEPC_EXTERN PetscErrorCode EPSAppendOptionsPrefix(EPS,const char*);
@@ -458,6 +461,11 @@ SLEPC_EXTERN PetscErrorCode EPSCISSSetUseST(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSCISSGetUseST(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSCISSGetKSPs(EPS,PetscInt*,KSP**);
 
+PETSC_EXTERN PetscErrorCode EPSLyapIISetLME(EPS,LME);
+PETSC_EXTERN PetscErrorCode EPSLyapIIGetLME(EPS,LME*);
+PETSC_EXTERN PetscErrorCode EPSLyapIISetRanks(EPS,PetscInt,PetscInt);
+PETSC_EXTERN PetscErrorCode EPSLyapIIGetRanks(EPS,PetscInt*,PetscInt*);
+
 SLEPC_EXTERN PetscErrorCode EPSBLOPEXSetBlockSize(EPS,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSBLOPEXGetBlockSize(EPS,PetscInt*);
 
