@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,8 +12,8 @@
    This file is included by slepcsys.h and should not be used directly.
 */
 
-#if !defined(__SLEPCMATH_H)
-#define __SLEPCMATH_H
+#if !defined(SLEPCMATH_H)
+#define SLEPCMATH_H
 
 /*
     Default tolerance for the different solvers, depending on the precision
