@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -34,6 +34,7 @@ typedef struct {
   PetscInt  bs;        /* block size */
   PetscBool lock;      /* soft locking active/inactive */
   PetscReal restart;   /* restart parameter */
+  PetscInt  guard;     /* number of guard vectors */
 } EPS_LOBPCG;
 
 PetscErrorCode EPSSetDimensions_LOBPCG(EPS eps,PetscInt nev,PetscInt *ncv,PetscInt *mpd)
@@ -72,6 +73,10 @@ PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
 
   if (!ctx->restart) ctx->restart = 0.9;
 
+  /* number of guard vectors */
+  if (ctx->bs==1) ctx->guard = 0;
+  else ctx->guard = PetscMin((PetscInt)((1.0-ctx->restart)*ctx->bs+0.45),ctx->bs-1);
+
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
   ierr = EPS_SetInnerProduct(eps);CHKERRQ(ierr);
   ierr = DSSetType(eps->ds,DSGHEP);CHKERRQ(ierr);
@@ -84,13 +89,13 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
-  PetscInt       i,j,k,ld,nv,ini,nmat,nc,nconv,locked,guard,its;
+  PetscInt       i,j,k,ld,nv,ini,nmat,nc,nconv,locked,its;
   PetscReal      norm;
   PetscScalar    *eigr,dot;
   PetscBool      breakdown,countc,flip=PETSC_FALSE,checkprecond=PETSC_FALSE;
   Mat            A,B,M;
   Vec            v,z,w=eps->work[0];
-  BV             X,Y,Z,R,P,AX,BX;
+  BV             X,Y=NULL,Z,R,P,AX,BX;
   SlepcSC        sc;
 
   PetscFunctionBegin;
@@ -100,8 +105,6 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
   if (nmat>1) { ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr); }
   else B = NULL;
 
-  guard = (PetscInt)((1.0-ctx->restart)*ctx->bs);  /* number of guard vectors */
-
   if (eps->which==EPS_LARGEST_REAL) {  /* flip spectrum */
     flip = PETSC_TRUE;
     ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
@@ -122,7 +125,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     ierr = BVDuplicateResize(eps->V,ctx->bs,&BX);CHKERRQ(ierr);
   }
   nc = eps->nds;
-  if (nc>0 || eps->nev>ctx->bs-guard) {
+  if (nc>0 || eps->nev>ctx->bs-ctx->guard) {
     ierr = BVDuplicateResize(eps->V,nc+eps->nev,&Y);CHKERRQ(ierr);
   }
   if (nc>0) {
@@ -214,19 +217,19 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       ierr = EPSMonitor(eps,eps->its+its,eps->nconv,eps->eigr,eps->eigi,eps->errest,locked+ctx->bs);CHKERRQ(ierr);
     }
     ierr = (*eps->stopping)(eps,eps->its+its,eps->max_it,eps->nconv,eps->nev,&eps->reason,eps->stoppingctx);CHKERRQ(ierr);
-    if (eps->reason != EPS_CONVERGED_ITERATING || nconv >= ctx->bs-guard) {
+    if (eps->reason != EPS_CONVERGED_ITERATING || nconv >= ctx->bs-ctx->guard) {
       ierr = BVSetActiveColumns(eps->V,locked,eps->nconv);CHKERRQ(ierr);
       ierr = BVSetActiveColumns(X,0,nconv);CHKERRQ(ierr);
       ierr = BVCopy(X,eps->V);CHKERRQ(ierr);
     }
     if (eps->reason != EPS_CONVERGED_ITERATING) {
       break;
-    } else if (nconv >= ctx->bs-guard) {
+    } else if (nconv >= ctx->bs-ctx->guard) {
       eps->its += its-1;
       its = 0;
     } else its++;
 
-    if (nconv >= ctx->bs-guard) {  /* force hard locking of vectors and compute new R */
+    if (nconv >= ctx->bs-ctx->guard) {  /* force hard locking of vectors and compute new R */
 
       /* extend constraints */
       ierr = BVSetActiveColumns(Y,nc+locked,nc+locked+nconv);CHKERRQ(ierr);
@@ -306,7 +309,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     /* 9. Apply preconditioner to the residuals */
     for (j=ini;j<ctx->bs;j++) {
       ierr = BVGetColumn(R,j,&v);CHKERRQ(ierr);
-      ierr = STMatSolve(eps->st,v,w);CHKERRQ(ierr);
+      ierr = STApply(eps->st,v,w);CHKERRQ(ierr);
       if (checkprecond) {
         ierr = VecDot(v,w,&dot);CHKERRQ(ierr);
         if (PetscRealPart(dot)<0.0) {
@@ -402,7 +405,7 @@ diverged:
   if (B) {
     ierr = BVDestroy(&BX);CHKERRQ(ierr);
   }
-  if (nc>0 || eps->nev>ctx->bs-guard) {
+  if (nc>0 || eps->nev>ctx->bs-ctx->guard) {
     ierr = BVDestroy(&Y);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -425,7 +428,7 @@ static PetscErrorCode EPSLOBPCGSetBlockSize_LOBPCG(EPS eps,PetscInt bs)
 /*@
    EPSLOBPCGSetBlockSize - Sets the block size of the LOBPCG method.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -479,7 +482,7 @@ PetscErrorCode EPSLOBPCGGetBlockSize(EPS eps,PetscInt *bs)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(bs,2);
+  PetscValidIntPointer(bs,2);
   ierr = PetscUseMethod(eps,"EPSLOBPCGGetBlockSize_C",(EPS,PetscInt*),(eps,bs));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -489,10 +492,11 @@ static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
 
   PetscFunctionBegin;
-  if (restart==PETSC_DEFAULT) ctx->restart = 0.6;
-  else {
-    if (restart<0.1 || restart>1.0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The restart argument %g must be in the range [0.1,1.0]",(double)restart);
+  if (restart==PETSC_DEFAULT) restart = 0.9;
+  if (restart<0.1 || restart>1.0) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The restart argument %g must be in the range [0.1,1.0]",(double)restart);
+  if (restart != ctx->restart) {
     ctx->restart = restart;
+    eps->state = EPS_STATE_INITIAL;
   }
   PetscFunctionReturn(0);
 }
@@ -503,7 +507,7 @@ static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
    current block iterate that must have converged in order to force a
    restart with hard locking.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -560,7 +564,7 @@ PetscErrorCode EPSLOBPCGGetRestart(EPS eps,PetscReal *restart)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(restart,2);
+  PetscValidRealPointer(restart,2);
   ierr = PetscUseMethod(eps,"EPSLOBPCGGetRestart_C",(EPS,PetscReal*),(eps,restart));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -578,7 +582,7 @@ static PetscErrorCode EPSLOBPCGSetLocking_LOBPCG(EPS eps,PetscBool lock)
    EPSLOBPCGSetLocking - Choose between locking and non-locking variants of
    the LOBPCG method.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - the eigenproblem solver context
@@ -637,7 +641,7 @@ PetscErrorCode EPSLOBPCGGetLocking(EPS eps,PetscBool *lock)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(lock,2);
+  PetscValidBoolPointer(lock,2);
   ierr = PetscUseMethod(eps,"EPSLOBPCGGetLocking_C",(EPS,PetscBool*),(eps,lock));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -652,7 +656,7 @@ PetscErrorCode EPSView_LOBPCG(EPS eps,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"  block size %D\n",ctx->bs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  restart parameter=%g (using %d guard vectors)\n",(double)ctx->restart,(int)((1.0-ctx->restart)*ctx->bs));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  restart parameter=%g (using %D guard vectors)\n",(double)ctx->restart,ctx->guard);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  soft locking %sactivated\n",ctx->lock?"":"de");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -696,7 +700,7 @@ PetscErrorCode EPSDestroy_LOBPCG(EPS eps)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
+SLEPC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
 {
   EPS_LOBPCG     *lobpcg;
   PetscErrorCode ierr;
