@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -14,9 +14,9 @@
 
 #include <slepc/private/bvimpl.h>
 #include <stdlib.h>
-#include <blopex_interpreter.h>
-#include <blopex_temp_multivector.h>
-#include "slepc-interface.h"
+#include <interpreter.h>
+#include <temp_multivector.h>
+#include "blopex.h"
 
 static void* mv_TempMultiVectorCreateFromBV(void* ii_,BlopexInt n,void* sample)
 {
@@ -56,17 +56,6 @@ static void* mv_TempMultiVectorCreateFromBV(void* ii_,BlopexInt n,void* sample)
   return x;
 }
 
-static void mv_TempMultiPETSCVectorDestroy(void* x_)
-{
-  mv_TempMultiVector* x = (mv_TempMultiVector*)x_;
-
-  if (!x) return;
-
-  if (x->ownsVectors && x->vector) free(x->vector);
-  if (x->mask && x->ownsMask) free(x->mask);
-  free(x);
-}
-
 /*
     Create an InterfaceInterpreter using the PETSc implementation
     but overloading CreateMultiVector that doesn't create any
@@ -80,12 +69,3 @@ int SLEPCSetupInterpreter(mv_InterfaceInterpreter *i)
   return 0;
 }
 
-/*
-    Change the multivector destructor in order to destroy the multivector
-    structure without destroy the PETSc vectors.
-*/
-void SLEPCSetupInterpreterForDignifiedDeath(mv_InterfaceInterpreter *i)
-{
-  i->DestroyMultiVector = mv_TempMultiPETSCVectorDestroy;
-}
-
