@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -41,7 +41,13 @@ static const char citation[] =
 
 #define SLICE_PTOL PETSC_SQRT_MACHINE_EPSILON
 
-static PetscErrorCode EPSSliceResetSR(EPS eps) {
+#define InertiaMismatch(h,d) \
+  do { \
+    SETERRQ1(PetscObjectComm((PetscObject)h),1,"Mismatch between number of values found and information from inertia%s",d?"":", consider using EPSKrylovSchurSetDetectZeros()"); \
+  } while (0)
+
+static PetscErrorCode EPSSliceResetSR(EPS eps)
+{
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
   EPS_SR          sr=ctx->sr;
@@ -314,7 +320,7 @@ static PetscErrorCode EPSSliceGetInertia(EPS eps,PetscReal shift,PetscInt *inert
   KSP            ksp;
   PC             pc;
   Mat            F;
-  PetscReal      nzshift;
+  PetscReal      nzshift=shift;
 
   PetscFunctionBegin;
   if (shift >= PETSC_MAX_REAL) { /* Right-open interval */
@@ -328,12 +334,12 @@ static PetscErrorCode EPSSliceGetInertia(EPS eps,PetscReal shift,PetscInt *inert
        the symbolic factorizations */
     nzshift = (shift==0.0)? 10.0/PETSC_MAX_REAL: shift;
     ierr = STSetShift(eps->st,nzshift);CHKERRQ(ierr);
-    ierr = STSetUp(eps->st);CHKERRQ(ierr);
     ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
     ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
     ierr = PCFactorGetMatrix(pc,&F);CHKERRQ(ierr);
     ierr = MatGetInertia(F,inertia,zeros,NULL);CHKERRQ(ierr);
   }
+  if (inertia) { ierr = PetscInfo2(eps,"Computed inertia at shift %g: %D\n",(double)nzshift,*inertia);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
@@ -352,13 +358,12 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
   PetscBool       issinv;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data,*ctx_glob;
   EPS_SR          sr,sr_loc,sr_glob;
-  PetscInt        nEigs,dssz=1,i,zeros=0,off=0;
+  PetscInt        nEigs,dssz=1,i,zeros=0,off=0,method,hiteig=0;
   PetscMPIInt     nproc,rank=0,aux;
   PetscReal       r;
   MPI_Request     req;
   Mat             A,B=NULL;
   SlepcSC         sc;
-  PetscInt        flg=0;
   DSParallelType  ptype;
 
   PetscFunctionBegin;
@@ -440,6 +445,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     } else {
       nEigs = sr_loc->numEigs;
       sr->inertia0 = sr_loc->inertia0;
+      sr->dir = sr_loc->dir;
     }
     sr->inertia1 = sr->inertia0+sr->dir*nEigs;
     sr->numEigs = nEigs;
@@ -456,15 +462,23 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
       if ((sr->dir>0&&ctx->subc->color==ctx->npart-1)||(sr->dir<0&&ctx->subc->color==0)) sr->hasEnd = sr_glob->hasEnd;
       else sr->hasEnd = PETSC_TRUE;
     }
+    ierr = STSetUp(eps->st);CHKERRQ(ierr);
 
     /* compute inertia0 */
     ierr = EPSSliceGetInertia(eps,sr->int0,&sr->inertia0,ctx->detect?&zeros:NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsGetInt(NULL,NULL,"-eps_krylovschur_hiteigenvalue",&flg,NULL);CHKERRQ(ierr);
+    /* undocumented option to control what to do when an eigenvalue is found:
+       - error out if it's the endpoint of the user-provided interval (or sub-interval)
+       - if it's an endpoint computed internally:
+          + if hiteig=0 error out
+          + else if hiteig=1 the subgroup that hit the eigenvalue does nothing
+          + otherwise the subgroup that hit the eigenvalue perturbs the shift and recomputes inertia
+    */
+    ierr = PetscOptionsGetInt(NULL,NULL,"-eps_krylovschur_hiteigenvalue",&hiteig,NULL);CHKERRQ(ierr);
     if (zeros) { /* error in factorization */
       if (sr->int0==ctx->eps->inta || sr->int0==ctx->eps->intb) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in the interval endpoint");
-      else if(ctx_glob->subintset && !flg) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in an interval endpoint defined by user");
+      else if (ctx_glob->subintset && !hiteig) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in an interval endpoint defined by user");
       else {
-        if (flg==1) { /* idle subgroup */
+        if (hiteig==1) { /* idle subgroup */
           sr->inertia0 = -1;
         } else { /* perturb shift */
           sr->int0 *= (1.0+SLICE_PTOL);
@@ -477,7 +491,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
       /* inertia1 is received from neighbour */
       ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
       if (!rank) {
-        if ( sr->inertia0!=-1 && ((sr->dir>0 && ctx->subc->color>0) || (sr->dir<0 && ctx->subc->color<ctx->npart-1)) ) { /* send inertia0 to neighbour0 */
+        if (sr->inertia0!=-1 && ((sr->dir>0 && ctx->subc->color>0) || (sr->dir<0 && ctx->subc->color<ctx->npart-1))) { /* send inertia0 to neighbour0 */
           ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
           ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
         }
@@ -485,7 +499,7 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
           ierr = MPI_Recv(&(sr->inertia1),1,MPIU_INT,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRQ(ierr);
           ierr = MPI_Recv(&(sr->int1),1,MPIU_REAL,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRQ(ierr);
         }
-        if ( sr->inertia0==-1 && !(sr->dir>0 && ctx->subc->color==ctx->npart-1) && !(sr->dir<0 && ctx->subc->color==0)) {
+        if (sr->inertia0==-1 && !(sr->dir>0 && ctx->subc->color==ctx->npart-1) && !(sr->dir<0 && ctx->subc->color==0)) {
           sr->inertia0 = sr->inertia1; sr->int0 = sr->int1;
           ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
           ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
@@ -529,6 +543,8 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     }
     ierr = DSGetParallel(ctx->eps->ds,&ptype);CHKERRQ(ierr);
     ierr = DSSetParallel(eps->ds,ptype);CHKERRQ(ierr);
+    ierr = DSGetMethod(ctx->eps->ds,&method);CHKERRQ(ierr);
+    ierr = DSSetMethod(eps->ds,method);CHKERRQ(ierr);
   }
   ierr = DSSetType(eps->ds,DSHEP);CHKERRQ(ierr);
   ierr = DSSetCompact(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
@@ -785,6 +801,10 @@ static PetscErrorCode EPSCreateShift(EPS eps,PetscReal val,EPS_shift neighb0,EPS
 
   PetscFunctionBegin;
   sr = ctx->sr;
+  if ((neighb0 && val==neighb0->value) || (neighb1 && val==neighb1->value)) {
+    sr->nPend++;
+    PetscFunctionReturn(0);
+  }
   ierr = PetscNewLog(eps,&s);CHKERRQ(ierr);
   s->value = val;
   s->neighb[0] = neighb0;
@@ -814,12 +834,12 @@ static PetscErrorCode EPSCreateShift(EPS eps,PetscReal val,EPS_shift neighb0,EPS
 /* Prepare for Rational Krylov update */
 static PetscErrorCode EPSPrepareRational(EPS eps)
 {
-  EPS_KRYLOVSCHUR  *ctx=(EPS_KRYLOVSCHUR*)eps->data;
-  PetscErrorCode   ierr;
-  PetscInt         dir,i,k,ld,nv;
-  PetscScalar      *A;
-  EPS_SR           sr = ctx->sr;
-  Vec              v;
+  EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
+  PetscErrorCode  ierr;
+  PetscInt        dir,i,k,ld,nv;
+  PetscScalar     *A;
+  EPS_SR          sr = ctx->sr;
+  Vec             v;
 
   PetscFunctionBegin;
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
@@ -839,7 +859,7 @@ static PetscErrorCode EPSPrepareRational(EPS eps)
   }
   /* Copy to DS */
   ierr = DSGetArray(eps->ds,DS_MAT_A,&A);CHKERRQ(ierr);
-  ierr = PetscMemzero(A,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(A,ld*ld);CHKERRQ(ierr);
   for (i=0;i<k;i++) {
     A[i*(1+ld)] = sr->S[i];
     A[k+i*ld] = sr->S[sr->nS+i];
@@ -858,22 +878,31 @@ static PetscErrorCode EPSPrepareRational(EPS eps)
 /* Provides next shift to be computed */
 static PetscErrorCode EPSExtractShift(EPS eps)
 {
-  PetscErrorCode   ierr;
-  PetscInt         iner,zeros=0;
-  EPS_KRYLOVSCHUR  *ctx=(EPS_KRYLOVSCHUR*)eps->data;
-  EPS_SR           sr;
-  PetscReal        newShift;
-  EPS_shift        sPres;
+  PetscErrorCode  ierr;
+  PetscInt        iner,zeros=0;
+  EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
+  EPS_SR          sr;
+  PetscReal       newShift,diam,ptol;
+  EPS_shift       sPres;
 
   PetscFunctionBegin;
   sr = ctx->sr;
   if (sr->nPend > 0) {
+    if (sr->sPres==sr->pending[sr->nPend-1]) {
+      eps->reason = EPS_CONVERGED_ITERATING;
+      eps->its = 0;
+      sr->nPend--;
+      sr->sPres->rep = PETSC_TRUE;
+      PetscFunctionReturn(0);
+    }
     sr->sPrev = sr->sPres;
     sr->sPres = sr->pending[--sr->nPend];
     sPres = sr->sPres;
     ierr = EPSSliceGetInertia(eps,sPres->value,&iner,ctx->detect?&zeros:NULL);CHKERRQ(ierr);
     if (zeros) {
-      newShift = sPres->value*(1.0+SLICE_PTOL);
+      diam = PetscMin(PetscAbsReal(sPres->neighb[0]->value-sPres->value),PetscAbsReal(sPres->neighb[1]->value-sPres->value));
+      ptol = PetscMin(SLICE_PTOL,diam/2);
+      newShift = sPres->value*(1.0+ptol);
       if (sr->dir*(sPres->neighb[0] && newShift-sPres->neighb[0]->value) < 0) newShift = (sPres->value+sPres->neighb[0]->value)/2;
       else if (sPres->neighb[1] && sr->dir*(sPres->neighb[1]->value-newShift) < 0) newShift = (sPres->value+sPres->neighb[1]->value)/2;
       ierr = EPSSliceGetInertia(eps,newShift,&iner,&zeros);CHKERRQ(ierr);
@@ -897,18 +926,15 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
-  PetscInt        i,k,l,ld,nv,*iwork,j;
-  Mat             U;
+  PetscInt        i,k,l,ld,nv,*iwork,j,count0,count1,iterCompl=0,n0,n1;
+  Mat             U,Op;
   PetscScalar     *Q,*A;
-  PetscReal       *a,*b,beta;
-  PetscBool       breakdown;
-  PetscInt        count0,count1;
-  PetscReal       lambda;
+  PetscReal       *a,*b,beta,lambda;
   EPS_shift       sPres;
-  PetscBool       complIterating;
-  PetscBool       sch0,sch1;
-  PetscInt        iterCompl=0,n0,n1;
+  PetscBool       breakdown,complIterating,sch0,sch1;
   EPS_SR          sr = ctx->sr;
+#define MSGLEN 100
+  char            messg[MSGLEN];
 
   PetscFunctionBegin;
   /* Spectrum slicing data */
@@ -918,7 +944,7 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   ierr = PetscMalloc1(2*ld,&iwork);CHKERRQ(ierr);
   count0=0;count1=0; /* Found on both sides */
-  if (sr->nS > 0 && (sPres->neighb[0] == sr->sPrev || sPres->neighb[1] == sr->sPrev)) {
+  if (!sPres->rep && sr->nS > 0 && (sPres->neighb[0] == sr->sPrev || sPres->neighb[1] == sr->sPrev)) {
     /* Rational Krylov */
     ierr = DSTranslateRKS(eps->ds,sr->sPrev->value-sPres->value);CHKERRQ(ierr);
     ierr = DSGetDimensions(eps->ds,NULL,NULL,NULL,&l,NULL);CHKERRQ(ierr);
@@ -939,7 +965,9 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
     nv = PetscMin(eps->nconv+eps->mpd,eps->ncv);
     ierr = DSGetArrayReal(eps->ds,DS_MAT_T,&a);CHKERRQ(ierr);
     b = a + ld;
-    ierr = EPSFullLanczos(eps,a,b,eps->nconv+l,&nv,&breakdown);CHKERRQ(ierr);
+    ierr = STGetOperator(eps->st,&Op);CHKERRQ(ierr);
+    ierr = BVMatLanczos(eps->V,Op,a,b,eps->nconv+l,&nv,&breakdown);CHKERRQ(ierr);
+    ierr = STRestoreOperator(eps->st,&Op);CHKERRQ(ierr);
     sr->nv = nv;
     beta = b[nv-1];
     ierr = DSRestoreArrayReal(eps->ds,DS_MAT_T,&a);CHKERRQ(ierr);
@@ -976,7 +1004,7 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
     ierr = DSSynchronize(eps->ds,eps->eigr,NULL);CHKERRQ(ierr);
 
     /* Residual */
-    ierr = EPSKrylovConvergence(eps,PETSC_TRUE,eps->nconv,nv-eps->nconv,beta,1.0,&k);CHKERRQ(ierr);
+    ierr = EPSKrylovConvergence(eps,PETSC_TRUE,eps->nconv,nv-eps->nconv,beta,0.0,1.0,&k);CHKERRQ(ierr);
     /* Checking values obtained for completing */
     for (i=0;i<k;i++) {
       sr->back[i]=eps->eigr[i];
@@ -1066,7 +1094,9 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
   }
   sPres->comp[0] = PetscNot(count0 < sPres->nsch[0]);
   sPres->comp[1] = PetscNot(count1 < sPres->nsch[1]);
-  if (count0 > sPres->nsch[0] || count1 > sPres->nsch[1]) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
+  ierr = PetscSNPrintf(messg,MSGLEN,"Lanczos: %D evals in [%g,%g]%s and %D evals in [%g,%g]%s\n",count0,(double)(sr->dir==1)?sPres->ext[0]:sPres->value,(double)(sr->dir==1)?sPres->value:sPres->ext[0],(sPres->comp[0])?"*":"",count1,(double)(sr->dir==1)?sPres->value:sPres->ext[1],(double)(sr->dir==1)?sPres->ext[1]:sPres->value,(sPres->comp[1])?"*":"");CHKERRQ(ierr);
+  ierr = PetscInfo(eps,messg);CHKERRQ(ierr);
+  if (count0 > sPres->nsch[0] || count1 > sPres->nsch[1]) InertiaMismatch(eps,ctx->detect);
   ierr = PetscFree(iwork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -1086,11 +1116,9 @@ static PetscErrorCode EPSGetNewShiftValue(EPS eps,PetscInt side,PetscReal *newS)
   sr = ctx->sr;
   sPres = sr->sPres;
   if (sPres->neighb[side]) {
-  /* Completing a previous interval */
-    if (!sPres->neighb[side]->neighb[side] && sPres->neighb[side]->nconv[side]==0) { /* One of the ends might be too far from eigenvalues */
-      if (side) *newS = (sPres->value + PetscRealPart(sr->eigr[sr->perm[sr->indexEig-1]]))/2;
-      else *newS = (sPres->value + PetscRealPart(sr->eigr[sr->perm[0]]))/2;
-    } else *newS=(sPres->value + sPres->neighb[side]->value)/2;
+    /* Completing a previous interval */
+    *newS = (sPres->value + sPres->neighb[side]->value)/2;
+    if (PetscAbsReal(sPres->value - *newS)/PetscAbsReal(sPres->value)<=100*PETSC_SQRT_MACHINE_EPSILON) *newS = sPres->value;
   } else { /* (Only for side=1). Creating a new interval. */
     if (sPres->neigs==0) {/* No value has been accepted*/
       if (sPres->neighb[0]) {
@@ -1154,8 +1182,8 @@ static PetscErrorCode EPSGetNewShiftValue(EPS eps,PetscInt side,PetscReal *newS)
 */
 static PetscErrorCode sortRealEigenvalues(PetscScalar *r,PetscInt *perm,PetscInt nr,PetscBool prev,PetscInt dir)
 {
-  PetscReal      re;
-  PetscInt       i,j,tmp;
+  PetscReal re;
+  PetscInt  i,j,tmp;
 
   PetscFunctionBegin;
   if (!prev) for (i=0;i<nr;i++) perm[i] = i;
@@ -1259,12 +1287,12 @@ static PetscErrorCode EPSLookForDeflation(EPS eps)
   /* The number of values on each side are found */
   if (sPres->neighb[0]) {
     sPres->nsch[0] = (sr->dir)*(sPres->inertia - sPres->neighb[0]->inertia)-count0;
-    if (sPres->nsch[0]<0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
+    if (sPres->nsch[0]<0) InertiaMismatch(eps,ctx->detect);
   } else sPres->nsch[0] = 0;
 
   if (sPres->neighb[1]) {
     sPres->nsch[1] = (sr->dir)*(sPres->neighb[1]->inertia - sPres->inertia) - count1;
-    if (sPres->nsch[1]<0) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Mismatch between number of values found and information from inertia, consider using EPSKrylovSchurSetDetectZeros()");
+    if (sPres->nsch[1]<0) InertiaMismatch(eps,ctx->detect);
   } else sPres->nsch[1] = (sr->dir)*(sr->inertia1 - sPres->inertia);
 
   /* Completing vector of indexes for deflation */
