@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -21,7 +21,7 @@
    References:
 
        [1] "Lanczos Methods in SLEPc", SLEPc Technical Report STR-5,
-           available at http://slepc.upv.es.
+           available at https://slepc.upv.es.
 */
 
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
@@ -50,6 +50,7 @@ PetscErrorCode EPSSetUp_Lanczos(EPS eps)
     case EPS_LARGEST_IMAGINARY:
     case EPS_SMALLEST_IMAGINARY:
     case EPS_TARGET_IMAGINARY:
+    case EPS_ALL:
       SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
     default: ; /* default case to remove warning */
   }
@@ -100,7 +101,7 @@ static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
 {
   PetscErrorCode ierr;
   PetscInt       i,j,m = *M;
-  Vec            vj,vj1;
+  Mat            Op;
   PetscBool      *which,lwhich[100];
   PetscScalar    *hwork,lhwork[100];
 
@@ -114,12 +115,9 @@ static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
   for (i=0;i<k;i++) which[i] = PETSC_TRUE;
 
   ierr = BVSetActiveColumns(eps->V,0,m);CHKERRQ(ierr);
+  ierr = STGetOperator(eps->st,&Op);CHKERRQ(ierr);
   for (j=k;j<m;j++) {
-    ierr = BVGetColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVGetColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = STApply(eps->st,vj,vj1);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
+    ierr = BVMatMultColumn(eps->V,Op,j);CHKERRQ(ierr);
     which[j] = PETSC_TRUE;
     if (j-2>=k) which[j-2] = PETSC_FALSE;
     ierr = BVOrthogonalizeSomeColumn(eps->V,j+1,which,hwork,beta+j,breakdown);CHKERRQ(ierr);
@@ -131,6 +129,7 @@ static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
       ierr = BVScaleColumn(eps->V,j+1,1/beta[j]);CHKERRQ(ierr);
     }
   }
+  ierr = STRestoreOperator(eps->st,&Op);CHKERRQ(ierr);
   if (m > 100) {
     ierr = PetscFree2(which,hwork);CHKERRQ(ierr);
   }
@@ -156,17 +155,13 @@ static PetscErrorCode EPSLocalLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
 */
 static PetscErrorCode DenseTridiagonal(PetscInt n_,PetscReal *D,PetscReal *E,PetscReal *w,PetscScalar *V)
 {
-#if defined(SLEPC_MISSING_LAPACK_STEVR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"STEVR - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscReal      abstol = 0.0,vl,vu,*work;
   PetscBLASInt   il,iu,m,*isuppz,n,lwork,*iwork,liwork,info;
   const char     *jobz;
 #if defined(PETSC_USE_COMPLEX)
   PetscInt       i,j;
-  PetscReal      *VV;
+  PetscReal      *VV=NULL;
 #endif
 
   PetscFunctionBegin;
@@ -198,7 +193,6 @@ static PetscErrorCode DenseTridiagonal(PetscInt n_,PetscReal *D,PetscReal *E,Pet
 #endif
   ierr = PetscFree3(isuppz,work,iwork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -209,7 +203,8 @@ static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *be
   PetscErrorCode ierr;
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS*)eps->data;
   PetscInt       i,j,m = *M,n,nritz=0,nritzo;
-  Vec            vj,vj1,av;
+  Vec            vj1,av;
+  Mat            Op;
   PetscReal      *d,*e,*ritz,norm;
   PetscScalar    *Y,*hwork;
   PetscBool      *which;
@@ -217,16 +212,13 @@ static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *be
   PetscFunctionBegin;
   ierr = PetscCalloc6(m+1,&d,m,&e,m,&ritz,m*m,&Y,m,&which,m,&hwork);CHKERRQ(ierr);
   for (i=0;i<k;i++) which[i] = PETSC_TRUE;
+  ierr = STGetOperator(eps->st,&Op);CHKERRQ(ierr);
 
   for (j=k;j<m;j++) {
     ierr = BVSetActiveColumns(eps->V,0,m);CHKERRQ(ierr);
 
     /* Lanczos step */
-    ierr = BVGetColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVGetColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = STApply(eps->st,vj,vj1);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
+    ierr = BVMatMultColumn(eps->V,Op,j);CHKERRQ(ierr);
     which[j] = PETSC_TRUE;
     if (j-2>=k) which[j-2] = PETSC_FALSE;
     ierr = BVOrthogonalizeSomeColumn(eps->V,j+1,which,hwork,&norm,breakdown);CHKERRQ(ierr);
@@ -279,6 +271,7 @@ static PetscErrorCode EPSSelectiveLanczos(EPS eps,PetscReal *alpha,PetscReal *be
     ierr = BVScaleColumn(eps->V,j+1,1.0/norm);CHKERRQ(ierr);
   }
 
+  ierr = STRestoreOperator(eps->st,&Op);CHKERRQ(ierr);
   ierr = PetscFree6(d,e,ritz,Y,which,hwork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -366,7 +359,7 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
   PetscErrorCode ierr;
   EPS_LANCZOS    *lanczos = (EPS_LANCZOS*)eps->data;
   PetscInt       i,j,m = *M;
-  Vec            vj,vj1;
+  Mat            Op;
   PetscReal      norm,*omega,lomega[100],*omega_old,lomega_old[100],eps1,delta,eta;
   PetscBool      *which,lwhich[100],*which2,lwhich2[100];
   PetscBool      reorth = PETSC_FALSE,force_reorth = PETSC_FALSE;
@@ -391,16 +384,13 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
     anorm = 1.0;
     estimate_anorm = PETSC_TRUE;
   }
-  for (i=0;i<m-k;i++) omega[i] = omega_old[i] = 0.0;
+  for (i=0;i<PetscMax(100,m);i++) omega[i] = omega_old[i] = 0.0;
   for (i=0;i<k;i++) which[i] = PETSC_TRUE;
 
   ierr = BVSetActiveColumns(eps->V,0,m);CHKERRQ(ierr);
+  ierr = STGetOperator(eps->st,&Op);CHKERRQ(ierr);
   for (j=k;j<m;j++) {
-    ierr = BVGetColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVGetColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = STApply(eps->st,vj,vj1);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
+    ierr = BVMatMultColumn(eps->V,Op,j);CHKERRQ(ierr);
     if (fro) {
       /* Lanczos step with full reorthogonalization */
       ierr = BVOrthogonalizeColumn(eps->V,j+1,hwork,&norm,breakdown);CHKERRQ(ierr);
@@ -462,6 +452,7 @@ static PetscErrorCode EPSPartialLanczos(EPS eps,PetscReal *alpha,PetscReal *beta
     ierr = BVScaleColumn(eps->V,j+1,1.0/norm);CHKERRQ(ierr);
   }
 
+  ierr = STRestoreOperator(eps->st,&Op);CHKERRQ(ierr);
   if (m>100) {
     ierr = PetscFree5(omega,omega_old,which,which2,hwork);CHKERRQ(ierr);
   }
@@ -493,6 +484,7 @@ static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
   PetscInt           i,n=*m;
   PetscReal          betam;
   BVOrthogRefineType orthog_ref;
+  Mat                Op;
 
   PetscFunctionBegin;
   switch (lanczos->reorthog) {
@@ -500,7 +492,9 @@ static PetscErrorCode EPSBasicLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,P
       ierr = EPSLocalLanczos(eps,alpha,beta,k,m,breakdown);CHKERRQ(ierr);
       break;
     case EPS_LANCZOS_REORTHOG_FULL:
-      ierr = EPSFullLanczos(eps,alpha,beta,k,m,breakdown);CHKERRQ(ierr);
+      ierr = STGetOperator(eps->st,&Op);CHKERRQ(ierr);
+      ierr = BVMatLanczos(eps->V,Op,alpha,beta,k,m,breakdown);CHKERRQ(ierr);
+      ierr = STRestoreOperator(eps->st,&Op);CHKERRQ(ierr);
       break;
     case EPS_LANCZOS_REORTHOG_SELECTIVE:
       ierr = EPSSelectiveLanczos(eps,alpha,beta,k,m,breakdown,anorm);CHKERRQ(ierr);
@@ -759,7 +753,7 @@ static PetscErrorCode EPSLanczosSetReorthog_Lanczos(EPS eps,EPSLanczosReorthogTy
    EPSLanczosSetReorthog - Sets the type of reorthogonalization used during the Lanczos
    iteration.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -856,7 +850,7 @@ PetscErrorCode EPSView_Lanczos(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode EPSCreate_Lanczos(EPS eps)
+SLEPC_EXTERN PetscErrorCode EPSCreate_Lanczos(EPS eps)
 {
   EPS_LANCZOS    *ctx;
   PetscErrorCode ierr;
