@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -19,7 +19,7 @@
    EPSMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps      - the eigensolver context
@@ -42,7 +42,7 @@ PetscErrorCode EPSMonitorSetFromOptions(EPS eps,const char name[],const char hel
   PetscViewerAndFormat *vf;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)eps),((PetscObject)eps)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)eps),((PetscObject)eps)->options,((PetscObject)eps)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -58,7 +58,7 @@ PetscErrorCode EPSMonitorSetFromOptions(EPS eps,const char name[],const char hel
    EPSConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps      - the eigensolver context
@@ -80,7 +80,7 @@ PetscErrorCode EPSConvMonitorSetFromOptions(EPS eps,const char name[],const char
   SlepcConvMonitor  ctx;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)eps),((PetscObject)eps)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)eps),((PetscObject)eps)->options,((PetscObject)eps)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = SlepcConvMonitorCreate(viewer,format,&ctx);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -94,7 +94,7 @@ PetscErrorCode EPSConvMonitorSetFromOptions(EPS eps,const char name[],const char
    This routine must be called before EPSSetUp() if the user is to be
    allowed to set the solver type.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 .  eps - the eigensolver context
@@ -108,7 +108,7 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
 {
   PetscErrorCode ierr;
   char           type[256];
-  PetscBool      set,flg,flg1,flg2,flg3,purif;
+  PetscBool      set,flg,flg1,flg2,flg3,bval;
   PetscReal      r,array[2]={0,0};
   PetscScalar    s;
   PetscInt       i,j,k;
@@ -228,8 +228,10 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
     }
 
     ierr = PetscOptionsBool("-eps_true_residual","Compute true residuals explicitly","EPSSetTrueResidual",eps->trueres,&eps->trueres,NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsBool("-eps_purify","Postprocess eigenvectors for purification","EPSSetPurify",eps->purify,&purif,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = EPSSetPurify(eps,purif);CHKERRQ(ierr); }
+    ierr = PetscOptionsBool("-eps_purify","Postprocess eigenvectors for purification","EPSSetPurify",eps->purify,&bval,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSSetPurify(eps,bval);CHKERRQ(ierr); }
+    ierr = PetscOptionsBool("-eps_two_sided","Use two-sided variant (to compute left eigenvectors)","EPSSetTwoSided",eps->twosided,&bval,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = EPSSetTwoSided(eps,bval);CHKERRQ(ierr); }
 
     /* -----------------------------------------------------------------------*/
     /*
@@ -322,7 +324,7 @@ PetscErrorCode EPSGetTolerances(EPS eps,PetscReal *tol,PetscInt *maxits)
    EPSSetTolerances - Sets the tolerance and maximum iteration count used
    by the EPS convergence tests.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigensolver context
@@ -395,7 +397,7 @@ PetscErrorCode EPSGetDimensions(EPS eps,PetscInt *nev,PetscInt *ncv,PetscInt *mp
    EPSSetDimensions - Sets the number of eigenvalues to compute
    and the dimension of the subspace.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigensolver context
@@ -458,7 +460,7 @@ PetscErrorCode EPSSetDimensions(EPS eps,PetscInt nev,PetscInt ncv,PetscInt mpd)
    EPSSetWhichEigenpairs - Specifies which portion of the spectrum is
    to be sought.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps   - eigensolver context obtained from EPSCreate()
@@ -576,7 +578,7 @@ PetscErrorCode EPSGetWhichEigenpairs(EPS eps,EPSWhich *which)
    EPSSetEigenvalueComparison - Specifies the eigenvalue comparison function
    when EPSSetWhichEigenpairs() is set to EPS_WHICH_USER.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - eigensolver context obtained from EPSCreate()
@@ -618,7 +620,7 @@ PetscErrorCode EPSSetEigenvalueComparison(EPS eps,PetscErrorCode (*func)(PetscSc
    eigenvalues according to an arbitrary selection criterion. This criterion
    can be based on a computation involving the current eigenvector approximation.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - eigensolver context obtained from EPSCreate()
@@ -674,7 +676,7 @@ PetscErrorCode EPSSetArbitrarySelection(EPS eps,PetscErrorCode (*func)(PetscScal
    EPSSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps     - eigensolver context obtained from EPSCreate()
@@ -726,7 +728,7 @@ PetscErrorCode EPSSetConvergenceTestFunction(EPS eps,PetscErrorCode (*func)(EPS,
    EPSSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - eigensolver context obtained from EPSCreate()
@@ -798,7 +800,7 @@ PetscErrorCode EPSGetConvergenceTest(EPS eps,EPSConv *conv)
    EPSSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the eigensolver.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps     - eigensolver context obtained from EPSCreate()
@@ -851,7 +853,7 @@ PetscErrorCode EPSSetStoppingTestFunction(EPS eps,PetscErrorCode (*func)(EPS,Pet
    EPSSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the eigensolver.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - eigensolver context obtained from EPSCreate()
@@ -916,7 +918,7 @@ PetscErrorCode EPSGetStoppingTest(EPS eps,EPSStop *stop)
 /*@
    EPSSetProblemType - Specifies the type of the eigenvalue problem.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps      - the eigensolver context
@@ -1020,7 +1022,7 @@ PetscErrorCode EPSGetProblemType(EPS eps,EPSProblemType *type)
    EPSSetExtraction - Specifies the type of extraction technique to be employed
    by the eigensolver.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - the eigensolver context
@@ -1086,7 +1088,7 @@ PetscErrorCode EPSGetExtraction(EPS eps,EPSExtraction *extr)
    EPSSetBalance - Specifies the balancing technique to be employed by the
    eigensolver, and some parameters associated to it.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps    - the eigensolver context
@@ -1188,11 +1190,73 @@ PetscErrorCode EPSGetBalance(EPS eps,EPSBalance *bal,PetscInt *its,PetscReal *cu
   PetscFunctionReturn(0);
 }
 
+/*@
+   EPSSetTwoSided - Sets the solver to use a two-sided variant so that left
+   eigenvectors are also computed.
+
+   Logically Collective on eps
+
+   Input Parameters:
++  eps      - the eigensolver context
+-  twosided - whether the two-sided variant is to be used or not
+
+   Options Database Keys:
+.  -eps_two_sided <boolean> - Sets/resets the twosided flag
+
+   Notes:
+   If the user sets twosided=PETSC_TRUE then the solver uses a variant of
+   the algorithm that computes both right and left eigenvectors. This is
+   usually much more costly. This option is not available in all solvers.
+
+   When using two-sided solvers, the problem matrices must have both the
+   MatMult and MatMultTranspose operations defined.
+
+   Level: advanced
+
+.seealso: EPSGetTwoSided(), EPSGetLeftEigenvector()
+@*/
+PetscErrorCode EPSSetTwoSided(EPS eps,PetscBool twosided)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidLogicalCollectiveBool(eps,twosided,2);
+  if (twosided!=eps->twosided) {
+    eps->twosided = twosided;
+    eps->state    = EPS_STATE_INITIAL;
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   EPSGetTwoSided - Returns the flag indicating whether a two-sided variant
+   of the algorithm is being used or not.
+
+   Not Collective
+
+   Input Parameter:
+.  eps - the eigensolver context
+
+   Output Parameter:
+.  twosided - the returned flag
+
+   Level: advanced
+
+.seealso: EPSSetTwoSided()
+@*/
+PetscErrorCode EPSGetTwoSided(EPS eps,PetscBool *twosided)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidBoolPointer(twosided,2);
+  *twosided = eps->twosided;
+  PetscFunctionReturn(0);
+}
+
 /*@
    EPSSetTrueResidual - Specifies if the solver must compute the true residual
    explicitly or not.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps     - the eigensolver context
@@ -1244,7 +1308,7 @@ PetscErrorCode EPSGetTrueResidual(EPS eps,PetscBool *trueres)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(trueres,2);
+  PetscValidBoolPointer(trueres,2);
   *trueres = eps->trueres;
   PetscFunctionReturn(0);
 }
@@ -1253,7 +1317,7 @@ PetscErrorCode EPSGetTrueResidual(EPS eps,PetscBool *trueres)
    EPSSetTrackAll - Specifies if the solver must compute the residual norm of all
    approximate eigenpairs or not.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps      - the eigensolver context
@@ -1301,7 +1365,7 @@ PetscErrorCode EPSGetTrackAll(EPS eps,PetscBool *trackall)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(trackall,2);
+  PetscValidBoolPointer(trackall,2);
   *trackall = eps->trackall;
   PetscFunctionReturn(0);
 }
@@ -1309,7 +1373,7 @@ PetscErrorCode EPSGetTrackAll(EPS eps,PetscBool *trackall)
 /*@
    EPSSetPurify - Deactivate eigenvector purification (which is activated by default).
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps    - the eigensolver context
@@ -1333,8 +1397,10 @@ PetscErrorCode EPSSetPurify(EPS eps,PetscBool purify)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveBool(eps,purify,2);
-  eps->purify = purify;
-  if (purify && !eps->purify) eps->state = EPS_STATE_INITIAL;
+  if (purify!=eps->purify) {
+    eps->purify = purify;
+    eps->state  = EPS_STATE_INITIAL;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -1358,7 +1424,7 @@ PetscErrorCode EPSGetPurify(EPS eps,PetscBool *purify)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(purify,2);
+  PetscValidBoolPointer(purify,2);
   *purify = eps->purify;
   PetscFunctionReturn(0);
 }
@@ -1367,7 +1433,7 @@ PetscErrorCode EPSGetPurify(EPS eps,PetscBool *purify)
    EPSSetOptionsPrefix - Sets the prefix used for searching for all
    EPS options in the database.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigensolver context
@@ -1411,7 +1477,7 @@ PetscErrorCode EPSSetOptionsPrefix(EPS eps,const char *prefix)
    EPSAppendOptionsPrefix - Appends to the prefix used for searching for all
    EPS options in the database.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps - the eigensolver context
