@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -40,39 +40,6 @@ PetscErrorCode MFNSetUp_Krylov(MFN mfn)
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode MFNBasicArnoldi(MFN mfn,PetscScalar *H,PetscInt ldh,PetscInt k,PetscInt *M,PetscReal *beta,PetscBool *breakdown)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *a;
-  PetscInt       j,nc,n,m = *M;
-  Vec            vj,vj1,buf;
-
-  PetscFunctionBegin;
-  ierr = BVSetActiveColumns(mfn->V,0,m);CHKERRQ(ierr);
-  for (j=k;j<m;j++) {
-    ierr = BVGetColumn(mfn->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVGetColumn(mfn->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = MatMult(mfn->A,vj,vj1);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(mfn->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(mfn->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = BVOrthonormalizeColumn(mfn->V,j+1,PETSC_FALSE,beta,breakdown);CHKERRQ(ierr);
-    if (*breakdown) {
-      *M = j+1;
-      break;
-    }
-  }
-  /* extract Hessenberg matrix from the BV object */
-  ierr = BVGetNumConstraints(mfn->V,&nc);CHKERRQ(ierr);
-  ierr = BVGetSizes(mfn->V,NULL,NULL,&n);CHKERRQ(ierr);
-  ierr = BVGetBufferVec(mfn->V,&buf);CHKERRQ(ierr);
-  ierr = VecGetArray(buf,&a);CHKERRQ(ierr);
-  for (j=k;j<*M;j++) {
-    ierr = PetscMemcpy(H+j*ldh,a+nc+(j+1)*(nc+n),(j+2)*sizeof(PetscScalar));CHKERRQ(ierr);
-  }
-  ierr = VecRestoreArray(buf,&a);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
 {
   PetscErrorCode ierr;
@@ -99,7 +66,7 @@ PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
     mfn->its++;
 
     /* compute Arnoldi factorization */
-    ierr = MFNBasicArnoldi(mfn,array,ld,0,&m,&beta,&breakdown);CHKERRQ(ierr);
+    ierr = BVMatArnoldi(mfn->V,mfn->transpose_solve?mfn->AT:mfn->A,array,ld,0,&m,&beta,&breakdown);CHKERRQ(ierr);
 
     /* save previous Hessenberg matrix in G; allocate new storage for H and f(H) */
     if (mfn->its>1) { G = H; H = NULL; }
@@ -110,12 +77,12 @@ PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
     /* glue together the previous H and the new H obtained with Arnoldi */
     ierr = MatDenseGetArray(H,&harray);CHKERRQ(ierr);
     for (j=0;j<m;j++) {
-      ierr = PetscMemcpy(harray+n+(j+n)*ldh,array+j*ld,m*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(harray+n+(j+n)*ldh,array+j*ld,m);CHKERRQ(ierr);
     }
     if (mfn->its>1) {
       ierr = MatDenseGetArray(G,&garray);CHKERRQ(ierr);
       for (j=0;j<n;j++) {
-        ierr = PetscMemcpy(harray+j*ldh,garray+j*n,n*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = PetscArraycpy(harray+j*ldh,garray+j*n,n);CHKERRQ(ierr);
       }
       ierr = MatDenseRestoreArray(G,&garray);CHKERRQ(ierr);
       ierr = MatDestroy(&G);CHKERRQ(ierr);
@@ -166,7 +133,7 @@ PetscErrorCode MFNSolve_Krylov(MFN mfn,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode MFNCreate_Krylov(MFN mfn)
+SLEPC_EXTERN PetscErrorCode MFNCreate_Krylov(MFN mfn)
 {
   PetscFunctionBegin;
   mfn->ops->solve          = MFNSolve_Krylov;
