@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -21,7 +21,7 @@ PetscLogEvent     MFN_SetUp = 0,MFN_Solve = 0;
 /*@C
    MFNView - Prints the MFN data structure.
 
-   Collective on MFN
+   Collective on mfn
 
    Input Parameters:
 +  mfn - the matrix function solver context
@@ -42,25 +42,22 @@ PetscLogEvent     MFN_SetUp = 0,MFN_Solve = 0;
    PetscViewerASCIIOpen() - output to a specified file.
 
    Level: beginner
-
-.seealso: PetscViewerASCIIOpen()
 @*/
 PetscErrorCode MFNView(MFN mfn,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscBool      isascii;
-  PetscInt       tabs;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)mfn));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)mfn),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(mfn,1,viewer,2);
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)mfn)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)mfn,viewer);CHKERRQ(ierr);
     if (mfn->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -70,7 +67,6 @@ PetscErrorCode MFNView(MFN mfn,PetscViewer viewer)
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %D\n",mfn->ncv);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %D\n",mfn->max_it);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",(double)mfn->tol);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else {
     if (mfn->ops->view) {
       ierr = (*mfn->ops->view)(mfn,viewer);CHKERRQ(ierr);
@@ -86,11 +82,34 @@ PetscErrorCode MFNView(MFN mfn,PetscViewer viewer)
 }
 
 /*@C
-   MFNReasonView - Displays the reason an MFN solve converged or diverged.
+   MFNViewFromOptions - View from options
 
    Collective on MFN
 
-   Parameter:
+   Input Parameters:
++  mfn  - the matrix function context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: MFNView(), MFNCreate()
+@*/
+PetscErrorCode MFNViewFromOptions(MFN mfn,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)mfn,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+/*@C
+   MFNReasonView - Displays the reason an MFN solve converged or diverged.
+
+   Collective on mfn
+
+   Input Parameters:
 +  mfn - the matrix function context
 -  viewer - the viewer to display the reason
 
@@ -124,9 +143,9 @@ PetscErrorCode MFNReasonView(MFN mfn,PetscViewer viewer)
    MFNReasonViewFromOptions - Processes command line options to determine if/how
    the MFN converged reason is to be viewed.
 
-   Collective on MFN
+   Collective on mfn
 
-   Input Parameters:
+   Input Parameter:
 .  mfn - the matrix function context
 
    Level: developer
@@ -142,7 +161,7 @@ PetscErrorCode MFNReasonViewFromOptions(MFN mfn)
   PetscFunctionBegin;
   if (incall) PetscFunctionReturn(0);
   incall = PETSC_TRUE;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)mfn),((PetscObject)mfn)->prefix,"-mfn_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)mfn),((PetscObject)mfn)->options,((PetscObject)mfn)->prefix,"-mfn_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = MFNReasonView(mfn,viewer);CHKERRQ(ierr);
@@ -156,7 +175,7 @@ PetscErrorCode MFNReasonViewFromOptions(MFN mfn)
 /*@
    MFNCreate - Creates the default MFN context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -209,7 +228,7 @@ PetscErrorCode MFNCreate(MPI_Comm comm,MFN *outmfn)
 /*@C
    MFNSetType - Selects the particular solver to be used in the MFN object.
 
-   Logically Collective on MFN
+   Logically Collective on mfn
 
    Input Parameters:
 +  mfn  - the matrix function context
@@ -314,6 +333,7 @@ PetscErrorCode MFNRegister(const char *name,PetscErrorCode (*function)(MFN))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MFNInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&MFNList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -322,7 +342,7 @@ PetscErrorCode MFNRegister(const char *name,PetscErrorCode (*function)(MFN))
    MFNReset - Resets the MFN context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on MFN
+   Collective on mfn
 
    Input Parameter:
 .  mfn - matrix function context obtained from MFNCreate()
@@ -350,7 +370,7 @@ PetscErrorCode MFNReset(MFN mfn)
 /*@
    MFNDestroy - Destroys the MFN context.
 
-   Collective on MFN
+   Collective on mfn
 
    Input Parameter:
 .  mfn - matrix function context obtained from MFNCreate()
@@ -370,6 +390,7 @@ PetscErrorCode MFNDestroy(MFN *mfn)
   ierr = MFNReset(*mfn);CHKERRQ(ierr);
   if ((*mfn)->ops->destroy) { ierr = (*(*mfn)->ops->destroy)(*mfn);CHKERRQ(ierr); }
   ierr = FNDestroy(&(*mfn)->fn);CHKERRQ(ierr);
+  ierr = MatDestroy(&(*mfn)->AT);CHKERRQ(ierr);
   ierr = MFNMonitorCancel(*mfn);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(mfn);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -378,7 +399,7 @@ PetscErrorCode MFNDestroy(MFN *mfn)
 /*@
    MFNSetBV - Associates a basis vectors object to the matrix function solver.
 
-   Collective on MFN
+   Collective on mfn
 
    Input Parameters:
 +  mfn - matrix function context obtained from MFNCreate()
@@ -432,7 +453,9 @@ PetscErrorCode MFNGetBV(MFN mfn,BV *bv)
   PetscValidPointer(bv,2);
   if (!mfn->V) {
     ierr = BVCreate(PetscObjectComm((PetscObject)mfn),&mfn->V);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)mfn->V,(PetscObject)mfn,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)mfn,(PetscObject)mfn->V);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)mfn->V,((PetscObject)mfn)->options);CHKERRQ(ierr);
   }
   *bv = mfn->V;
   PetscFunctionReturn(0);
@@ -441,7 +464,7 @@ PetscErrorCode MFNGetBV(MFN mfn,BV *bv)
 /*@
    MFNSetFN - Specifies the function to be computed.
 
-   Collective on MFN
+   Collective on mfn
 
    Input Parameters:
 +  mfn - matrix function context obtained from MFNCreate()
@@ -494,7 +517,9 @@ PetscErrorCode MFNGetFN(MFN mfn,FN *fn)
   PetscValidPointer(fn,2);
   if (!mfn->fn) {
     ierr = FNCreate(PetscObjectComm((PetscObject)mfn),&mfn->fn);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)mfn->fn,(PetscObject)mfn,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)mfn,(PetscObject)mfn->fn);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)mfn->fn,((PetscObject)mfn)->options);CHKERRQ(ierr);
   }
   *fn = mfn->fn;
   PetscFunctionReturn(0);
