@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -19,7 +19,7 @@
    NEPMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep      - the nonlinear eigensolver context
@@ -42,7 +42,7 @@ PetscErrorCode NEPMonitorSetFromOptions(NEP nep,const char name[],const char hel
   PetscViewerAndFormat *vf;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -58,7 +58,7 @@ PetscErrorCode NEPMonitorSetFromOptions(NEP nep,const char name[],const char hel
    NEPConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep      - the nonlinear eigensolver context
@@ -80,7 +80,7 @@ PetscErrorCode NEPConvMonitorSetFromOptions(NEP nep,const char name[],const char
   SlepcConvMonitor  ctx;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)nep),((PetscObject)nep)->options,((PetscObject)nep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = SlepcConvMonitorCreate(viewer,format,&ctx);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -94,7 +94,7 @@ PetscErrorCode NEPConvMonitorSetFromOptions(NEP nep,const char name[],const char
    This routine must be called before NEPSetUp() if the user is to be
    allowed to set the solver type.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 .  nep - the nonlinear eigensolver context
@@ -108,7 +108,7 @@ PetscErrorCode NEPSetFromOptions(NEP nep)
 {
   PetscErrorCode  ierr;
   char            type[256];
-  PetscBool       set,flg,flg1,flg2,flg3,flg4,flg5;
+  PetscBool       set,flg,flg1,flg2,flg3,flg4,flg5,bval;
   PetscReal       r;
   PetscScalar     s;
   PetscInt        i,j,k;
@@ -203,6 +203,9 @@ PetscErrorCode NEPSetFromOptions(NEP nep)
       ierr = NEPSetTarget(nep,s);CHKERRQ(ierr);
     }
 
+    ierr = PetscOptionsBool("-nep_two_sided","Use two-sided variant (to compute left eigenvectors)","NEPSetTwoSided",nep->twosided,&bval,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = NEPSetTwoSided(nep,bval);CHKERRQ(ierr); }
+
     /* -----------------------------------------------------------------------*/
     /*
       Cancels all monitors hardwired into code before call to NEPSetFromOptions()
@@ -250,8 +253,10 @@ PetscErrorCode NEPSetFromOptions(NEP nep)
   ierr = BVSetFromOptions(nep->V);CHKERRQ(ierr);
   if (!nep->rg) { ierr = NEPGetRG(nep,&nep->rg);CHKERRQ(ierr); }
   ierr = RGSetFromOptions(nep->rg);CHKERRQ(ierr);
-  if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
-  ierr = DSSetFromOptions(nep->ds);CHKERRQ(ierr);
+  if (nep->useds) {
+    if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
+    ierr = DSSetFromOptions(nep->ds);CHKERRQ(ierr);
+  }
   if (!nep->refineksp) { ierr = NEPRefineGetKSP(nep,&nep->refineksp);CHKERRQ(ierr); }
   ierr = KSPSetFromOptions(nep->refineksp);CHKERRQ(ierr);
   if (nep->fui==NEP_USER_INTERFACE_SPLIT) for (i=0;i<nep->nt;i++) {ierr = FNSetFromOptions(nep->f[i]);CHKERRQ(ierr);}
@@ -291,7 +296,7 @@ PetscErrorCode NEPGetTolerances(NEP nep,PetscReal *tol,PetscInt *maxits)
    NEPSetTolerances - Sets the tolerance and maximum iteration count used
    by the NEP convergence tests.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep    - the nonlinear eigensolver context
@@ -367,7 +372,7 @@ PetscErrorCode NEPGetDimensions(NEP nep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
    NEPSetDimensions - Sets the number of eigenvalues to compute
    and the dimension of the subspace.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -426,7 +431,7 @@ PetscErrorCode NEPSetDimensions(NEP nep,PetscInt nev,PetscInt ncv,PetscInt mpd)
     NEPSetWhichEigenpairs - Specifies which portion of the spectrum is
     to be sought.
 
-    Logically Collective on NEP
+    Logically Collective on nep
 
     Input Parameters:
 +   nep   - eigensolver context obtained from NEPCreate()
@@ -491,7 +496,7 @@ PetscErrorCode NEPSetWhichEigenpairs(NEP nep,NEPWhich which)
 #if defined(PETSC_USE_COMPLEX)
     case NEP_TARGET_IMAGINARY:
 #endif
-    case EPS_ALL:
+    case NEP_ALL:
     case NEP_WHICH_USER:
       if (nep->which != which) {
         nep->state = NEP_STATE_INITIAL;
@@ -536,7 +541,7 @@ PetscErrorCode NEPGetWhichEigenpairs(NEP nep,NEPWhich *which)
    NEPSetEigenvalueComparison - Specifies the eigenvalue comparison function
    when NEPSetWhichEigenpairs() is set to NEP_WHICH_USER.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep  - eigensolver context obtained from NEPCreate()
@@ -576,7 +581,7 @@ PetscErrorCode NEPSetEigenvalueComparison(NEP nep,PetscErrorCode (*func)(PetscSc
 /*@
    NEPSetProblemType - Specifies the type of the nonlinear eigenvalue problem.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep  - the nonlinear eigensolver context
@@ -635,11 +640,73 @@ PetscErrorCode NEPGetProblemType(NEP nep,NEPProblemType *type)
   PetscFunctionReturn(0);
 }
 
+/*@
+   NEPSetTwoSided - Sets the solver to use a two-sided variant so that left
+   eigenvectors are also computed.
+
+   Logically Collective on nep
+
+   Input Parameters:
++  nep      - the eigensolver context
+-  twosided - whether the two-sided variant is to be used or not
+
+   Options Database Keys:
+.  -nep_two_sided <boolean> - Sets/resets the twosided flag
+
+   Notes:
+   If the user sets twosided=PETSC_TRUE then the solver uses a variant of
+   the algorithm that computes both right and left eigenvectors. This is
+   usually much more costly. This option is not available in all solvers.
+
+   When using two-sided solvers, the problem matrices must have both the
+   MatMult and MatMultTranspose operations defined.
+
+   Level: advanced
+
+.seealso: NEPGetTwoSided(), NEPGetLeftEigenvector()
+@*/
+PetscErrorCode NEPSetTwoSided(NEP nep,PetscBool twosided)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveBool(nep,twosided,2);
+  if (twosided!=nep->twosided) {
+    nep->twosided = twosided;
+    nep->state    = NEP_STATE_INITIAL;
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   NEPGetTwoSided - Returns the flag indicating whether a two-sided variant
+   of the algorithm is being used or not.
+
+   Not Collective
+
+   Input Parameter:
+.  nep - the eigensolver context
+
+   Output Parameter:
+.  twosided - the returned flag
+
+   Level: advanced
+
+.seealso: NEPSetTwoSided()
+@*/
+PetscErrorCode NEPGetTwoSided(NEP nep,PetscBool *twosided)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidBoolPointer(twosided,2);
+  *twosided = nep->twosided;
+  PetscFunctionReturn(0);
+}
+
 /*@C
    NEPSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep     - nonlinear eigensolver context obtained from NEPCreate()
@@ -691,7 +758,7 @@ PetscErrorCode NEPSetConvergenceTestFunction(NEP nep,PetscErrorCode (*func)(NEP,
    NEPSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep  - nonlinear eigensolver context obtained from NEPCreate()
@@ -762,7 +829,7 @@ PetscErrorCode NEPGetConvergenceTest(NEP nep,NEPConv *conv)
    NEPSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the eigensolver.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep     - nonlinear eigensolver context obtained from NEPCreate()
@@ -815,7 +882,7 @@ PetscErrorCode NEPSetStoppingTestFunction(NEP nep,PetscErrorCode (*func)(NEP,Pet
    NEPSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the eigensolver.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep  - nonlinear eigensolver context obtained from NEPCreate()
@@ -881,7 +948,7 @@ PetscErrorCode NEPGetStoppingTest(NEP nep,NEPStop *stop)
    NEPSetTrackAll - Specifies if the solver must compute the residual of all
    approximate eigenpairs or not.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep      - the eigensolver context
@@ -928,7 +995,7 @@ PetscErrorCode NEPGetTrackAll(NEP nep,PetscBool *trackall)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(trackall,2);
+  PetscValidBoolPointer(trackall,2);
   *trackall = nep->trackall;
   PetscFunctionReturn(0);
 }
@@ -937,7 +1004,7 @@ PetscErrorCode NEPGetTrackAll(NEP nep,PetscBool *trackall)
    NEPSetRefine - Specifies the refinement type (and options) to be used
    after the solve.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep    - the nonlinear eigensolver context
@@ -1063,7 +1130,7 @@ PetscErrorCode NEPGetRefine(NEP nep,NEPRefine *refine,PetscInt *npart,PetscReal
    NEPSetOptionsPrefix - Sets the prefix used for searching for all
    NEP options in the database.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -1105,7 +1172,7 @@ PetscErrorCode NEPSetOptionsPrefix(NEP nep,const char *prefix)
    NEPAppendOptionsPrefix - Appends to the prefix used for searching for all
    NEP options in the database.
 
-   Logically Collective on NEP
+   Logically Collective on nep
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
