@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -17,7 +17,7 @@
    NEPSetUp - Sets up all the internal data structures necessary for the
    execution of the NEP solver.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameter:
 .  nep   - solver context
@@ -56,11 +56,12 @@ PetscErrorCode NEPSetUp(NEP nep)
   if (!((PetscObject)nep)->type_name) {
     ierr = NEPSetType(nep,NEPRII);CHKERRQ(ierr);
   }
-  if (!nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
+  if (nep->useds && !nep->ds) { ierr = NEPGetDS(nep,&nep->ds);CHKERRQ(ierr); }
   if (!nep->rg) { ierr = NEPGetRG(nep,&nep->rg);CHKERRQ(ierr); }
   if (!((PetscObject)nep->rg)->type_name) {
     ierr = RGSetType(nep->rg,RGINTERVAL);CHKERRQ(ierr);
   }
+  if (nep->twosided && !nep->hasts) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"This solver does not support computing left eigenvectors (no two-sided variant)");
 
   /* set problem dimensions */
   switch (nep->fui) {
@@ -77,11 +78,6 @@ PetscErrorCode NEPSetUp(NEP nep)
     ierr = MatGetSize(nep->A[0],&nep->n,NULL);CHKERRQ(ierr);
     ierr = MatGetLocalSize(nep->A[0],&nep->nloc,NULL);CHKERRQ(ierr);
     break;
-  case NEP_USER_INTERFACE_DERIVATIVES:
-    ierr = NEPGetDerivatives(nep,&T,NULL,NULL);CHKERRQ(ierr);
-    ierr = MatGetSize(T,&nep->n,NULL);CHKERRQ(ierr);
-    ierr = MatGetLocalSize(T,&nep->nloc,NULL);CHKERRQ(ierr);
-    break;
   }
 
   /* set default problem type */
@@ -186,15 +182,16 @@ PetscErrorCode NEPSetUp(NEP nep)
   nep->sc->mapobj = NULL;
 
   /* fill sorting criterion for DS */
-  ierr = DSGetSlepcSC(nep->ds,&sc);CHKERRQ(ierr);
-  sc->comparison    = nep->sc->comparison;
-  sc->comparisonctx = nep->sc->comparisonctx;
-  ierr = PetscObjectTypeCompare((PetscObject)nep,NEPNLEIGS,&flg);CHKERRQ(ierr);
-  if (!flg) {
-    sc->map    = NULL;
-    sc->mapobj = NULL;
+  if (nep->useds) {
+    ierr = DSGetSlepcSC(nep->ds,&sc);CHKERRQ(ierr);
+    sc->comparison    = nep->sc->comparison;
+    sc->comparisonctx = nep->sc->comparisonctx;
+    ierr = PetscObjectTypeCompare((PetscObject)nep,NEPNLEIGS,&flg);CHKERRQ(ierr);
+    if (!flg) {
+      sc->map    = NULL;
+      sc->mapobj = NULL;
+    }
   }
-  if (nep->ncv > nep->n) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"ncv must be the problem size at most");
   if (nep->nev > nep->ncv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"nev bigger than ncv");
 
   /* process initial vectors */
@@ -214,7 +211,7 @@ PetscErrorCode NEPSetUp(NEP nep)
    NEPSetInitialSpace - Specify a basis of vectors that constitute the initial
    space, that is, the subspace from which the solver starts to iterate.
 
-   Collective on NEP and Vec
+   Collective on nep
 
    Input Parameter:
 +  nep   - the nonlinear eigensolver context
@@ -236,7 +233,7 @@ PetscErrorCode NEPSetUp(NEP nep)
 
    Level: intermediate
 @*/
-PetscErrorCode NEPSetInitialSpace(NEP nep,PetscInt n,Vec *is)
+PetscErrorCode NEPSetInitialSpace(NEP nep,PetscInt n,Vec is[])
 {
   PetscErrorCode ierr;
 
@@ -244,6 +241,10 @@ PetscErrorCode NEPSetInitialSpace(NEP nep,PetscInt n,Vec *is)
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,n,2);
   if (n<0) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  if (n>0) {
+    PetscValidPointer(is,3);
+    PetscValidHeaderSpecific(*is,VEC_CLASSID,3);
+  }
   ierr = SlepcBasisReference_Private(n,is,&nep->nini,&nep->IS);CHKERRQ(ierr);
   if (n>0) nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
@@ -275,7 +276,7 @@ PetscErrorCode NEPSetDimensions_Default(NEP nep,PetscInt nev,PetscInt *ncv,Petsc
    NEPAllocateSolution - Allocate memory storage for common variables such
    as eigenvalues and eigenvectors.
 
-   Collective on NEP
+   Collective on nep
 
    Input Parameters:
 +  nep   - eigensolver context
@@ -283,7 +284,7 @@ PetscErrorCode NEPSetDimensions_Default(NEP nep,PetscInt nev,PetscInt *ncv,Petsc
            working basis slightly larger than ncv
 
    Developers Note:
-   This is PETSC_EXTERN because it may be required by user plugin NEP
+   This is SLEPC_EXTERN because it may be required by user plugin NEP
    implementations.
 
    Level: developer
@@ -327,6 +328,12 @@ PetscErrorCode NEPAllocateSolution(NEP nep,PetscInt extra)
   } else {
     ierr = BVResize(nep->V,requested,PETSC_FALSE);CHKERRQ(ierr);
   }
+
+  /* allocate W */
+  if (nep->twosided) {
+    ierr = BVDestroy(&nep->W);CHKERRQ(ierr);
+    ierr = BVDuplicate(nep->V,&nep->W);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
