@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -50,6 +50,7 @@ PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
     if (sinv) pep->which = PEP_TARGET_MAGNITUDE;
     else pep->which = PEP_LARGEST_MAGNITUDE;
   }
+  if (pep->which==PEP_ALL) SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of pep->which");
 
   if (pep->nmat!=3) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver only available for quadratic problems");
   if (pep->basis!=PEP_BASIS_MONOMIAL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver not implemented for non-monomial bases");
@@ -91,7 +92,7 @@ PetscErrorCode PEPExtractVectors_QArnoldi(PEP pep)
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&X0);CHKERRQ(ierr);
   ierr = MatDenseGetArray(X0,&pX0);CHKERRQ(ierr);
   for (i=0;i<k;i++) {
-    ierr = PetscMemcpy(pX0+i*k,X+i*ldds,k*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(pX0+i*k,X+i*ldds,k);CHKERRQ(ierr);
   }
   ierr = MatDenseRestoreArray(X0,&pX0);CHKERRQ(ierr);
   ierr = BVMultInPlace(pep->V,X0,0,k);CHKERRQ(ierr);
@@ -253,9 +254,9 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
 
   /* clean projected matrix (including the extra-arrow) */
   ierr = DSGetArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-  ierr = PetscMemzero(S,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(S,ld*ld);CHKERRQ(ierr);
   ierr = DSRestoreArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-  
+
    /* Restart loop */
   l = 0;
   while (pep->reason == PEP_CONVERGED_ITERATING) {
@@ -310,7 +311,7 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
     pep->nconv = k;
     ierr = PEPMonitor(pep,pep->its,nconv,pep->eigr,pep->eigi,pep->errest,nv);CHKERRQ(ierr);
   }
-
+  ierr = BVSetActiveColumns(pep->V,0,pep->nconv);CHKERRQ(ierr);
   for (j=0;j<pep->nconv;j++) {
     pep->eigr[j] *= pep->sfactor;
     pep->eigi[j] *= pep->sfactor;
@@ -345,7 +346,7 @@ static PetscErrorCode PEPQArnoldiSetRestart_QArnoldi(PEP pep,PetscReal keep)
    method, in particular the proportion of basis vectors that must be kept
    after restart.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - the eigenproblem solver context
@@ -402,7 +403,7 @@ PetscErrorCode PEPQArnoldiGetRestart(PEP pep,PetscReal *keep)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(keep,2);
+  PetscValidRealPointer(keep,2);
   ierr = PetscUseMethod(pep,"PEPQArnoldiGetRestart_C",(PEP,PetscReal*),(pep,keep));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -420,7 +421,7 @@ static PetscErrorCode PEPQArnoldiSetLocking_QArnoldi(PEP pep,PetscBool lock)
    PEPQArnoldiSetLocking - Choose between locking and non-locking variants of
    the Q-Arnoldi method.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - the eigenproblem solver context
@@ -482,7 +483,7 @@ PetscErrorCode PEPQArnoldiGetLocking(PEP pep,PetscBool *lock)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(lock,2);
+  PetscValidBoolPointer(lock,2);
   ierr = PetscUseMethod(pep,"PEPQArnoldiGetLocking_C",(PEP,PetscBool*),(pep,lock));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -534,7 +535,7 @@ PetscErrorCode PEPDestroy_QArnoldi(PEP pep)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
+SLEPC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
 {
   PEP_QARNOLDI   *ctx;
   PetscErrorCode ierr;
