@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -19,7 +19,7 @@
    PEPMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep      - the polynomial eigensolver context
@@ -42,7 +42,7 @@ PetscErrorCode PEPMonitorSetFromOptions(PEP pep,const char name[],const char hel
   PetscViewerAndFormat *vf;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -58,7 +58,7 @@ PetscErrorCode PEPMonitorSetFromOptions(PEP pep,const char name[],const char hel
    PEPConvMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type
    indicated by the user (for monitors that only show iteration numbers of convergence).
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 +  pep      - the polynomial eigensolver context
@@ -80,7 +80,7 @@ PetscErrorCode PEPConvMonitorSetFromOptions(PEP pep,const char name[],const char
   SlepcConvMonitor  ctx;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)pep),((PetscObject)pep)->options,((PetscObject)pep)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = SlepcConvMonitorCreate(viewer,format,&ctx);CHKERRQ(ierr);
     ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
@@ -94,7 +94,7 @@ PetscErrorCode PEPConvMonitorSetFromOptions(PEP pep,const char name[],const char
    This routine must be called before PEPSetUp() if the user is to be
    allowed to set the solver type.
 
-   Collective on PEP
+   Collective on pep
 
    Input Parameters:
 .  pep - the polynomial eigensolver context
@@ -318,7 +318,7 @@ PetscErrorCode PEPGetTolerances(PEP pep,PetscReal *tol,PetscInt *maxits)
    PEPSetTolerances - Sets the tolerance and maximum iteration count used
    by the PEP convergence tests.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep - the polynomial eigensolver context
@@ -394,7 +394,7 @@ PetscErrorCode PEPGetDimensions(PEP pep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
    PEPSetDimensions - Sets the number of eigenvalues to compute
    and the dimension of the subspace.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep - the polynomial eigensolver context
@@ -456,7 +456,7 @@ PetscErrorCode PEPSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
    PEPSetWhichEigenpairs - Specifies which portion of the spectrum is
    to be sought.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep   - eigensolver context obtained from PEPCreate()
@@ -572,7 +572,7 @@ PetscErrorCode PEPGetWhichEigenpairs(PEP pep,PEPWhich *which)
    PEPSetEigenvalueComparison - Specifies the eigenvalue comparison function
    when PEPSetWhichEigenpairs() is set to PEP_WHICH_USER.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - eigensolver context obtained from PEPCreate()
@@ -612,7 +612,7 @@ PetscErrorCode PEPSetEigenvalueComparison(PEP pep,PetscErrorCode (*func)(PetscSc
 /*@
    PEPSetProblemType - Specifies the type of the polynomial eigenvalue problem.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - the polynomial eigensolver context
@@ -682,7 +682,7 @@ PetscErrorCode PEPGetProblemType(PEP pep,PEPProblemType *type)
    PEPSetBasis - Specifies the type of polynomial basis used to describe the
    polynomial eigenvalue problem.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep   - the polynomial eigensolver context
@@ -739,7 +739,7 @@ PetscErrorCode PEPGetBasis(PEP pep,PEPBasis *basis)
    PEPSetTrackAll - Specifies if the solver must compute the residual of all
    approximate eigenpairs or not.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep      - the eigensolver context
@@ -786,7 +786,7 @@ PetscErrorCode PEPGetTrackAll(PEP pep,PetscBool *trackall)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(trackall,2);
+  PetscValidBoolPointer(trackall,2);
   *trackall = pep->trackall;
   PetscFunctionReturn(0);
 }
@@ -795,7 +795,7 @@ PetscErrorCode PEPGetTrackAll(PEP pep,PetscBool *trackall)
    PEPSetConvergenceTestFunction - Sets a function to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep     - eigensolver context obtained from PEPCreate()
@@ -847,7 +847,7 @@ PetscErrorCode PEPSetConvergenceTestFunction(PEP pep,PetscErrorCode (*func)(PEP,
    PEPSetConvergenceTest - Specifies how to compute the error estimate
    used in the convergence test.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - eigensolver context obtained from PEPCreate()
@@ -919,7 +919,7 @@ PetscErrorCode PEPGetConvergenceTest(PEP pep,PEPConv *conv)
    PEPSetStoppingTestFunction - Sets a function to decide when to stop the outer
    iteration of the eigensolver.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep     - eigensolver context obtained from PEPCreate()
@@ -972,7 +972,7 @@ PetscErrorCode PEPSetStoppingTestFunction(PEP pep,PetscErrorCode (*func)(PEP,Pet
    PEPSetStoppingTest - Specifies how to decide the termination of the outer
    loop of the eigensolver.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep  - eigensolver context obtained from PEPCreate()
@@ -1034,10 +1034,10 @@ PetscErrorCode PEPGetStoppingTest(PEP pep,PEPStop *stop)
   PetscFunctionReturn(0);
 }
 
-/*@C
+/*@
    PEPSetScale - Specifies the scaling strategy to be used.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep    - the eigensolver context
@@ -1060,7 +1060,7 @@ PetscErrorCode PEPGetStoppingTest(PEP pep,PEPStop *stop)
    In the scalar strategy, scaling is applied to the eigenvalue, that is,
    mu = lambda/alpha is the new eigenvalue and all matrices are scaled
    accordingly. After solving the scaled problem, the original lambda is
-   recovered. Parameter 'alpha' must be positive. Use PETSC_DECIDE to let
+   recovered. Parameter 'alpha' must be positive. Use PETSC_DEFAULT to let
    the solver compute a reasonable scaling factor.
 
    In the diagonal strategy, the solver works implicitly with matrix Dl*A*Dr,
@@ -1070,7 +1070,7 @@ PetscErrorCode PEPGetStoppingTest(PEP pep,PEPStop *stop)
    provided, these matrices are computed internally. This option requires
    that the polynomial coefficient matrices are of MATAIJ type.
    The parameter 'its' is the number of iterations performed by the method.
-   Parameter 'lambda' must be positive. Use PETSC_DECIDE or set lambda = 1.0 if
+   Parameter 'lambda' must be positive. Use PETSC_DEFAULT or set lambda = 1.0 if
    no information about eigenvalues is available.
 
    Level: intermediate
@@ -1166,7 +1166,7 @@ PetscErrorCode PEPGetScale(PEP pep,PEPScale *scale,PetscReal *alpha,Vec *Dl,Vec
 /*@
    PEPSetExtract - Specifies the extraction strategy to be used.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep     - the eigensolver context
@@ -1215,7 +1215,7 @@ PetscErrorCode PEPGetExtract(PEP pep,PEPExtract *extract)
    PEPSetRefine - Specifies the refinement type (and options) to be used
    after the solve.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep    - the polynomial eigensolver context
@@ -1341,7 +1341,7 @@ PetscErrorCode PEPGetRefine(PEP pep,PEPRefine *refine,PetscInt *npart,PetscReal
    PEPSetOptionsPrefix - Sets the prefix used for searching for all
    PEP options in the database.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep - the polynomial eigensolver context
@@ -1385,7 +1385,7 @@ PetscErrorCode PEPSetOptionsPrefix(PEP pep,const char *prefix)
    PEPAppendOptionsPrefix - Appends to the prefix used for searching for all
    PEP options in the database.
 
-   Logically Collective on PEP
+   Logically Collective on pep
 
    Input Parameters:
 +  pep - the polynomial eigensolver context
