@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -15,7 +15,7 @@
 
 #include <slepc/private/svdimpl.h>                /*I "slepcsvd.h" I*/
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
-#include "./cyclicimpl.h"
+#include "cyclic.h"
 
 static PetscErrorCode MatMult_Cyclic(Mat B,Vec x,Vec y)
 {
@@ -67,7 +67,7 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
   Vec               v;
   Mat               Zm,Zn;
   ST                st;
-#if defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_CUDA)
   PetscBool         cuda;
 #endif
 
@@ -110,7 +110,7 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
       ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cyclic->y2);CHKERRQ(ierr);
       ierr = MatCreateShell(PetscObjectComm((PetscObject)svd),m+n,m+n,M+N,M+N,svd,&cyclic->mat);CHKERRQ(ierr);
       ierr = MatShellSetOperation(cyclic->mat,MATOP_GET_DIAGONAL,(void(*)(void))MatGetDiagonal_Cyclic);CHKERRQ(ierr);
-#if defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_CUDA)
       ierr = PetscObjectTypeCompareAny((PetscObject)(svd->A?svd->A:svd->AT),&cuda,MATSEQAIJCUSPARSE,MATMPIAIJCUSPARSE,"");CHKERRQ(ierr);
       if (cuda) {
         ierr = MatShellSetOperation(cyclic->mat,MATOP_CREATE_VECS,(void(*)(void))MatCreateVecs_Cyclic_CUDA);CHKERRQ(ierr);
@@ -164,19 +164,19 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
         ierr = VecGetSize(svd->ISL[i],&isl);CHKERRQ(ierr);
         if (isl!=m) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Size mismatch for left initial vector");
         ierr = VecGetArrayRead(svd->ISL[i],&isa);CHKERRQ(ierr);
-        ierr = PetscMemcpy(va,isa,sizeof(PetscScalar)*m);CHKERRQ(ierr);
+        ierr = PetscArraycpy(va,isa,m);CHKERRQ(ierr);
         ierr = VecRestoreArrayRead(svd->IS[i],&isa);CHKERRQ(ierr);
       } else {
-        ierr = PetscMemzero(&va,sizeof(PetscScalar)*m);CHKERRQ(ierr);
+        ierr = PetscArrayzero(&va,m);CHKERRQ(ierr);
       }
       if (i<-svd->nini) {
         ierr = VecGetSize(svd->IS[i],&isl);CHKERRQ(ierr);
         if (isl!=n) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Size mismatch for right initial vector");
         ierr = VecGetArrayRead(svd->IS[i],&isa);CHKERRQ(ierr);
-        ierr = PetscMemcpy(va+m,isa,sizeof(PetscScalar)*n);CHKERRQ(ierr);
+        ierr = PetscArraycpy(va+m,isa,n);CHKERRQ(ierr);
         ierr = VecRestoreArrayRead(svd->IS[i],&isa);CHKERRQ(ierr);
       } else {
-        ierr = PetscMemzero(va+m,sizeof(PetscScalar)*n);CHKERRQ(ierr);
+        ierr = PetscArrayzero(va+m,n);CHKERRQ(ierr);
       }
       ierr = VecRestoreArray(v,&va);CHKERRQ(ierr);
       ierr = VecDestroy(&svd->IS[i]);CHKERRQ(ierr);
@@ -307,7 +307,7 @@ static PetscErrorCode SVDCyclicSetExplicitMatrix_Cyclic(SVD svd,PetscBool explic
    SVDCyclicSetExplicitMatrix - Indicate if the eigensolver operator
    H(A) = [ 0  A ; A^T 0 ] must be computed explicitly.
 
-   Logically Collective on SVD
+   Logically Collective on svd
 
    Input Parameters:
 +  svd      - singular value solver
@@ -361,7 +361,7 @@ PetscErrorCode SVDCyclicGetExplicitMatrix(SVD svd,PetscBool *explicitmatrix)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidPointer(explicitmatrix,2);
+  PetscValidBoolPointer(explicitmatrix,2);
   ierr = PetscUseMethod(svd,"SVDCyclicGetExplicitMatrix_C",(SVD,PetscBool*),(svd,explicitmatrix));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -385,7 +385,7 @@ static PetscErrorCode SVDCyclicSetEPS_Cyclic(SVD svd,EPS eps)
    SVDCyclicSetEPS - Associate an eigensolver object (EPS) to the
    singular value solver.
 
-   Collective on SVD
+   Collective on svd
 
    Input Parameters:
 +  svd - singular value solver
@@ -415,10 +415,11 @@ static PetscErrorCode SVDCyclicGetEPS_Cyclic(SVD svd,EPS *eps)
   PetscFunctionBegin;
   if (!cyclic->eps) {
     ierr = EPSCreate(PetscObjectComm((PetscObject)svd),&cyclic->eps);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)cyclic->eps,(PetscObject)svd,1);CHKERRQ(ierr);
     ierr = EPSSetOptionsPrefix(cyclic->eps,((PetscObject)svd)->prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(cyclic->eps,"svd_cyclic_");CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)cyclic->eps,(PetscObject)svd,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cyclic->eps);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)cyclic->eps,((PetscObject)svd)->options);CHKERRQ(ierr);
     ierr = EPSSetWhichEigenpairs(cyclic->eps,EPS_LARGEST_REAL);CHKERRQ(ierr);
     ierr = EPSMonitorSet(cyclic->eps,EPSMonitor_Cyclic,svd,NULL);CHKERRQ(ierr);
   }
@@ -501,7 +502,7 @@ PetscErrorCode SVDDestroy_Cyclic(SVD svd)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode SVDCreate_Cyclic(SVD svd)
+SLEPC_EXTERN PetscErrorCode SVDCreate_Cyclic(SVD svd)
 {
   PetscErrorCode ierr;
   SVD_CYCLIC     *cyclic;
