@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,7 +12,7 @@
 */
 
 #include <slepc/private/bvimpl.h>
-#include "./svecimpl.h"
+#include "svec.h"
 
 PetscErrorCode BVMult_Svec(BV Y,PetscScalar alpha,PetscScalar beta,BV X,Mat Q)
 {
@@ -206,32 +206,31 @@ PetscErrorCode BVNorm_Local_Svec(BV bv,PetscInt j,NormType type,PetscReal *val)
 PetscErrorCode BVMatMult_Svec(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
-  BV_SVEC        *v = (BV_SVEC*)V->data,*w = (BV_SVEC*)W->data;
-  PetscScalar    *pv,*pw;
   PetscInt       j;
   PetscBool      flg;
   Mat            Vmat,Wmat;
+  Vec            vv,ww;
 
   PetscFunctionBegin;
   ierr = MatHasOperation(A,MATOP_MAT_MULT,&flg);CHKERRQ(ierr);
   if (V->vmm && flg) {
     ierr = BVGetMat(V,&Vmat);CHKERRQ(ierr);
     ierr = BVGetMat(W,&Wmat);CHKERRQ(ierr);
-    ierr = MatMatMult(A,Vmat,MAT_REUSE_MATRIX,PETSC_DEFAULT,&Wmat);CHKERRQ(ierr);
+    ierr = MatProductCreateWithMat(A,Vmat,NULL,Wmat);CHKERRQ(ierr);
+    ierr = MatProductSetType(Wmat,MATPRODUCT_AB);CHKERRQ(ierr);
+    ierr = MatProductSetFromOptions(Wmat);CHKERRQ(ierr);
+    ierr = MatProductSymbolic(Wmat);CHKERRQ(ierr);
+    ierr = MatProductNumeric(Wmat);CHKERRQ(ierr);
     ierr = BVRestoreMat(V,&Vmat);CHKERRQ(ierr);
     ierr = BVRestoreMat(W,&Wmat);CHKERRQ(ierr);
   } else {
-    ierr = VecGetArray(v->v,&pv);CHKERRQ(ierr);
-    ierr = VecGetArray(w->v,&pw);CHKERRQ(ierr);
     for (j=0;j<V->k-V->l;j++) {
-      ierr = VecPlaceArray(V->cv[1],pv+(V->nc+V->l+j)*V->n);CHKERRQ(ierr);
-      ierr = VecPlaceArray(W->cv[1],pw+(W->nc+W->l+j)*W->n);CHKERRQ(ierr);
-      ierr = MatMult(A,V->cv[1],W->cv[1]);CHKERRQ(ierr);
-      ierr = VecResetArray(V->cv[1]);CHKERRQ(ierr);
-      ierr = VecResetArray(W->cv[1]);CHKERRQ(ierr);
+      ierr = BVGetColumn(V,V->l+j,&vv);CHKERRQ(ierr);
+      ierr = BVGetColumn(W,W->l+j,&ww);CHKERRQ(ierr);
+      ierr = MatMult(A,vv,ww);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(V,V->l+j,&vv);CHKERRQ(ierr);
+      ierr = BVRestoreColumn(W,W->l+j,&ww);CHKERRQ(ierr);
     }
-    ierr = VecRestoreArray(v->v,&pv);CHKERRQ(ierr);
-    ierr = VecRestoreArray(w->v,&pw);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -247,7 +246,7 @@ PetscErrorCode BVCopy_Svec(BV V,BV W)
   ierr = VecGetArray(w->v,&pw);CHKERRQ(ierr);
   pvc = pv+(V->nc+V->l)*V->n;
   pwc = pw+(W->nc+W->l)*W->n;
-  ierr = PetscMemcpy(pwc,pvc,(V->k-V->l)*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(pwc,pvc,(V->k-V->l)*V->n);CHKERRQ(ierr);
   ierr = VecRestoreArray(v->v,&pv);CHKERRQ(ierr);
   ierr = VecRestoreArray(w->v,&pw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -261,7 +260,7 @@ PetscErrorCode BVCopyColumn_Svec(BV V,PetscInt j,PetscInt i)
 
   PetscFunctionBegin;
   ierr = VecGetArray(v->v,&pv);CHKERRQ(ierr);
-  ierr = PetscMemcpy(pv+(V->nc+i)*V->n,pv+(V->nc+j)*V->n,V->n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(pv+(V->nc+i)*V->n,pv+(V->nc+j)*V->n,V->n);CHKERRQ(ierr);
   ierr = VecRestoreArray(v->v,&pv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -300,7 +299,7 @@ PetscErrorCode BVResize_Svec(BV bv,PetscInt m,PetscBool copy)
     if (copy) {
       ierr = VecGetArrayRead(ctx->v,&pv);CHKERRQ(ierr);
       ierr = VecGetArray(vnew,&pnew);CHKERRQ(ierr);
-      ierr = PetscMemcpy(pnew,pv,PetscMin(m,bv->m)*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(pnew,pv,PetscMin(m,bv->m)*bv->n);CHKERRQ(ierr);
       ierr = VecRestoreArrayRead(ctx->v,&pv);CHKERRQ(ierr);
       ierr = VecRestoreArray(vnew,&pnew);CHKERRQ(ierr);
     }
@@ -419,18 +418,18 @@ PetscErrorCode BVDestroy_Svec(BV bv)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
+SLEPC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
 {
   PetscErrorCode    ierr;
   BV_SVEC           *ctx;
-  PetscInt          nloc,N,bs,tglobal,tlocal,lsplit;
+  PetscInt          nloc,N,bs,tglobal=0,tlocal,lsplit;
   PetscBool         seq;
   PetscScalar       *aa,*vv;
   const PetscScalar *array,*ptr;
   char              str[50];
   BV                parent;
   Vec               vpar;
-#if defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_CUDA)
   PetscScalar       *gpuarray,*gptr;
 #endif
 
@@ -448,8 +447,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
   ierr = VecGetSize(bv->t,&N);CHKERRQ(ierr);
   ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
   tlocal  = bv->m*nloc;
-  tglobal = bv->m*N;
-  if (tglobal<0) SETERRQ2(PetscObjectComm((PetscObject)bv),1,"The product %D times %D overflows the size of PetscInt; consider reducing the number of columns, or use BVVECS instead",bv->m,N);
+  ierr = PetscIntMultError(bv->m,N,&tglobal);CHKERRQ(ierr);
 
   if (bv->issplit) {
     /* split BV: create Vec sharing the memory of the parent BV */
@@ -457,10 +455,10 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
     lsplit = parent->lsplit;
     vpar = ((BV_SVEC*)parent->data)->v;
     if (bv->cuda) {
-#if defined(PETSC_HAVE_VECCUDA)
-      ierr = VecCUDAGetArrayReadWrite(vpar,&gpuarray);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_CUDA)
+      ierr = VecCUDAGetArray(vpar,&gpuarray);CHKERRQ(ierr);
       gptr = (bv->issplit==1)? gpuarray: gpuarray+lsplit*nloc;
-      ierr = VecCUDARestoreArrayReadWrite(vpar,&gpuarray);CHKERRQ(ierr);
+      ierr = VecCUDARestoreArray(vpar,&gpuarray);CHKERRQ(ierr);
       if (ctx->mpi) {
         ierr = VecCreateMPICUDAWithArray(PetscObjectComm((PetscObject)bv->t),bs,tlocal,bv->m*N,NULL,&ctx->v);CHKERRQ(ierr);
       } else {
@@ -495,7 +493,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
   if (bv->Acreate) {
     ierr = MatDenseGetArray(bv->Acreate,&aa);CHKERRQ(ierr);
     ierr = VecGetArray(ctx->v,&vv);CHKERRQ(ierr);
-    ierr = PetscMemcpy(vv,aa,tlocal*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(vv,aa,tlocal);CHKERRQ(ierr);
     ierr = VecRestoreArray(ctx->v,&vv);CHKERRQ(ierr);
     ierr = MatDenseRestoreArray(bv->Acreate,&aa);CHKERRQ(ierr);
     ierr = MatDestroy(&bv->Acreate);CHKERRQ(ierr);
@@ -505,7 +503,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
   ierr = VecDuplicateEmpty(bv->t,&bv->cv[1]);CHKERRQ(ierr);
 
   if (bv->cuda) {
-#if defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_CUDA)
     bv->ops->mult             = BVMult_Svec_CUDA;
     bv->ops->multvec          = BVMultVec_Svec_CUDA;
     bv->ops->multinplace      = BVMultInPlace_Svec_CUDA;
