@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -95,13 +95,13 @@ PetscErrorCode BVMultInPlace_BLAS_Private(BV bv,PetscInt m_,PetscInt k_,PetscInt
   if (l) {
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N",bt,&l,&n,&k,&one,A,&m,pb,&ldb,&zero,bv->work,&l));
     for (j=0;j<n;j++) {
-      ierr = PetscMemcpy(A+(s+j)*m,bv->work+j*l,l*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(A+(s+j)*m,bv->work+j*l,l);CHKERRQ(ierr);
     }
   }
   for (;l<m;l+=bs) {
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N",bt,&bs,&n,&k,&one,A+l,&m,pb,&ldb,&zero,bv->work,&bs));
     for (j=0;j<n;j++) {
-      ierr = PetscMemcpy(A+(s+j)*m+l,bv->work+j*bs,bs*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(A+(s+j)*m+l,bv->work+j*bs,bs);CHKERRQ(ierr);
     }
   }
   ierr = PetscLogFlops(2.0*m*n*k);CHKERRQ(ierr);
@@ -134,26 +134,26 @@ PetscErrorCode BVMultInPlace_Vecs_Private(BV bv,PetscInt m_,PetscInt n_,PetscInt
   if (l) {
     for (j=0;j<n;j++) {
       ierr = VecGetArrayRead(V[j],&pin);CHKERRQ(ierr);
-      ierr = PetscMemcpy(bv->work+j*l,pin,l*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(bv->work+j*l,pin,l);CHKERRQ(ierr);
       ierr = VecRestoreArrayRead(V[j],&pin);CHKERRQ(ierr);
     }
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N",bt,&l,&n,&n,&one,bv->work,&l,(PetscScalar*)B,&k,&zero,out,&l));
     for (j=0;j<n;j++) {
       ierr = VecGetArray(V[j],&pout);CHKERRQ(ierr);
-      ierr = PetscMemcpy(pout,out+j*l,l*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(pout,out+j*l,l);CHKERRQ(ierr);
       ierr = VecRestoreArray(V[j],&pout);CHKERRQ(ierr);
     }
   }
   for (;l<m;l+=bs) {
     for (j=0;j<n;j++) {
       ierr = VecGetArrayRead(V[j],&pin);CHKERRQ(ierr);
-      ierr = PetscMemcpy(bv->work+j*bs,pin+l,bs*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(bv->work+j*bs,pin+l,bs);CHKERRQ(ierr);
       ierr = VecRestoreArrayRead(V[j],&pin);CHKERRQ(ierr);
     }
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N",bt,&bs,&n,&n,&one,bv->work,&bs,(PetscScalar*)B,&k,&zero,out,&bs));
     for (j=0;j<n;j++) {
       ierr = VecGetArray(V[j],&pout);CHKERRQ(ierr);
-      ierr = PetscMemcpy(pout+l,out+j*bs,bs*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(pout+l,out+j*bs,bs);CHKERRQ(ierr);
       ierr = VecRestoreArray(V[j],&pout);CHKERRQ(ierr);
     }
   }
@@ -203,18 +203,18 @@ PetscErrorCode BVDot_BLAS_Private(BV bv,PetscInt m_,PetscInt n_,PetscInt k_,Pets
     if (ldc==m) {
       ierr = BVAllocateWork_Private(bv,m*n);CHKERRQ(ierr);
       if (k) PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&m,&n,&k,&one,(PetscScalar*)A,&k,(PetscScalar*)B,&k,&zero,bv->work,&ldc));
-      else { ierr = PetscMemzero(bv->work,m*n*sizeof(PetscScalar));CHKERRQ(ierr); }
+      else { ierr = PetscArrayzero(bv->work,m*n);CHKERRQ(ierr); }
       ierr = PetscMPIIntCast(m*n,&len);CHKERRQ(ierr);
       ierr = MPI_Allreduce(bv->work,C,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
     } else {
       ierr = BVAllocateWork_Private(bv,2*m*n);CHKERRQ(ierr);
       CC = bv->work+m*n;
       if (k) PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&m,&n,&k,&one,(PetscScalar*)A,&k,(PetscScalar*)B,&k,&zero,bv->work,&m));
-      else { ierr = PetscMemzero(bv->work,m*n*sizeof(PetscScalar));CHKERRQ(ierr); }
+      else { ierr = PetscArrayzero(bv->work,m*n);CHKERRQ(ierr); }
       ierr = PetscMPIIntCast(m*n,&len);CHKERRQ(ierr);
       ierr = MPI_Allreduce(bv->work,CC,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
       for (j=0;j<n;j++) {
-        ierr = PetscMemcpy(C+j*ldc,CC+j*m,m*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = PetscArraycpy(C+j*ldc,CC+j*m,m);CHKERRQ(ierr);
       }
     }
   } else {
@@ -244,7 +244,7 @@ PetscErrorCode BVDotVec_BLAS_Private(BV bv,PetscInt n_,PetscInt k_,const PetscSc
     if (n) {
       PetscStackCallBLAS("BLASgemv",BLASgemv_("C",&n,&k,&done,A,&n,x,&one,&zero,bv->work,&one));
     } else {
-      ierr = PetscMemzero(bv->work,k*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArrayzero(bv->work,k);CHKERRQ(ierr);
     }
     ierr = PetscMPIIntCast(k,&len);CHKERRQ(ierr);
     ierr = MPI_Allreduce(bv->work,y,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
@@ -265,7 +265,7 @@ PetscErrorCode BVScale_BLAS_Private(BV bv,PetscInt n_,PetscScalar *A,PetscScalar
 
   PetscFunctionBegin;
   if (alpha == (PetscScalar)0.0) {
-    ierr = PetscMemzero(A,n_*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArrayzero(A,n_);CHKERRQ(ierr);
   } else if (alpha!=(PetscScalar)1.0) {
     ierr = PetscBLASIntCast(n_,&n);CHKERRQ(ierr);
     PetscStackCallBLAS("BLASscal",BLASscal_(&n,&alpha,A,&one));
@@ -274,150 +274,3 @@ PetscErrorCode BVScale_BLAS_Private(BV bv,PetscInt n_,PetscScalar *A,PetscScalar
   PetscFunctionReturn(0);
 }
 
-/*
-    Compute ||A|| for an mxn matrix
-*/
-PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscScalar *A,NormType type,PetscReal *nrm,PetscBool mpi)
-{
-  PetscErrorCode ierr;
-  PetscBLASInt   m,n,i,j;
-  PetscMPIInt    len;
-  PetscReal      lnrm,*rwork=NULL,*rwork2=NULL;
-
-  PetscFunctionBegin;
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m_,&m);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(n_,&n);CHKERRQ(ierr);
-  if (type==NORM_FROBENIUS || type==NORM_2) {
-    lnrm = LAPACKlange_("F",&m,&n,(PetscScalar*)A,&m,rwork);
-    if (mpi) {
-      lnrm = lnrm*lnrm;
-      ierr = MPI_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-      *nrm = PetscSqrtReal(*nrm);
-    } else *nrm = lnrm;
-    ierr = PetscLogFlops(2.0*m*n);CHKERRQ(ierr);
-  } else if (type==NORM_1) {
-    if (mpi) {
-      ierr = BVAllocateWork_Private(bv,2*n_);CHKERRQ(ierr);
-      rwork = (PetscReal*)bv->work;
-      rwork2 = rwork+n_;
-      ierr = PetscMemzero(rwork,n_*sizeof(PetscReal));CHKERRQ(ierr);
-      ierr = PetscMemzero(rwork2,n_*sizeof(PetscReal));CHKERRQ(ierr);
-      for (j=0;j<n_;j++) {
-        for (i=0;i<m_;i++) {
-          rwork[j] += PetscAbsScalar(A[i+j*m_]);
-        }
-      }
-      ierr = PetscMPIIntCast(n_,&len);CHKERRQ(ierr);
-      ierr = MPI_Allreduce(rwork,rwork2,len,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-      *nrm = 0.0;
-      for (j=0;j<n_;j++) if (rwork2[j] > *nrm) *nrm = rwork2[j];
-    } else {
-      *nrm = LAPACKlange_("O",&m,&n,(PetscScalar*)A,&m,rwork);
-    }
-    ierr = PetscLogFlops(1.0*m*n);CHKERRQ(ierr);
-  } else if (type==NORM_INFINITY) {
-    ierr = BVAllocateWork_Private(bv,m_);CHKERRQ(ierr);
-    rwork = (PetscReal*)bv->work;
-    lnrm = LAPACKlange_("I",&m,&n,(PetscScalar*)A,&m,rwork);
-    if (mpi) {
-      ierr = MPI_Allreduce(&lnrm,nrm,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-    } else *nrm = lnrm;
-    ierr = PetscLogFlops(1.0*m*n);CHKERRQ(ierr);
-  }
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-/*
-    QR factorization of an mxn matrix
-*/
-PetscErrorCode BVOrthogonalize_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,PetscScalar *Q,PetscScalar *R)
-{
-#if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(PETSC_MISSING_LAPACK_ORGQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF/ORGQR - Lapack routines are unavailable");
-#else
-  PetscErrorCode ierr;
-  PetscBLASInt   m,n,i,j,k,l,nb,lwork,info;
-  PetscScalar    *tau,*work,*Rl=NULL,*A=NULL,*C=NULL,one=1.0,zero=0.0;
-  PetscMPIInt    rank,size,len;
-  PetscBool      mpi;
-
-  PetscFunctionBegin;
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m_,&m);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(n_,&n);CHKERRQ(ierr);
-  k = PetscMin(m,n);
-  nb = 16;
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)bv),&size);CHKERRQ(ierr);
-  if (size>2) SETERRQ(PetscObjectComm((PetscObject)bv),1,"Not implemented yet for more than two MPI processes");
-  mpi = size>1? PETSC_TRUE: PETSC_FALSE;
-  if (mpi) {
-    ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)bv),&rank);CHKERRQ(ierr);
-    ierr = BVAllocateWork_Private(bv,k+n*nb+n*n+n*n*size+m*n);CHKERRQ(ierr);
-  } else {
-    ierr = BVAllocateWork_Private(bv,k+n*nb);CHKERRQ(ierr);
-   }
-  tau = bv->work;
-  work = bv->work+k;
-  ierr = PetscBLASIntCast(n*nb,&lwork);CHKERRQ(ierr);
-  if (mpi) {
-    Rl = bv->work+k+n*nb;
-    A  = bv->work+k+n*nb+n*n;
-    C  = bv->work+k+n*nb+n*n+n*n*size;
-  }
-
-  /* Compute QR */
-  PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&m,&n,Q,&m,tau,work,&lwork,&info));
-  SlepcCheckLapackInfo("geqrf",info);
-
-  /* Extract R */
-  if (R || mpi) {
-    ierr = PetscMemzero(mpi? Rl: R,n*n*sizeof(PetscScalar));CHKERRQ(ierr);
-    for (j=0;j<n;j++) {
-      for (i=0;i<=j;i++) {
-        if (mpi) Rl[i+j*n] = Q[i+j*m];
-        else R[i+j*n] = Q[i+j*m];
-      }
-    }
-  }
-
-  /* Compute orthogonal matrix in Q */
-  PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&m,&n,&k,Q,&m,tau,work,&lwork,&info));
-  SlepcCheckLapackInfo("ungqr",info);
-
-  if (mpi) {
-
-    /* Stack triangular matrices */
-    ierr = PetscBLASIntCast(n*size,&l);CHKERRQ(ierr);
-    ierr = PetscMPIIntCast(n,&len);CHKERRQ(ierr);
-    for (j=0;j<n;j++) {
-      ierr = MPI_Allgather(Rl+j*n,len,MPIU_SCALAR,A+j*l,len,MPIU_SCALAR,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
-    }
-
-    /* Compute QR */
-    PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&l,&n,A,&l,tau,work,&lwork,&info));
-    SlepcCheckLapackInfo("geqrf",info);
-
-    /* Extract R */
-    if (R) {
-      ierr = PetscMemzero(R,n*n*sizeof(PetscScalar));CHKERRQ(ierr);
-      for (j=0;j<n;j++)
-        for (i=0;i<=j;i++)
-          R[i+j*n] = A[i+j*l];
-    }
-
-    /* Accumulate orthogonal matrix */
-    PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&l,&n,&n,A,&l,tau,work,&lwork,&info));
-    SlepcCheckLapackInfo("ungqr",info);
-    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&m,&n,&n,&one,Q,&m,A+rank*n,&l,&zero,C,&m));
-    ierr = PetscMemcpy(Q,C,m*n*sizeof(PetscScalar));CHKERRQ(ierr);
-  }
-
-  ierr = PetscLogFlops(3.0*m*n*n);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-#endif
-}
-
