@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -56,6 +56,7 @@ PetscErrorCode BVInitializePackage(void)
 {
   char           logList[256];
   PetscBool      opt,pkg;
+  PetscClassId   classids[1];
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -84,12 +85,9 @@ PetscErrorCode BVInitializePackage(void)
   ierr = PetscLogEventRegister("BVMatProject",BV_CLASSID,&BV_MatProject);CHKERRQ(ierr);
   /* MPI reduction operation used in BVOrthogonalize */
   ierr = MPI_Op_create(SlepcGivensPacked,PETSC_TRUE,&MPIU_TSQR);CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrInList("bv",logList,',',&pkg);CHKERRQ(ierr);
-    if (pkg) { ierr = PetscInfoDeactivateClass(BV_CLASSID);CHKERRQ(ierr); }
-  }
+  /* Process Info */
+  classids[0] = BV_CLASSID;
+  ierr = PetscInfoProcessClass("bv",1,&classids[0]);CHKERRQ(ierr);
   /* Process summary exclusions */
   ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
@@ -104,7 +102,7 @@ PetscErrorCode BVInitializePackage(void)
 /*@
    BVDestroy - Destroys BV context that was created with BVCreate().
 
-   Collective on BV
+   Collective on bv
 
    Input Parameter:
 .  bv - the basis vectors context
@@ -144,7 +142,7 @@ PetscErrorCode BVDestroy(BV *bv)
 /*@
    BVCreate - Creates a basis vectors context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -224,7 +222,7 @@ PetscErrorCode BVCreate(MPI_Comm comm,BV *newbv)
 /*@
    BVCreateFromMat - Creates a basis vectors object from a dense Mat object.
 
-   Collective on Mat
+   Collective on A
 
    Input Parameter:
 .  A - a dense tall-skinny matrix
@@ -266,7 +264,7 @@ PetscErrorCode BVCreateFromMat(Mat A,BV *bv)
 /*@
    BVInsertVec - Insert a vector into the specified column.
 
-   Collective on BV
+   Collective on V
 
    Input Parameters:
 +  V - basis vectors
@@ -306,7 +304,7 @@ PetscErrorCode BVInsertVec(BV V,PetscInt j,Vec w)
 /*@
    BVInsertVecs - Insert a set of vectors into the specified columns.
 
-   Collective on BV
+   Collective on V
 
    Input Parameters:
 +  V - basis vectors
@@ -339,7 +337,7 @@ PetscErrorCode BVInsertVecs(BV V,PetscInt s,PetscInt *m,Vec *W,PetscBool orth)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(V,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(V,s,2);
-  PetscValidPointer(m,3);
+  PetscValidIntPointer(m,3);
   PetscValidLogicalCollectiveInt(V,*m,3);
   if (!*m) PetscFunctionReturn(0);
   if (*m<0) SETERRQ1(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Number of vectors (given %D) cannot be negative",*m);
@@ -379,7 +377,7 @@ PetscErrorCode BVInsertVecs(BV V,PetscInt s,PetscInt *m,Vec *W,PetscBool orth)
 /*@
    BVInsertConstraints - Insert a set of vectors as constraints.
 
-   Collective on BV
+   Collective on V
 
    Input Parameters:
 +  V - basis vectors
@@ -418,7 +416,7 @@ PetscErrorCode BVInsertConstraints(BV V,PetscInt *nc,Vec *C)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(V,BV_CLASSID,1);
-  PetscValidPointer(nc,2);
+  PetscValidIntPointer(nc,2);
   PetscValidLogicalCollectiveInt(V,*nc,2);
   if (!*nc) PetscFunctionReturn(0);
   if (*nc<0) SETERRQ1(PetscObjectComm((PetscObject)V),PETSC_ERR_ARG_OUTOFRANGE,"Number of constraints (given %D) cannot be negative",*nc);
@@ -446,7 +444,7 @@ PetscErrorCode BVInsertConstraints(BV V,PetscInt *nc,Vec *C)
    BVSetOptionsPrefix - Sets the prefix used for searching for all
    BV options in the database.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 +  bv     - the basis vectors context
@@ -475,7 +473,7 @@ PetscErrorCode BVSetOptionsPrefix(BV bv,const char *prefix)
    BVAppendOptionsPrefix - Appends to the prefix used for searching for all
    BV options in the database.
 
-   Logically Collective on BV
+   Logically Collective on bv
 
    Input Parameters:
 +  bv     - the basis vectors context
@@ -565,7 +563,7 @@ static PetscErrorCode BVView_Default(BV bv,PetscViewer viewer)
 /*@C
    BVView - Prints the BV data structure.
 
-   Collective on BV
+   Collective on bv
 
    Input Parameters:
 +  bv     - the BV context
@@ -583,14 +581,11 @@ static PetscErrorCode BVView_Default(BV bv,PetscViewer viewer)
    PetscViewerASCIIOpen() (output to a specified file).
 
    Level: beginner
-
-.seealso: PetscViewerASCIIOpen()
 @*/
 PetscErrorCode BVView(BV bv,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
   PetscBool         isascii;
-  PetscInt          tabs;
   PetscViewerFormat format;
   const char        *orthname[2] = {"classical","modified"};
   const char        *refname[3] = {"if needed","never","always"};
@@ -604,8 +599,6 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)bv)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)bv,viewer);CHKERRQ(ierr);
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
@@ -630,8 +623,11 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
         } else {
           ierr = PetscViewerASCIIPrintf(viewer,"  non-standard inner product\n");CHKERRQ(ierr);
         }
+        ierr = PetscViewerASCIIPrintf(viewer,"  inner product matrix:\n");CHKERRQ(ierr);
         ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
         ierr = MatView(bv->matrix,viewer);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
         ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
       }
       switch (bv->vmm) {
@@ -653,13 +649,36 @@ PetscErrorCode BVView(BV bv,PetscViewer viewer)
       if (bv->ops->view) { ierr = (*bv->ops->view)(bv,viewer);CHKERRQ(ierr); }
       else { ierr = BVView_Default(bv,viewer);CHKERRQ(ierr); }
     }
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else {
     ierr = (*bv->ops->view)(bv,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
+/*@C
+   BVViewFromOptions - View from options
+
+   Collective on BV
+
+   Input Parameters:
++  bv   - the basis vectors context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: BVView(), BVCreate()
+@*/
+PetscErrorCode BVViewFromOptions(BV bv,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)bv,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    BVRegister - Adds a new storage format to the BV package.
 
@@ -682,6 +701,7 @@ PetscErrorCode BVRegister(const char *name,PetscErrorCode (*function)(BV))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = BVInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&BVList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -700,3 +720,26 @@ PetscErrorCode BVAllocateWork_Private(BV bv,PetscInt s)
   PetscFunctionReturn(0);
 }
 
+#if defined(PETSC_USE_DEBUG)
+/*
+   SlepcDebugBVView - partially view a BV object, to be used from within a debugger.
+
+     ini, end: columns to be viewed
+     s: name of Matlab variable
+     filename: optionally write output to a file
+ */
+PETSC_UNUSED PetscErrorCode SlepcDebugBVView(BV bv,PetscInt ini,PetscInt end,const char *s,const char *filename)
+{
+  PetscErrorCode ierr;
+  PetscInt       N,m;
+  PetscScalar    *array;
+
+  PetscFunctionBegin;
+  ierr = BVGetArray(bv,&array);CHKERRQ(ierr);
+  ierr = BVGetSizes(bv,NULL,&N,&m);CHKERRQ(ierr);
+  ierr = SlepcDebugViewMatrix(N,end-ini+1,array+ini*N,NULL,N,s,filename);CHKERRQ(ierr);
+  ierr = BVRestoreArray(bv,&array);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+#endif
+
