@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -75,10 +75,6 @@ PetscErrorCode BVNorm_LAPACK_Private(BV bv,PetscInt m_,PetscInt n_,const PetscSc
  */
 PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
 {
-#if defined(PETSC_MISSING_LAPACK_POTRF) || defined(SLEPC_MISSING_LAPACK_TRTRI)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"POTRF/TRTRI - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS;
@@ -146,7 +142,6 @@ PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
   ierr = MatDenseRestoreArray(R,&pR);CHKERRQ(ierr);
   if (S!=R) { ierr = MatDenseRestoreArray(S,&pS);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -155,10 +150,6 @@ PetscErrorCode BVMatCholInv_LAPACK_Private(BV bv,Mat R,Mat S)
  */
 PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
 {
-#if defined(SLEPC_MISSING_LAPACK_TRTRI)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TRTRI - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS;
@@ -202,7 +193,6 @@ PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
   ierr = MatDenseRestoreArray(R,&pR);CHKERRQ(ierr);
   if (S!=R) { ierr = MatDenseRestoreArray(S,&pS);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -214,10 +204,6 @@ PetscErrorCode BVMatTriInv_LAPACK_Private(BV bv,Mat R,Mat S)
  */
 PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
 {
-#if defined(SLEPC_MISSING_LAPACK_SYEV)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"SYEV - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,j,k,l,n,m,ld,lds;
   PetscScalar    *pR,*pS,*D,*work,a;
@@ -296,7 +282,6 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
   ierr = MatDenseRestoreArray(R,&pR);CHKERRQ(ierr);
   if (S!=R) { ierr = MatDenseRestoreArray(S,&pS);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -304,10 +289,6 @@ PetscErrorCode BVMatSVQB_LAPACK_Private(BV bv,Mat R,Mat S)
 */
 PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscScalar *Q,PetscScalar *R,PetscInt ldr)
 {
-#if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(PETSC_MISSING_LAPACK_ORGQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF/ORGQR - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       level,plevel,nlevels,powtwo,lda,worklen;
   PetscBLASInt   m,n,i,j,k,l,s,nb,sz,lwork,info;
@@ -438,7 +419,6 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV bv,PetscInt m_,PetscInt n_,PetscSc
   ierr = PetscLogFlops(3.0*m*n*n);CHKERRQ(ierr);
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -473,10 +453,6 @@ SLEPC_EXTERN void MPIAPI SlepcGivensPacked(void *in,void *inout,PetscMPIInt *len
 */
 PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV bv,PetscInt m_,PetscInt n_,PetscScalar *Q,PetscScalar *R,PetscInt ldr)
 {
-#if defined(PETSC_MISSING_LAPACK_GEQRF)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       worklen;
   PetscBLASInt   m,n,i,j,s,nb,lwork,info;
@@ -519,7 +495,12 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV bv,PetscInt m_,PetscInt n_,P
     for (i=0;i<n;i++) {
       for (j=i;j<n;j++) R1[(2*n-i-1)*i/2+j] = (i<m)?A[i+j*m]:0.0;
     }
+#if defined(PETSC_HAVE_I_MPI_NUMVERSION)
+    ierr = MPI_Reduce(R1,R2,1,tmat,MPIU_TSQR,0,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+    ierr = MPI_Bcast(R2,1,tmat,0,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+#else
     ierr = MPI_Allreduce(R1,R2,1,tmat,MPIU_TSQR,PetscObjectComm((PetscObject)bv));CHKERRQ(ierr);
+#endif
     for (i=0;i<n;i++) {
       for (j=0;j<i;j++) R[i+j*ldr] = 0.0;
       for (j=i;j<n;j++) R[i+j*ldr] = R2[(2*n-i-1)*i/2+j];
@@ -530,6 +511,5 @@ PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV bv,PetscInt m_,PetscInt n_,P
   ierr = PetscLogFlops(3.0*m*n*n);CHKERRQ(ierr);
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
