@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -42,8 +42,8 @@ PetscErrorCode DSSwitchFormat_GHIEP(DS ds,PetscBool tocompact)
   n = ds->n;
   ld = ds->ld;
   if (tocompact) { /* switch from dense (arrow) to compact storage */
-    ierr = PetscMemzero(T,3*ld*sizeof(PetscReal));CHKERRQ(ierr);
-    ierr = PetscMemzero(S,ld*sizeof(PetscReal));CHKERRQ(ierr);
+    ierr = PetscArrayzero(T,3*ld);CHKERRQ(ierr);
+    ierr = PetscArrayzero(S,ld);CHKERRQ(ierr);
     for (i=0;i<n-1;i++) {
       T[i]    = PetscRealPart(A[i+i*ld]);
       T[ld+i] = PetscRealPart(A[i+1+i*ld]);
@@ -53,8 +53,8 @@ PetscErrorCode DSSwitchFormat_GHIEP(DS ds,PetscBool tocompact)
     S[n-1] = PetscRealPart(B[n-1+(n-1)*ld]);
     for (i=ds->l;i<ds->k;i++) T[2*ld+i] = PetscRealPart(A[ds->k+i*ld]);
   } else { /* switch from compact (arrow) to dense storage */
-    ierr = PetscMemzero(A,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = PetscMemzero(B,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArrayzero(A,ld*ld);CHKERRQ(ierr);
+    ierr = PetscArrayzero(B,ld*ld);CHKERRQ(ierr);
     for (i=0;i<n-1;i++) {
       A[i+i*ld]     = T[i];
       A[i+1+i*ld]   = T[ld+i];
@@ -151,18 +151,12 @@ PetscErrorCode DSView_GHIEP(DS ds,PetscViewer viewer)
     ierr = DSViewMat(ds,viewer,DS_MAT_A);CHKERRQ(ierr);
     ierr = DSViewMat(ds,viewer,DS_MAT_B);CHKERRQ(ierr);
   }
-  if (ds->state>DS_STATE_INTERMEDIATE) {
-    ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr);
-  }
+  if (ds->state>DS_STATE_INTERMEDIATE) { ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *rnorm)
 {
-#if defined(SLEPC_MISSING_LAPACK_LAG2)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAG2 - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscReal      b[4],M[4],d1,d2,s1,s2,e;
   PetscReal      scal1,scal2,wr1,wr2,wi,ep,norm;
@@ -183,9 +177,9 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
   else e = 0.0;
   if (e == 0.0) { /* Real */
     if (ds->state>=DS_STATE_CONDENSED) {
-      ierr = PetscMemcpy(X+k*ld,Q+k*ld,ld*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(X+k*ld,Q+k*ld,ld);CHKERRQ(ierr);
     } else {
-      ierr = PetscMemzero(X+k*ds->ld,ds->ld*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArrayzero(X+k*ds->ld,ds->ld);CHKERRQ(ierr);
       X[k+k*ds->ld] = 1.0;
     }
     if (rnorm) *rnorm = PetscAbsScalar(X[ds->n-1+k*ld]);
@@ -224,7 +218,7 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
         PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n_,&two,&two,&alpha,ds->mat[DS_MAT_Q]+k*ld,&ld,Y,&two,&zeroS,X+k*ld,&ld));
         if (rnorm) *rnorm = SlepcAbsEigenvalue(X[ds->n-1+k*ld],X[ds->n-1+(k+1)*ld]);
       } else {
-        ierr = PetscMemzero(X+k*ld,2*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = PetscArrayzero(X+k*ld,2*ld);CHKERRQ(ierr);
         X[k*ld+k]       = Y[0]*norm;
         X[k*ld+k+1]     = Y[1]*norm;
         X[(k+1)*ld+k]   = Y[2]*norm;
@@ -232,11 +226,11 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
       }
 #else
       if (SlepcAbs(s1*d1-wr1,wi)<SlepcAbs(s2*d2-wr1,wi)) {
-        Y[0] = wr1-s2*d2+PETSC_i*wi;
+        Y[0] = PetscCMPLX(wr1-s2*d2,wi);
         Y[1] = s2*e;
       } else {
         Y[0] = s1*e;
-        Y[1] = wr1-s1*d1+PETSC_i*wi;
+        Y[1] = PetscCMPLX(wr1-s1*d1,wi);
       }
       norm = BLASnrm2_(&two,Y,&one);
       norm = 1.0/norm;
@@ -245,7 +239,7 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
         PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n_,&two,&alpha,ds->mat[DS_MAT_Q]+k*ld,&ld,Y,&one,&zeroS,X+k*ld,&one));
         if (rnorm) *rnorm = PetscAbsScalar(X[ds->n-1+k*ld]);
       } else {
-        ierr = PetscMemzero(X+k*ld,2*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = PetscArrayzero(X+k*ld,2*ld);CHKERRQ(ierr);
         X[k*ld+k]   = Y[0]*norm;
         X[k*ld+k+1] = Y[1]*norm;
       }
@@ -256,7 +250,6 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
     }
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSVectors_GHIEP(DS ds,DSMatType mat,PetscInt *k,PetscReal *rnorm)
@@ -276,9 +269,9 @@ PetscErrorCode DSVectors_GHIEP(DS ds,DSMatType mat,PetscInt *k,PetscReal *rnorm)
           e = (ds->compact)?*(ds->rmat[DS_MAT_T]+ds->ld+i):PetscRealPart(*(ds->mat[DS_MAT_A]+(i+1)+ds->ld*i));
           if (e == 0.0) { /* real */
             if (ds->state >= DS_STATE_CONDENSED) {
-              ierr = PetscMemcpy(ds->mat[mat]+i*ds->ld,ds->mat[DS_MAT_Q]+i*ds->ld,ds->ld*sizeof(PetscScalar));CHKERRQ(ierr);
+              ierr = PetscArraycpy(ds->mat[mat]+i*ds->ld,ds->mat[DS_MAT_Q]+i*ds->ld,ds->ld);CHKERRQ(ierr);
             } else {
-              ierr = PetscMemzero(ds->mat[mat]+i*ds->ld,ds->ld*sizeof(PetscScalar));CHKERRQ(ierr);
+              ierr = PetscArrayzero(ds->mat[mat]+i*ds->ld,ds->ld);CHKERRQ(ierr);
               *(ds->mat[mat]+i+i*ds->ld) = 1.0;
             }
           } else {
@@ -303,10 +296,6 @@ PetscErrorCode DSVectors_GHIEP(DS ds,DSMatType mat,PetscInt *k,PetscReal *rnorm)
 */
 PetscErrorCode DSGHIEPComplexEigs(DS ds,PetscInt n0,PetscInt n1,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_LAG2)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAG2 - Lapack routine is unavailable");
-#else
   PetscInt     k,ld;
   PetscBLASInt two=2;
   PetscScalar  *A,*B;
@@ -346,21 +335,20 @@ PetscErrorCode DSGHIEPComplexEigs(DS ds,PetscInt n0,PetscInt n1,PetscScalar *wr,
       /* Compute eigenvalues of the block */
       PetscStackCallBLAS("LAPACKlag2",LAPACKlag2_(M,&two,b,&two,&ep,&scal1,&scal2,&wr1,&wr2,&wi1));
       if (scal1<ep) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
-      wr[k] = wr1/scal1;
       if (wi1==0.0) { /* Real eigenvalues */
         if (scal2<ep) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
-        wr[k+1] = wr2/scal2;
+        wr[k] = wr1/scal1; wr[k+1] = wr2/scal2;
 #if !defined(PETSC_USE_COMPLEX)
-        wi[k]   = 0.0;
-        wi[k+1] = 0.0;
+        wi[k] = wi[k+1] = 0.0;
 #endif
       } else { /* Complex eigenvalues */
 #if !defined(PETSC_USE_COMPLEX)
+        wr[k]   = wr1/scal1;
         wr[k+1] = wr[k];
         wi[k]   = wi1/scal1;
         wi[k+1] = -wi[k];
 #else
-        wr[k]  += PETSC_i*wi1/scal1;
+        wr[k]   = PetscCMPLX(wr1,wi1)/scal1;
         wr[k+1] = PetscConj(wr[k]);
 #endif
       }
@@ -373,7 +361,6 @@ PetscErrorCode DSGHIEPComplexEigs(DS ds,PetscInt n0,PetscInt n1,PetscScalar *wr,
   }
 #endif
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
@@ -384,7 +371,7 @@ PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *r
 
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(wi,3);
+  PetscValidScalarPointer(wi,3);
 #endif
   n = ds->n;
   d = ds->rmat[DS_MAT_T];
@@ -398,18 +385,18 @@ PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *r
   }
   ierr = DSSortEigenvalues_Private(ds,rr,ri,perm,PETSC_TRUE);CHKERRQ(ierr);
   if (!ds->compact) { ierr = DSSwitchFormat_GHIEP(ds,PETSC_TRUE);CHKERRQ(ierr); }
-  ierr = PetscMemcpy(ds->work,wr,n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(ds->work,wr,n);CHKERRQ(ierr);
   for (i=ds->l;i<n;i++) wr[i] = *(ds->work+perm[i]);
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscMemcpy(ds->work,wi,n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(ds->work,wi,n);CHKERRQ(ierr);
   for (i=ds->l;i<n;i++) wi[i] = *(ds->work+perm[i]);
 #endif
-  ierr = PetscMemcpy(ds->rwork,s,n*sizeof(PetscReal));CHKERRQ(ierr);
+  ierr = PetscArraycpy(ds->rwork,s,n);CHKERRQ(ierr);
   for (i=ds->l;i<n;i++) s[i] = *(ds->rwork+perm[i]);
-  ierr = PetscMemcpy(ds->rwork,d,n*sizeof(PetscReal));CHKERRQ(ierr);
+  ierr = PetscArraycpy(ds->rwork,d,n);CHKERRQ(ierr);
   for (i=ds->l;i<n;i++) d[i] = *(ds->rwork+perm[i]);
-  ierr = PetscMemcpy(ds->rwork,e,(n-1)*sizeof(PetscReal));CHKERRQ(ierr);
-  ierr = PetscMemzero(e+ds->l,(n-1-ds->l)*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(ds->rwork,e,n-1);CHKERRQ(ierr);
+  ierr = PetscArrayzero(e+ds->l,n-1-ds->l);CHKERRQ(ierr);
   for (i=ds->l;i<n-1;i++) {
     if (perm[i]<n-1) e[i] = *(ds->rwork+perm[i]);
   }
@@ -424,10 +411,6 @@ PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *r
 */
 PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_HSEIN)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"HSEIN - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,off;
   PetscBLASInt   *select,*infoC,ld,n1,mout,info;
@@ -497,7 +480,6 @@ PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
   SlepcCheckLapackInfo("hsein",info);
   ierr = DSGHIEPOrthogEigenv(ds,DS_MAT_X,wr,wi,PETSC_TRUE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -505,10 +487,6 @@ PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
 */
 PetscErrorCode DSGHIEPRealBlocks(DS ds)
 {
-#if defined(SLEPC_MISSING_LAPACK_LAG2)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAG2 - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscReal      e,d1,d2,s1,s2,ss1,ss2,t,dd,ss;
@@ -553,8 +531,7 @@ PetscErrorCode DSGHIEPRealBlocks(DS ds)
         }
         Y[0] = 1/PetscSqrtReal(1 + t*t); Y[3] = Y[0]; /* c */
         Y[1] = Y[0]*t; Y[2] = -Y[1]; /* s */
-        wr1 = d1+t*e;
-        wr2 = d2-t*e;
+        wr1 = d1+t*e; wr2 = d2-t*e;
         ss1 = s1; ss2 = s2;
         isreal = PETSC_TRUE;
       } else {
@@ -618,22 +595,17 @@ PetscErrorCode DSGHIEPRealBlocks(DS ds)
           A[i+ld*(i+1)]   = 0.0;
         }
         PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&m,&two,&two,&oneS,ds->mat[DS_MAT_Q]+ds->l+i*ld,&ld,Y,&two,&zeroS,ds->work,&m));
-        ierr = PetscMemcpy(ds->mat[DS_MAT_Q]+ds->l+i*ld,ds->work,m*sizeof(PetscScalar));CHKERRQ(ierr);
-        ierr = PetscMemcpy(ds->mat[DS_MAT_Q]+ds->l+(i+1)*ld,ds->work+m,m*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = PetscArraycpy(ds->mat[DS_MAT_Q]+ds->l+i*ld,ds->work,m);CHKERRQ(ierr);
+        ierr = PetscArraycpy(ds->mat[DS_MAT_Q]+ds->l+(i+1)*ld,ds->work+m,m);CHKERRQ(ierr);
       }
       i++;
     }
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(PETSC_MISSING_LAPACK_HSEQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"HSEQR - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,off;
   PetscBLASInt   n1,ld,one,info,lwork;
@@ -662,7 +634,8 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   /* Quick return if possible */
   if (n1 == 1) {
-    Q[off] = 1.0;
+    for (i=0;i<=ds->l;i++) Q[i+i*ld] = 1.0;
+    ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
     if (!ds->compact) {
       d[ds->l] = PetscRealPart(A[off]);
       s[ds->l] = PetscRealPart(B[off]);
@@ -706,7 +679,7 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
   PetscStackCallBLAS("LAPACKhseqr",LAPACKhseqr_("E","N",&n1,&one,&n1,H+off,&ld,wr+ds->l,wi+ds->l,NULL,&ld,ds->work,&lwork,&info));
 #else
   PetscStackCallBLAS("LAPACKhseqr",LAPACKhseqr_("E","N",&n1,&one,&n1,H+off,&ld,wr+ds->l,NULL,&ld,ds->work,&lwork,&info));
-
+  for (i=ds->l;i<ds->n;i++) if (PetscAbsReal(PetscImaginaryPart(wr[i]))<10*PETSC_MACHINE_EPSILON) wr[i] = PetscRealPart(wr[i]);
   /* Sort to have consecutive conjugate pairs */
   for (i=ds->l;i<ds->n;i++) {
       j=i+1;
@@ -734,15 +707,10 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
   }
 #endif
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_GEEVX)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEEVX - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,j,off,nwu=0,n,lw,lwr,nwru=0;
   PetscBLASInt   n_,ld,info,lwork,ilo,ihi;
@@ -776,7 +744,8 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   nwru += ld;
   /* Quick return if possible */
   if (n_ == 1) {
-    Q[off] = 1.0;
+    for (i=0;i<=ds->l;i++) Q[i+i*ld] = 1.0;
+    ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
     if (!ds->compact) {
       d[ds->l] = PetscRealPart(A[off]);
       s[ds->l] = PetscRealPart(B[off]);
@@ -789,7 +758,7 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   /* Form pseudo-symmetric matrix */
   H =  ds->work+nwu;
   nwu += n*n;
-  ierr = PetscMemzero(H,n*n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(H,n*n);CHKERRQ(ierr);
   if (ds->compact) {
     for (i=0;i<n-1;i++) {
       H[i+i*n]     = s[ds->l+i]*d[ds->l+i];
@@ -831,7 +800,7 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
     } else { /* complex eigenvalue */
       if (j!=i+1) {
         wr[j] = wr[i+1];
-        ierr = PetscMemcpy(X+j*ds->ld,X+(i+1)*ds->ld,ds->ld*sizeof(PetscScalar));CHKERRQ(ierr);
+        ierr = PetscArraycpy(X+j*ds->ld,X+(i+1)*ds->ld,ds->ld);CHKERRQ(ierr);
       }
       if (PetscImaginaryPart(wr[i])<0) {
         wr[i] = PetscConj(wr[i]);
@@ -863,7 +832,6 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   }
 #endif
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSynchronize_GHIEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
@@ -876,8 +844,8 @@ PetscErrorCode DSSynchronize_GHIEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
   if (ds->compact) kr = 4*ld;
   else k = 2*(ds->n-l)*ld;
   if (ds->state>DS_STATE_RAW) k += (ds->n-l)*ld;
-  if (eigr) k += (ds->n-l); 
-  if (eigi) k += (ds->n-l); 
+  if (eigr) k += (ds->n-l);
+  if (eigi) k += (ds->n-l);
   ierr = DSAllocateWork_Private(ds,k+kr,0,0);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(k*sizeof(PetscScalar)+kr*sizeof(PetscReal),&size);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
@@ -933,7 +901,7 @@ PetscErrorCode DSHermitian_GHIEP(DS ds,DSMatType m,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode DSCreate_GHIEP(DS ds)
+SLEPC_EXTERN PetscErrorCode DSCreate_GHIEP(DS ds)
 {
   PetscFunctionBegin;
   ds->ops->allocate      = DSAllocate_GHIEP;
