@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -50,7 +50,7 @@ PetscErrorCode DSAllocate_HEP(DS ds,PetscInt ld)
     -----------------------------------------
 */
 
-static PetscErrorCode DSSwitchFormat_HEP(DS ds,PetscBool tocompact)
+static PetscErrorCode DSSwitchFormat_HEP(DS ds)
 {
   PetscErrorCode ierr;
   PetscReal      *T = ds->rmat[DS_MAT_T];
@@ -58,33 +58,20 @@ static PetscErrorCode DSSwitchFormat_HEP(DS ds,PetscBool tocompact)
   PetscInt       i,n=ds->n,k=ds->k,ld=ds->ld;
 
   PetscFunctionBegin;
-  if (tocompact) { /* switch from dense (arrow) to compact storage */
-    ierr = PetscMemzero(T,3*ld*sizeof(PetscReal));CHKERRQ(ierr);
-    for (i=0;i<k;i++) {
-      T[i] = PetscRealPart(A[i+i*ld]);
-      T[i+ld] = PetscRealPart(A[k+i*ld]);
-    }
-    for (i=k;i<n-1;i++) {
-      T[i]    = PetscRealPart(A[i+i*ld]);
-      T[i+ld] = PetscRealPart(A[i+1+i*ld]);
-    }
-    T[n-1] = PetscRealPart(A[n-1+(n-1)*ld]);
-    if (ds->extrarow) T[n-1+ld] = PetscRealPart(A[n+(n-1)*ld]);
-  } else { /* switch from compact (arrow) to dense storage */
-    ierr = PetscMemzero(A,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
-    for (i=0;i<k;i++) {
-      A[i+i*ld] = T[i];
-      A[k+i*ld] = T[i+ld];
-      A[i+k*ld] = T[i+ld];
-    }
-    A[k+k*ld] = T[k];
-    for (i=k+1;i<n;i++) {
-      A[i+i*ld]     = T[i];
-      A[i-1+i*ld]   = T[i-1+ld];
-      A[i+(i-1)*ld] = T[i-1+ld];
-    }
-    if (ds->extrarow) A[n+(n-1)*ld] = T[n-1+ld];
+  /* switch from compact (arrow) to dense storage */
+  ierr = PetscArrayzero(A,ld*ld);CHKERRQ(ierr);
+  for (i=0;i<k;i++) {
+    A[i+i*ld] = T[i];
+    A[k+i*ld] = T[i+ld];
+    A[i+k*ld] = T[i+ld];
   }
+  A[k+k*ld] = T[k];
+  for (i=k+1;i<n;i++) {
+    A[i+i*ld]     = T[i];
+    A[i-1+i*ld]   = T[i-1+ld];
+    A[i+(i-1)*ld] = T[i-1+ld];
+  }
+  if (ds->extrarow) A[n+(n-1)*ld] = T[n-1+ld];
   PetscFunctionReturn(0);
 }
 
@@ -150,9 +137,7 @@ PetscErrorCode DSView_HEP(DS ds,PetscViewer viewer)
   } else {
     ierr = DSViewMat(ds,viewer,DS_MAT_A);CHKERRQ(ierr);
   }
-  if (ds->state>DS_STATE_INTERMEDIATE) {
-    ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr);
-  }
+  if (ds->state>DS_STATE_INTERMEDIATE) { ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
@@ -168,16 +153,16 @@ PetscErrorCode DSVectors_HEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
     case DS_MAT_Y:
       if (j) {
         if (ds->state>=DS_STATE_CONDENSED) {
-          ierr = PetscMemcpy(ds->mat[mat]+(*j)*ld,Q+(*j)*ld,ld*sizeof(PetscScalar));CHKERRQ(ierr);
+          ierr = PetscArraycpy(ds->mat[mat]+(*j)*ld,Q+(*j)*ld,ld);CHKERRQ(ierr);
         } else {
-          ierr = PetscMemzero(ds->mat[mat]+(*j)*ld,ld*sizeof(PetscScalar));CHKERRQ(ierr);
+          ierr = PetscArrayzero(ds->mat[mat]+(*j)*ld,ld);CHKERRQ(ierr);
           *(ds->mat[mat]+(*j)+(*j)*ld) = 1.0;
         }
       } else {
         if (ds->state>=DS_STATE_CONDENSED) {
-          ierr = PetscMemcpy(ds->mat[mat],Q,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+          ierr = PetscArraycpy(ds->mat[mat],Q,ld*ld);CHKERRQ(ierr);
         } else {
-          ierr = PetscMemzero(ds->mat[mat],ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+          ierr = PetscArrayzero(ds->mat[mat],ld*ld);CHKERRQ(ierr);
           for (i=0;i<ds->n;i++) *(ds->mat[mat]+i+i*ld) = 1.0;
         }
       }
@@ -242,10 +227,6 @@ PetscErrorCode DSVectors_HEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 */
 static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,PetscScalar *Q,PetscBLASInt ld)
 {
-#if defined(SLEPC_MISSING_LAPACK_LARTG)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LARTG - Lapack routine is unavailable");
-#else
   PetscBLASInt i,j,j2,one=1;
   PetscReal    c,s,p,off,temp;
 
@@ -256,7 +237,7 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
 
     /* Eliminate entry e(j) by a rotation in the planes (j,j+1) */
     temp = e[j+1];
-    PetscStackCallBLAS("LAPACKlartg",LAPACKlartg_(&temp,&e[j],&c,&s,&e[j+1]));
+    PetscStackCallBLAS("LAPACKlartg",LAPACKREALlartg_(&temp,&e[j],&c,&s,&e[j+1]));
     s = -s;
 
     /* Apply rotation to diagonal elements */
@@ -267,14 +248,14 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
 
     /* Apply rotation to Q */
     j2 = j+2;
-    PetscStackCallBLAS("BLASrot",BLASrot_(&j2,Q+j*ld,&one,Q+(j+1)*ld,&one,&c,&s));
+    PetscStackCallBLAS("BLASrot",BLASMIXEDrot_(&j2,Q+j*ld,&one,Q+(j+1)*ld,&one,&c,&s));
 
     /* Chase newly introduced off-diagonal entry to the top left corner */
     for (i=j-1;i>=0;i--) {
       off  = -s*e[i];
       e[i] = c*e[i];
       temp = e[i+1];
-      PetscStackCallBLAS("LAPACKlartg",LAPACKlartg_(&temp,&off,&c,&s,&e[i+1]));
+      PetscStackCallBLAS("LAPACKlartg",LAPACKREALlartg_(&temp,&off,&c,&s,&e[i+1]));
       s = -s;
       temp = (d[i]-d[i+1])*s - 2.0*c*e[i];
       p = s*temp;
@@ -282,11 +263,10 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
       d[i] -= p;
       e[i] = -e[i] - c*temp;
       j2 = j+2;
-      PetscStackCallBLAS("BLASrot",BLASrot_(&j2,Q+i*ld,&one,Q+(i+1)*ld,&one,&c,&s));
+      PetscStackCallBLAS("BLASrot",BLASMIXEDrot_(&j2,Q+i*ld,&one,Q+(i+1)*ld,&one,&c,&s));
     }
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -294,10 +274,6 @@ static PetscErrorCode ArrowTridiag(PetscBLASInt n,PetscReal *d,PetscReal *e,Pets
 */
 static PetscErrorCode DSIntermediate_HEP(DS ds)
 {
-#if defined(SLEPC_MISSING_LAPACK_SYTRD) || defined(SLEPC_MISSING_LAPACK_ORGTR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"SYTRD/ORGTR - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   n1,n2,n3,lwork,info,l,n,ld,off;
@@ -316,7 +292,7 @@ static PetscErrorCode DSIntermediate_HEP(DS ds)
   Q  = ds->mat[DS_MAT_Q];
   d  = ds->rmat[DS_MAT_T];
   e  = ds->rmat[DS_MAT_T]+ld;
-  ierr = PetscMemzero(Q,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(Q,ld*ld);CHKERRQ(ierr);
   for (i=0;i<n;i++) Q[i+i*ld] = 1.0;
 
   if (ds->compact) {
@@ -344,7 +320,6 @@ static PetscErrorCode DSIntermediate_HEP(DS ds)
     }
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSort_HEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
@@ -408,10 +383,6 @@ PetscErrorCode DSUpdateExtraRow_HEP(DS ds)
 
 PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(PETSC_MISSING_LAPACK_STEQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"STEQR - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   n1,n2,n3,info,l,n,ld,off;
@@ -445,10 +416,10 @@ PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   /* Create diagonal matrix as a result */
   if (ds->compact) {
-    ierr = PetscMemzero(e,(n-1)*sizeof(PetscReal));CHKERRQ(ierr);
+    ierr = PetscArrayzero(e,n-1);CHKERRQ(ierr);
   } else {
     for (i=l;i<n;i++) {
-      ierr = PetscMemzero(A+l+i*ld,(n-l)*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArrayzero(A+l+i*ld,n-l);CHKERRQ(ierr);
     }
     for (i=l;i<n;i++) A[i+i*ld] = d[i];
   }
@@ -456,15 +427,10 @@ PetscErrorCode DSSolve_HEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   /* Set zero wi */
   if (wi) for (i=l;i<n;i++) wi[i] = 0.0;
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_STEVR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"STEVR - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   n1,n2,n3,lwork,liwork,info,l,n,m,ld,off,il,iu,*isuppz;
@@ -528,10 +494,10 @@ PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   /* Create diagonal matrix as a result */
   if (ds->compact) {
-    ierr = PetscMemzero(e,(n-1)*sizeof(PetscReal));CHKERRQ(ierr);
+    ierr = PetscArrayzero(e,n-1);CHKERRQ(ierr);
   } else {
     for (i=l;i<n;i++) {
-      ierr = PetscMemzero(A+l+i*ld,(n-l)*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArrayzero(A+l+i*ld,n-l);CHKERRQ(ierr);
     }
     for (i=l;i<n;i++) A[i+i*ld] = d[i];
   }
@@ -539,15 +505,10 @@ PetscErrorCode DSSolve_HEP_MRRR(DS ds,PetscScalar *wr,PetscScalar *wi)
   /* Set zero wi */
   if (wi) for (i=l;i<n;i++) wi[i] = 0.0;
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_STEDC)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"STEDC - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   n1,info,l,ld,off,lrwork,liwork;
@@ -593,10 +554,10 @@ PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   /* Create diagonal matrix as a result */
   if (ds->compact) {
-    ierr = PetscMemzero(e,(ds->n-1)*sizeof(PetscReal));CHKERRQ(ierr);
+    ierr = PetscArrayzero(e,ds->n-1);CHKERRQ(ierr);
   } else {
     for (i=l;i<ds->n;i++) {
-      ierr = PetscMemzero(A+l+i*ld,(ds->n-l)*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArrayzero(A+l+i*ld,ds->n-l);CHKERRQ(ierr);
     }
     for (i=l;i<ds->n;i++) A[i+i*ld] = d[i];
   }
@@ -604,7 +565,6 @@ PetscErrorCode DSSolve_HEP_DC(DS ds,PetscScalar *wr,PetscScalar *wi)
   /* Set zero wi */
   if (wi) for (i=l;i<ds->n;i++) wi[i] = 0.0;
   PetscFunctionReturn(0);
-#endif
 }
 
 #if !defined(PETSC_USE_COMPLEX)
@@ -635,7 +595,7 @@ PetscErrorCode DSSolve_HEP_BDC(DS ds,PetscScalar *wr,PetscScalar *wi)
   rwork  = ds->rwork;
   ksizes = ds->iwork;
   iwork  = ds->iwork+nblks;
-  ierr = PetscMemzero(iwork,liwork*sizeof(PetscBLASInt));CHKERRQ(ierr);
+  ierr = PetscArrayzero(iwork,liwork);CHKERRQ(ierr);
 
   /* Copy matrix to block tridiagonal format */
   j=0;
@@ -661,10 +621,10 @@ PetscErrorCode DSSolve_HEP_BDC(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   /* Create diagonal matrix as a result */
   if (ds->compact) {
-    ierr = PetscMemzero(e,(ds->n-1)*sizeof(PetscReal));CHKERRQ(ierr);
+    ierr = PetscArrayzero(e,ds->n-1);CHKERRQ(ierr);
   } else {
     for (i=0;i<ds->n;i++) {
-      ierr = PetscMemzero(A+i*ld,ds->n*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArrayzero(A+i*ld,ds->n);CHKERRQ(ierr);
     }
     for (i=0;i<ds->n;i++) A[i+i*ld] = wr[i];
   }
@@ -702,8 +662,7 @@ PetscErrorCode DSSynchronize_HEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
   if (ds->compact) kr = 3*ld;
   else k = (ds->n-l)*ld;
   if (ds->state>DS_STATE_RAW) k += (ds->n-l)*ld;
-  if (eigr) k += (ds->n-l); 
-  if (eigi) k += (ds->n-l); 
+  if (eigr) k += (ds->n-l);
   ierr = DSAllocateWork_Private(ds,k+kr,0,0);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(k*sizeof(PetscScalar)+kr*sizeof(PetscReal),&size);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
@@ -722,9 +681,6 @@ PetscErrorCode DSSynchronize_HEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
     if (eigr) {
       ierr = MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
     }
-    if (eigi) {
-      ierr = MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
   }
   ierr = MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
   if (rank) {
@@ -739,19 +695,12 @@ PetscErrorCode DSSynchronize_HEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
     if (eigr) {
       ierr = MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
     }
-    if (eigi) {
-      ierr = MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
   }
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode DSCond_HEP(DS ds,PetscReal *cond)
 {
-#if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(SLEPC_MISSING_LAPACK_LANGE)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRF/GETRI/LANGE - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscScalar    *work;
   PetscReal      *rwork;
@@ -768,12 +717,12 @@ PetscErrorCode DSCond_HEP(DS ds,PetscReal *cond)
   work  = ds->work;
   rwork = ds->rwork;
   ipiv  = ds->iwork;
-  ierr = DSSwitchFormat_HEP(ds,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = DSSwitchFormat_HEP(ds);CHKERRQ(ierr);
 
   /* use workspace matrix W to avoid overwriting A */
   ierr = DSAllocateMat_Private(ds,DS_MAT_W);CHKERRQ(ierr);
   A = ds->mat[DS_MAT_W];
-  ierr = PetscMemcpy(A,ds->mat[DS_MAT_A],sizeof(PetscScalar)*ds->ld*ds->ld);CHKERRQ(ierr);
+  ierr = PetscArraycpy(A,ds->mat[DS_MAT_A],ds->ld*ds->ld);CHKERRQ(ierr);
 
   /* norm of A */
   hn = LAPACKlange_("I",&n,&n,A,&ld,rwork);
@@ -787,15 +736,10 @@ PetscErrorCode DSCond_HEP(DS ds,PetscReal *cond)
 
   *cond = hn*hin;
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSTranslateRKS_HEP(DS ds,PetscScalar alpha)
 {
-#if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(PETSC_MISSING_LAPACK_ORGQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF/ORGQR - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,j,k=ds->k;
   PetscScalar    *Q,*A,*R,*tau,*work;
@@ -813,8 +757,8 @@ PetscErrorCode DSTranslateRKS_HEP(DS ds,PetscScalar alpha)
   R  = ds->mat[DS_MAT_W];
 
   /* copy I+alpha*A */
-  ierr = PetscMemzero(Q,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
-  ierr = PetscMemzero(R,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(Q,ld*ld);CHKERRQ(ierr);
+  ierr = PetscArrayzero(R,ld*ld);CHKERRQ(ierr);
   for (i=0;i<k;i++) {
     Q[i+i*ld] = 1.0 + alpha*A[i+i*ld];
     Q[k+i*ld] = alpha*A[k+i*ld];
@@ -832,8 +776,8 @@ PetscErrorCode DSTranslateRKS_HEP(DS ds,PetscScalar alpha)
       R[i+j*ld] = Q[i+j*ld];
 
   /* compute orthogonal matrix in Q */
-  PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&n1,&n1,&n0,Q,&ld,tau,work,&lwork,&info));
-  SlepcCheckLapackInfo("ungqr",info);
+  PetscStackCallBLAS("LAPACKorgqr",LAPACKorgqr_(&n1,&n1,&n0,Q,&ld,tau,work,&lwork,&info));
+  SlepcCheckLapackInfo("orgqr",info);
 
   /* compute the updated matrix of projected problem */
   for (j=0;j<k;j++)
@@ -844,7 +788,6 @@ PetscErrorCode DSTranslateRKS_HEP(DS ds,PetscScalar alpha)
   for (i=0;i<k;i++)
     A[ld*i+i] -= alpha;
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSHermitian_HEP(DS ds,DSMatType m,PetscBool *flg)
@@ -855,7 +798,7 @@ PetscErrorCode DSHermitian_HEP(DS ds,DSMatType m,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode DSCreate_HEP(DS ds)
+SLEPC_EXTERN PetscErrorCode DSCreate_HEP(DS ds)
 {
   PetscFunctionBegin;
   ds->ops->allocate      = DSAllocate_HEP;
