@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -26,28 +26,21 @@ PetscErrorCode DSAllocate_NHEP(DS ds,PetscInt ld)
 
 PetscErrorCode DSView_NHEP(DS ds,PetscViewer viewer)
 {
-  PetscErrorCode ierr;
+  PetscErrorCode    ierr;
+  PetscViewerFormat format;
 
   PetscFunctionBegin;
+  ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+  if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) PetscFunctionReturn(0);
   ierr = DSViewMat(ds,viewer,DS_MAT_A);CHKERRQ(ierr);
-  if (ds->state>DS_STATE_INTERMEDIATE) {
-    ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr);
-  }
-  if (ds->mat[DS_MAT_X]) {
-    ierr = DSViewMat(ds,viewer,DS_MAT_X);CHKERRQ(ierr);
-  }
-  if (ds->mat[DS_MAT_Y]) {
-    ierr = DSViewMat(ds,viewer,DS_MAT_Y);CHKERRQ(ierr);
-  }
+  if (ds->state>DS_STATE_INTERMEDIATE) { ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr); }
+  if (ds->mat[DS_MAT_X]) { ierr = DSViewMat(ds,viewer,DS_MAT_X);CHKERRQ(ierr); }
+  if (ds->mat[DS_MAT_Y]) { ierr = DSViewMat(ds,viewer,DS_MAT_Y);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode DSVectors_NHEP_Refined_Some(DS ds,PetscInt *k,PetscReal *rnorm,PetscBool left)
 {
-#if defined(PETSC_MISSING_LAPACK_GESVD)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESVD - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,j;
   PetscBLASInt   info,ld,n,n1,lwork,inc=1;
@@ -94,7 +87,6 @@ static PetscErrorCode DSVectors_NHEP_Refined_Some(DS ds,PetscInt *k,PetscReal *r
      accumulating the transformation matrix Q */
   PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&done,Q,&ld,W+n-1,&ld,&zero,X+(*k)*ld,&inc));
   PetscFunctionReturn(0);
-#endif
 }
 
 static PetscErrorCode DSVectors_NHEP_Refined_All(DS ds,PetscBool left)
@@ -111,10 +103,6 @@ static PetscErrorCode DSVectors_NHEP_Refined_All(DS ds,PetscBool left)
 
 static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rnorm,PetscBool left)
 {
-#if defined(SLEPC_MISSING_LAPACK_TREVC)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   mm=1,mout,info,ld,n,*select,inc = 1;
@@ -151,7 +139,7 @@ static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rno
 
   /* accumulate and normalize eigenvectors */
   if (ds->state>=DS_STATE_CONDENSED) {
-    ierr = PetscMemcpy(ds->work,Y,mout*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(ds->work,Y,mout*ld);CHKERRQ(ierr);
     PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&done,Q,&ld,ds->work,&inc,&zero,Y,&inc));
 #if !defined(PETSC_USE_COMPLEX)
     if (iscomplex) PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&done,Q,&ld,ds->work+ld,&inc,&zero,Y+ld,&inc));
@@ -177,15 +165,10 @@ static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rno
     else *rnorm = PetscAbsScalar(Y[n-1]);
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
 {
-#if defined(SLEPC_MISSING_LAPACK_TREVC)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   n,ld,mout,info,inc = 1;
@@ -210,7 +193,7 @@ static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
   if (ds->state>=DS_STATE_CONDENSED) {
     /* DSSolve() has been called, backtransform with matrix Q */
     back = "B";
-    ierr = PetscMemcpy(Z,ds->mat[DS_MAT_Q],ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(Z,ds->mat[DS_MAT_Q],ld*ld);CHKERRQ(ierr);
   } else back = "A";
 #if !defined(PETSC_USE_COMPLEX)
   ierr = DSAllocateWork_Private(ds,3*ld,0,0);CHKERRQ(ierr);
@@ -239,7 +222,6 @@ static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
     if (iscomplex) i++;
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
@@ -287,10 +269,6 @@ PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 
 static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
-#if defined(PETSC_MISSING_LAPACK_TRSEN)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TRSEN - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   info,n,ld,mout,lwork,*selection;
@@ -320,7 +298,7 @@ static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *w
 #endif
   /* Compute the selected eigenvalue to be in the leading position */
   ierr = DSSortEigenvalues_Private(ds,rr,ri,ds->perm,PETSC_FALSE);CHKERRQ(ierr);
-  ierr = PetscMemzero(selection,n*sizeof(PetscBLASInt));CHKERRQ(ierr);
+  ierr = PetscArrayzero(selection,n);CHKERRQ(ierr);
   for (i=0;i<*k;i++) selection[ds->perm[i]] = 1;
 #if !defined(PETSC_USE_COMPLEX)
   PetscStackCallBLAS("LAPACKtrsen",LAPACKtrsen_("N","V",selection,&n,T,&ld,Q,&ld,wr,wi,&mout,NULL,NULL,work,&lwork,iwork,&liwork,&info));
@@ -330,15 +308,10 @@ static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *w
   SlepcCheckLapackInfo("trsen",info);
   *k = mout;
   PetscFunctionReturn(0);
-#endif
 }
 
 static PetscErrorCode DSSort_NHEP_Total(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_TREXC)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TREXC - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PetscScalar    re;
   PetscInt       i,j,pos,result;
@@ -417,7 +390,6 @@ static PetscErrorCode DSSort_NHEP_Total(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSort_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
@@ -433,6 +405,70 @@ PetscErrorCode DSSort_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode DSSortWithPermutation_NHEP(DS ds,PetscInt *perm,PetscScalar *wr,PetscScalar *wi)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,j,pos,inc=1;
+  PetscBLASInt   ifst,ilst,info,n,ld;
+  PetscScalar    *T = ds->mat[DS_MAT_A];
+  PetscScalar    *Q = ds->mat[DS_MAT_Q];
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar    *work;
+#endif
+
+  PetscFunctionBegin;
+  ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+  ierr = DSAllocateWork_Private(ds,ld,0,0);CHKERRQ(ierr);
+  work = ds->work;
+#endif
+  for (i=ds->l;i<n-1;i++) {
+    pos = perm[i];
+#if !defined(PETSC_USE_COMPLEX)
+    inc = (pos<n-1 && T[pos+1+pos*ld] != 0.0)? 2: 1;
+#endif
+    if (pos!=i) {
+#if !defined(PETSC_USE_COMPLEX)
+      if ((T[pos+(pos-1)*ld] != 0.0 && perm[i+1]!=pos-1) || (T[pos+1+pos*ld] != 0.0 && perm[i+1]!=pos+1))
+ SETERRQ1(PETSC_COMM_SELF,1,"Invalid permutation due to a 2x2 block at position %D",pos);
+#endif
+
+      /* interchange blocks */
+      ierr = PetscBLASIntCast(pos+1,&ifst);CHKERRQ(ierr);
+      ierr = PetscBLASIntCast(i+1,&ilst);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+      PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,work,&info));
+#else
+      PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,&info));
+#endif
+      SlepcCheckLapackInfo("trexc",info);
+      for (j=i+1;j<n;j++) {
+        if (perm[j]>=i && perm[j]<pos) perm[j]+=inc;
+      }
+      perm[i] = i;
+      if (inc==2) perm[i+1] = i+1;
+    }
+    if (inc==2) i++;
+  }
+  /* recover original eigenvalues from T matrix */
+  for (j=ds->l;j<n;j++) {
+    wr[j] = T[j+j*ld];
+#if !defined(PETSC_USE_COMPLEX)
+    if (j<n-1 && T[j+1+j*ld] != 0.0) {
+      /* complex conjugate eigenvalue */
+      wi[j] = PetscSqrtReal(PetscAbsReal(T[j+1+j*ld])) * PetscSqrtReal(PetscAbsReal(T[j+(j+1)*ld]));
+      wr[j+1] = wr[j];
+      wi[j+1] = -wi[j];
+      j++;
+    } else {
+      wi[j] = 0.0;
+    }
+#endif
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode DSUpdateExtraRow_NHEP(DS ds)
 {
   PetscErrorCode ierr;
@@ -466,7 +502,7 @@ PetscErrorCode DSUpdateExtraRow_NHEP(DS ds)
   where S is an upper (quasi-)triangular matrix of order k, H is an upper Hessenberg
   matrix of order n-k, and R is all zeros except the first row (the arrow).
   The algorithm uses elementary reflectors to annihilate entries in the arrow, and
-  then proceeds upwards. 
+  then proceeds upwards.
   If ilo>1, then it is assumed that the first ilo-1 entries of the arrow are zero, and
   hence the first ilo-1 rows and columns of Q are set to the identity matrix.
 
@@ -474,10 +510,6 @@ PetscErrorCode DSUpdateExtraRow_NHEP(DS ds)
 */
 static PetscErrorCode ArrowHessenberg(PetscBLASInt n,PetscBLASInt k,PetscBLASInt ilo,PetscScalar *A,PetscBLASInt lda,PetscScalar *Q,PetscBLASInt ldq,PetscScalar *work)
 {
-#if defined(SLEPC_MISSING_LAPACK_LARFG) || defined(SLEPC_MISSING_LAPACK_LARF)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LARFG/LARF - Lapack routines are unavailable");
-#else
   PetscBLASInt i,j,n0,m,inc=1,incn=-1;
   PetscScalar  t,*v=work,*w=work+n,tau,tauc;
 
@@ -505,15 +537,10 @@ static PetscErrorCode ArrowHessenberg(PetscBLASInt n,PetscBLASInt k,PetscBLASInt
     }
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-#if defined(SLEPC_MISSING_LAPACK_GEHRD) || defined(SLEPC_MISSING_LAPACK_ORGHR) || defined(PETSC_MISSING_LAPACK_HSEQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEHRD/ORGHR/HSEQR - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscScalar    *work,*tau;
   PetscInt       i,j;
@@ -523,7 +550,7 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(wi,3);
+  PetscValidScalarPointer(wi,3);
 #endif
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
@@ -535,7 +562,7 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
   lwork = 6*ld;
 
   /* initialize orthogonal matrix */
-  ierr = PetscMemzero(Q,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(Q,ld*ld);CHKERRQ(ierr);
   for (i=0;i<n;i++) Q[i+i*ld] = 1.0;
   if (n==1) { /* quick return */
     wr[0] = A[0];
@@ -585,7 +612,6 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
   SlepcCheckLapackInfo("hseqr",info);
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSSynchronize_NHEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
@@ -597,8 +623,8 @@ PetscErrorCode DSSynchronize_NHEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
   PetscFunctionBegin;
   k = (ds->n-l)*ld;
   if (ds->state>DS_STATE_RAW) k += (ds->n-l)*ld;
-  if (eigr) k += ds->n-l; 
-  if (eigi) k += ds->n-l; 
+  if (eigr) k += ds->n-l;
+  if (eigi) k += ds->n-l;
   ierr = DSAllocateWork_Private(ds,k,0,0);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(k*sizeof(PetscScalar),&size);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
@@ -658,10 +684,6 @@ PetscErrorCode DSTruncate_NHEP(DS ds,PetscInt n)
 
 PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
 {
-#if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(SLEPC_MISSING_LAPACK_LANGE) || defined(SLEPC_MISSING_LAPACK_LANHS)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRF/GETRI/LANGE/LANHS - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscScalar    *work;
   PetscReal      *rwork;
@@ -682,7 +704,7 @@ PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
   /* use workspace matrix W to avoid overwriting A */
   ierr = DSAllocateMat_Private(ds,DS_MAT_W);CHKERRQ(ierr);
   A = ds->mat[DS_MAT_W];
-  ierr = PetscMemcpy(A,ds->mat[DS_MAT_A],sizeof(PetscScalar)*ds->ld*ds->ld);CHKERRQ(ierr);
+  ierr = PetscArraycpy(A,ds->mat[DS_MAT_A],ds->ld*ds->ld);CHKERRQ(ierr);
 
   /* norm of A */
   if (ds->state<DS_STATE_INTERMEDIATE) hn = LAPACKlange_("I",&n,&n,A,&ld,rwork);
@@ -697,15 +719,10 @@ PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
 
   *cond = hn*hin;
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,PetscBool recover,PetscScalar *gin,PetscReal *gamma)
 {
-#if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRS)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRF/GETRS - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,j;
   PetscBLASInt   *ipiv,info,n,ld,one=1,ncol;
@@ -729,10 +746,10 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
     /* use workspace matrix W to factor A-tau*eye(n) */
     ierr = DSAllocateMat_Private(ds,DS_MAT_W);CHKERRQ(ierr);
     B = ds->mat[DS_MAT_W];
-    ierr = PetscMemcpy(B,A,sizeof(PetscScalar)*ld*ld);CHKERRQ(ierr);
+    ierr = PetscArraycpy(B,A,ld*ld);CHKERRQ(ierr);
 
     /* Vector g initialy stores b = beta*e_n^T */
-    ierr = PetscMemzero(g,n*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArrayzero(g,n);CHKERRQ(ierr);
     g[n-1] = beta;
 
     /* g = (A-tau*eye(n))'\b */
@@ -749,7 +766,6 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
 
   } else { /* recover */
 
-    PetscValidPointer(g,6);
     ierr = DSAllocateWork_Private(ds,ld,0,0);CHKERRQ(ierr);
     ghat = ds->work;
     Q    = ds->mat[DS_MAT_Q];
@@ -774,10 +790,9 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
     *gamma = PetscSqrtReal(1.0+gnorm);
   }
   PetscFunctionReturn(0);
-#endif
 }
 
-PETSC_EXTERN PetscErrorCode DSCreate_NHEP(DS ds)
+SLEPC_EXTERN PetscErrorCode DSCreate_NHEP(DS ds)
 {
   PetscFunctionBegin;
   ds->ops->allocate      = DSAllocate_NHEP;
@@ -785,6 +800,7 @@ PETSC_EXTERN PetscErrorCode DSCreate_NHEP(DS ds)
   ds->ops->vectors       = DSVectors_NHEP;
   ds->ops->solve[0]      = DSSolve_NHEP;
   ds->ops->sort          = DSSort_NHEP;
+  ds->ops->sortperm      = DSSortWithPermutation_NHEP;
   ds->ops->synchronize   = DSSynchronize_NHEP;
   ds->ops->truncate      = DSTruncate_NHEP;
   ds->ops->update        = DSUpdateExtraRow_NHEP;
