@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -47,10 +47,6 @@ PetscErrorCode FNEvaluateFunction_Rational(FN fn,PetscScalar x,PetscScalar *y)
 
 static PetscErrorCode FNEvaluateFunctionMat_Rational_Private(FN fn,PetscScalar *Aa,PetscScalar *Ba,PetscInt m,PetscBool firstonly)
 {
-#if defined(PETSC_MISSING_LAPACK_GESV)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESV - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscBLASInt   n,k,ld,*ipiv,info;
@@ -67,24 +63,24 @@ static PetscErrorCode FNEvaluateFunctionMat_Rational_Private(FN fn,PetscScalar *
     P = Ba;
     ierr = PetscMalloc3(m*m,&Q,m*m,&W,ld,&ipiv);CHKERRQ(ierr);
   }
-  ierr = PetscMemzero(P,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(P,m*m);CHKERRQ(ierr);
   if (!ctx->np) {
     for (i=0;i<m;i++) P[i+i*ld] = 1.0;
   } else {
     for (i=0;i<m;i++) P[i+i*ld] = ctx->pcoeff[0];
     for (j=1;j<ctx->np;j++) {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&one,P,&ld,Aa,&ld,&zero,W,&ld));
-      ierr = PetscMemcpy(P,W,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(P,W,m*m);CHKERRQ(ierr);
       for (i=0;i<m;i++) P[i+i*ld] += ctx->pcoeff[j];
     }
     ierr = PetscLogFlops(2.0*n*n*n*(ctx->np-1));CHKERRQ(ierr);
   }
   if (ctx->nq) {
-    ierr = PetscMemzero(Q,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArrayzero(Q,m*m);CHKERRQ(ierr);
     for (i=0;i<m;i++) Q[i+i*ld] = ctx->qcoeff[0];
     for (j=1;j<ctx->nq;j++) {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&one,Q,&ld,Aa,&ld,&zero,W,&ld));
-      ierr = PetscMemcpy(Q,W,m*m*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = PetscArraycpy(Q,W,m*m);CHKERRQ(ierr);
       for (i=0;i<m;i++) Q[i+i*ld] += ctx->qcoeff[j];
     }
     PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&n,&k,Q,&ld,ipiv,P,&ld,&info));
@@ -92,13 +88,12 @@ static PetscErrorCode FNEvaluateFunctionMat_Rational_Private(FN fn,PetscScalar *
     ierr = PetscLogFlops(2.0*n*n*n*(ctx->nq-1)+2.0*n*n*n/3.0+2.0*n*n*k);CHKERRQ(ierr);
   }
   if (Aa==Ba) {
-    ierr = PetscMemcpy(Aa,P,m*k*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(Aa,P,m*k);CHKERRQ(ierr);
     ierr = PetscFree4(P,Q,W,ipiv);CHKERRQ(ierr);
   } else {
     ierr = PetscFree3(Q,W,ipiv);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode FNEvaluateFunctionMat_Rational(FN fn,Mat A,Mat B)
@@ -258,7 +253,7 @@ static PetscErrorCode FNRationalSetNumerator_Rational(FN fn,PetscInt np,PetscSca
    FNRationalSetNumerator - Sets the parameters defining the numerator of the
    rational function.
 
-   Logically Collective on FN
+   Logically Collective on fn
 
    Input Parameters:
 +  fn     - the math function context
@@ -285,7 +280,7 @@ PetscErrorCode FNRationalSetNumerator(FN fn,PetscInt np,PetscScalar *pcoeff)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,np,2);
-  if (np) PetscValidPointer(pcoeff,3);
+  if (np) PetscValidScalarPointer(pcoeff,3);
   ierr = PetscTryMethod(fn,"FNRationalSetNumerator_C",(FN,PetscInt,PetscScalar*),(fn,np,pcoeff));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -362,7 +357,7 @@ static PetscErrorCode FNRationalSetDenominator_Rational(FN fn,PetscInt nq,PetscS
    FNRationalSetDenominator - Sets the parameters defining the denominator of the
    rational function.
 
-   Logically Collective on FN
+   Logically Collective on fn
 
    Input Parameters:
 +  fn     - the math function context
@@ -389,7 +384,7 @@ PetscErrorCode FNRationalSetDenominator(FN fn,PetscInt nq,PetscScalar *qcoeff)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,nq,2);
-  if (nq) PetscValidPointer(qcoeff,3);
+  if (nq) PetscValidScalarPointer(qcoeff,3);
   ierr = PetscTryMethod(fn,"FNRationalSetDenominator_C",(FN,PetscInt,PetscScalar*),(fn,nq,qcoeff));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -426,8 +421,8 @@ static PetscErrorCode FNRationalGetDenominator_Rational(FN fn,PetscInt *nq,Petsc
 -  qcoeff - coefficients (array of scalar values, length nq)
 
    Notes:
-   The values passed by user with FNRationalSetDenominator() are returned (or null
-   pointers otherwise).
+   The values passed by user with FNRationalSetDenominator() are returned (or a null
+   pointer otherwise).
    The qcoeff array should be freed by the user when no longer needed.
 
    Level: intermediate
@@ -507,7 +502,7 @@ PetscErrorCode FNDestroy_Rational(FN fn)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode FNCreate_Rational(FN fn)
+SLEPC_EXTERN PetscErrorCode FNCreate_Rational(FN fn)
 {
   PetscErrorCode ierr;
   FN_RATIONAL    *ctx;
