@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -114,13 +114,9 @@ PetscErrorCode FNEvaluateFunctionMat_Sqrt_NS(FN fn,Mat A,Mat B)
  */
 static PetscErrorCode SlepcSqrtmSadeghi(PetscBLASInt n,PetscScalar *A,PetscBLASInt ld)
 {
-#if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRI)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRF/GETRI - Lapack routine is unavailable");
-#else
   PetscScalar        *M,*M2,*G,*X=A,*work,work1,alpha,sqrtnrm;
   PetscScalar        szero=0.0,sone=1.0,smfive=-5.0,s1d16=1.0/16.0;
-  PetscReal          tol,Mres,nrm,rwork[1];
+  PetscReal          tol,Mres=0.0,nrm,rwork[1];
   PetscBLASInt       N,i,it,*piv=NULL,info,lwork,query=-1;
   const PetscBLASInt one=1;
   PetscBool          converged=PETSC_FALSE;
@@ -137,7 +133,7 @@ static PetscErrorCode SlepcSqrtmSadeghi(PetscBLASInt n,PetscScalar *A,PetscBLASI
   ierr = PetscBLASIntCast((PetscInt)PetscRealPart(work1),&lwork);CHKERRQ(ierr);
 
   ierr = PetscMalloc5(N,&M,N,&M2,N,&G,lwork,&work,n,&piv);CHKERRQ(ierr);
-  ierr = PetscMemcpy(M,A,N*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArraycpy(M,A,N);CHKERRQ(ierr);
 
   /* scale M */
   nrm = LAPACKlange_("fro",&n,&n,M,&n,rwork);
@@ -147,10 +143,10 @@ static PetscErrorCode SlepcSqrtmSadeghi(PetscBLASInt n,PetscScalar *A,PetscBLASI
     PetscStackCallBLAS("BLASscal",BLASscal_(&N,&alpha,M,&one));
     tol *= nrm;
   }
-  ierr = PetscInfo2(NULL,"||A||_F = %g, new tol: %g\n",(double)nrm,(double)tol);
+  ierr = PetscInfo2(NULL,"||A||_F = %g, new tol: %g\n",(double)nrm,(double)tol);CHKERRQ(ierr);
 
   /* X = I */
-  ierr = PetscMemzero(X,N*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscArrayzero(X,N);CHKERRQ(ierr);
   for (i=0;i<n;i++) X[i+i*ld] = 1.0;
 
   for (it=0;it<MAXIT && !converged;it++) {
@@ -163,7 +159,7 @@ static PetscErrorCode SlepcSqrtmSadeghi(PetscBLASInt n,PetscScalar *A,PetscBLASI
     for (i=0;i<n;i++) G[i+i*ld] += 5.0/16.0;
 
     /* X = X*G */
-    ierr = PetscMemcpy(M2,X,N*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(M2,X,N);CHKERRQ(ierr);
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,M2,&ld,G,&ld,&szero,X,&ld));
 
     /* M = M*inv(G*G) */
@@ -173,16 +169,16 @@ static PetscErrorCode SlepcSqrtmSadeghi(PetscBLASInt n,PetscScalar *A,PetscBLASI
     PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,M2,&ld,piv,work,&lwork,&info));
     SlepcCheckLapackInfo("getri",info);
 
-    ierr = PetscMemcpy(G,M,N*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(G,M,N);CHKERRQ(ierr);
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,G,&ld,M2,&ld,&szero,M,&ld));
 
     /* check ||I-M|| */
-    ierr = PetscMemcpy(M2,M,N*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscArraycpy(M2,M,N);CHKERRQ(ierr);
     for (i=0;i<n;i++) M2[i+i*ld] -= 1.0;
     Mres = LAPACKlange_("fro",&n,&n,M2,&n,rwork);
     ierr = PetscIsNanReal(Mres);CHKERRQ(ierr);
     if (Mres<=tol) converged = PETSC_TRUE;
-    ierr = PetscInfo2(NULL,"it: %D res: %g\n",it,(double)Mres);
+    ierr = PetscInfo2(NULL,"it: %D res: %g\n",it,(double)Mres);CHKERRQ(ierr);
     ierr = PetscLogFlops(8.0*n*n*n+2.0*n*n+2.0*n*n*n/3.0+4.0*n*n*n/3.0+2.0*n*n*n+2.0*n*n);CHKERRQ(ierr);
   }
 
@@ -194,7 +190,6 @@ static PetscErrorCode SlepcSqrtmSadeghi(PetscBLASInt n,PetscScalar *A,PetscBLASI
   ierr = PetscFree5(M,M2,G,work,piv);CHKERRQ(ierr);
   ierr = SlepcResetFlushToZero(&ftz);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 PetscErrorCode FNEvaluateFunctionMat_Sqrt_Sadeghi(FN fn,Mat A,Mat B)
@@ -256,7 +251,7 @@ PetscErrorCode FNView_Sqrt(FN fn,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode FNCreate_Sqrt(FN fn)
+SLEPC_EXTERN PetscErrorCode FNCreate_Sqrt(FN fn)
 {
   PetscFunctionBegin;
   fn->ops->evaluatefunction          = FNEvaluateFunction_Sqrt;
