@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -50,6 +50,7 @@ PetscErrorCode RGInitializePackage(void)
 {
   char           logList[256];
   PetscBool      opt,pkg;
+  PetscClassId   classids[1];
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -59,12 +60,9 @@ PetscErrorCode RGInitializePackage(void)
   ierr = PetscClassIdRegister("Region",&RG_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = RGRegisterAll();CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrInList("rg",logList,',',&pkg);CHKERRQ(ierr);
-    if (pkg) { ierr = PetscInfoDeactivateClass(RG_CLASSID);CHKERRQ(ierr); }
-  }
+  /* Process Info */
+  classids[0] = RG_CLASSID;
+  ierr = PetscInfoProcessClass("rg",1,&classids[0]);CHKERRQ(ierr);
   /* Process summary exclusions */
   ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
@@ -79,7 +77,7 @@ PetscErrorCode RGInitializePackage(void)
 /*@
    RGCreate - Creates an RG context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -114,7 +112,7 @@ PetscErrorCode RGCreate(MPI_Comm comm,RG *newrg)
    RGSetOptionsPrefix - Sets the prefix used for searching for all
    RG options in the database.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameters:
 +  rg     - the region context
@@ -143,7 +141,7 @@ PetscErrorCode RGSetOptionsPrefix(RG rg,const char *prefix)
    RGAppendOptionsPrefix - Appends to the prefix used for searching for all
    RG options in the database.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameters:
 +  rg     - the region context
@@ -201,7 +199,7 @@ PetscErrorCode RGGetOptionsPrefix(RG rg,const char *prefix[])
 /*@C
    RGSetType - Selects the type for the RG object.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameter:
 +  rg   - the region context
@@ -261,7 +259,7 @@ PetscErrorCode RGGetType(RG rg,RGType *type)
 /*@
    RGSetFromOptions - Sets RG options from the options database.
 
-   Collective on RG
+   Collective on rg
 
    Input Parameters:
 .  rg - the region context
@@ -305,7 +303,7 @@ PetscErrorCode RGSetFromOptions(RG rg)
 /*@C
    RGView - Prints the RG data structure.
 
-   Collective on RG
+   Collective on rg
 
    Input Parameters:
 +  rg - the region context
@@ -327,19 +325,18 @@ PetscErrorCode RGSetFromOptions(RG rg)
 PetscErrorCode RGView(RG rg,PetscViewer viewer)
 {
   PetscBool      isdraw,isascii;
-  PetscInt       tabs;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)rg));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)rg),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(rg,1,viewer,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)rg)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)rg,viewer);CHKERRQ(ierr);
     if (rg->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -352,13 +349,36 @@ PetscErrorCode RGView(RG rg,PetscViewer viewer)
     if (rg->sfactor!=1.0) {
       ierr = PetscViewerASCIIPrintf(viewer,"  scaling factor = %g\n",(double)rg->sfactor);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else if (isdraw) {
     if (rg->ops->view) { ierr = (*rg->ops->view)(rg,viewer);CHKERRQ(ierr); }
   }
   PetscFunctionReturn(0);
 }
 
+/*@C
+   RGViewFromOptions - View from options
+
+   Collective on RG
+
+   Input Parameters:
++  rg   - the region context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: RGView(), RGCreate()
+@*/
+PetscErrorCode RGViewFromOptions(RG rg,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(rg,RG_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)rg,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    RGIsTrivial - Whether it is the trivial region (whole complex plane).
 
@@ -381,7 +401,7 @@ PetscErrorCode RGIsTrivial(RG rg,PetscBool *trivial)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
   PetscValidType(rg,1);
-  PetscValidPointer(trivial,2);
+  PetscValidBoolPointer(trivial,2);
   if (rg->ops->istrivial) {
     ierr = (*rg->ops->istrivial)(rg,trivial);CHKERRQ(ierr);
   } else *trivial = PETSC_FALSE;
@@ -422,11 +442,11 @@ PetscErrorCode RGCheckInside(RG rg,PetscInt n,PetscScalar *ar,PetscScalar *ai,Pe
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
   PetscValidType(rg,1);
-  PetscValidPointer(ar,3);
+  PetscValidScalarPointer(ar,3);
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(ai,4);
+  PetscValidScalarPointer(ai,4);
 #endif
-  PetscValidPointer(inside,5);
+  PetscValidIntPointer(inside,5);
 
   for (i=0;i<n;i++) {
 #if defined(PETSC_USE_COMPLEX)
@@ -447,7 +467,7 @@ PetscErrorCode RGCheckInside(RG rg,PetscInt n,PetscScalar *ar,PetscScalar *ai,Pe
 }
 
 /*@
-   RGComputeContour - Computes the coordinates of several points lying in the
+   RGComputeContour - Computes the coordinates of several points lying on the
    contour of the region.
 
    Not Collective
@@ -470,9 +490,9 @@ PetscErrorCode RGComputeContour(RG rg,PetscInt n,PetscScalar cr[],PetscScalar ci
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
   PetscValidType(rg,1);
-  PetscValidPointer(cr,3);
+  PetscValidScalarPointer(cr,3);
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(ci,4);
+  PetscValidScalarPointer(ci,4);
 #endif
   if (rg->complement) SETERRQ(PetscObjectComm((PetscObject)rg),PETSC_ERR_SUP,"Cannot compute contour of region with complement flag set");
   ierr = (*rg->ops->computecontour)(rg,n,cr,ci);CHKERRQ(ierr);
@@ -532,7 +552,7 @@ PetscErrorCode RGComputeBoundingBox(RG rg,PetscReal *a,PetscReal *b,PetscReal *c
    RGSetComplement - Sets a flag to indicate that the region is the complement
    of the specified one.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameters:
 +  rg  - the region context
@@ -574,7 +594,7 @@ PetscErrorCode RGGetComplement(RG rg,PetscBool *flg)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
-  PetscValidPointer(flg,2);
+  PetscValidBoolPointer(flg,2);
   *flg = rg->complement;
   PetscFunctionReturn(0);
 }
@@ -583,7 +603,7 @@ PetscErrorCode RGGetComplement(RG rg,PetscBool *flg)
    RGSetScale - Sets the scaling factor to be used when checking that a
    point is inside the region and when computing the contour.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameters:
 +  rg      - the region context
@@ -628,7 +648,7 @@ PetscErrorCode RGGetScale(RG rg,PetscReal *sfactor)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
-  PetscValidPointer(sfactor,2);
+  PetscValidRealPointer(sfactor,2);
   *sfactor = rg->sfactor;
   PetscFunctionReturn(0);
 }
@@ -637,7 +657,7 @@ PetscErrorCode RGGetScale(RG rg,PetscReal *sfactor)
    RGPushScale - Sets an additional scaling factor, that will multiply the
    user-defined scaling factor.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameters:
 +  rg      - the region context
@@ -690,7 +710,7 @@ PetscErrorCode RGPopScale(RG rg)
 /*@
    RGDestroy - Destroys RG context that was created with RGCreate().
 
-   Collective on RG
+   Collective on rg
 
    Input Parameter:
 .  rg - the region context
@@ -733,6 +753,7 @@ PetscErrorCode RGRegister(const char *name,PetscErrorCode (*function)(RG))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = RGInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&RGList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
