@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -14,7 +14,7 @@
 #include <slepc/private/stimpl.h>            /*I "slepcst.h" I*/
 
 PetscClassId     ST_CLASSID = 0;
-PetscLogEvent    ST_SetUp = 0,ST_Apply = 0,ST_ApplyTranspose = 0,ST_MatSetUp = 0,ST_MatMult = 0,ST_MatMultTranspose = 0,ST_MatSolve = 0,ST_MatSolveTranspose = 0;
+PetscLogEvent    ST_SetUp = 0,ST_ComputeOperator = 0,ST_Apply = 0,ST_ApplyTranspose = 0,ST_MatSetUp = 0,ST_MatMult = 0,ST_MatMultTranspose = 0,ST_MatSolve = 0,ST_MatSolveTranspose = 0;
 static PetscBool STPackageInitialized = PETSC_FALSE;
 
 const char *STMatModes[] = {"COPY","INPLACE","SHELL","STMatMode","ST_MATMODE_",0};
@@ -51,6 +51,7 @@ PetscErrorCode STInitializePackage(void)
 {
   char           logList[256];
   PetscBool      opt,pkg;
+  PetscClassId   classids[1];
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -62,6 +63,7 @@ PetscErrorCode STInitializePackage(void)
   ierr = STRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("STSetUp",ST_CLASSID,&ST_SetUp);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("STComputeOperator",ST_CLASSID,&ST_ComputeOperator);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STApply",ST_CLASSID,&ST_Apply);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STApplyTranspose",ST_CLASSID,&ST_ApplyTranspose);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STMatSetUp",ST_CLASSID,&ST_MatSetUp);CHKERRQ(ierr);
@@ -69,12 +71,9 @@ PetscErrorCode STInitializePackage(void)
   ierr = PetscLogEventRegister("STMatMultTranspose",ST_CLASSID,&ST_MatMultTranspose);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STMatSolve",ST_CLASSID,&ST_MatSolve);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("STMatSolveTranspose",ST_CLASSID,&ST_MatSolveTranspose);CHKERRQ(ierr);
-  /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
-  if (opt) {
-    ierr = PetscStrInList("st",logList,',',&pkg);CHKERRQ(ierr);
-    if (pkg) { ierr = PetscInfoDeactivateClass(ST_CLASSID);CHKERRQ(ierr); }
-  }
+  /* Process Info */
+  classids[0] = ST_CLASSID;
+  ierr = PetscInfoProcessClass("st",1,&classids[0]);CHKERRQ(ierr);
   /* Process summary exclusions */
   ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
@@ -90,7 +89,7 @@ PetscErrorCode STInitializePackage(void)
    STReset - Resets the ST context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on ST
+   Collective on st
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -106,24 +105,28 @@ PetscErrorCode STReset(ST st)
   PetscFunctionBegin;
   if (st) PetscValidHeaderSpecific(st,ST_CLASSID,1);
   if (!st) PetscFunctionReturn(0);
+  STCheckNotSeized(st,1);
   if (st->ops->reset) { ierr = (*st->ops->reset)(st);CHKERRQ(ierr); }
   if (st->ksp) { ierr = KSPReset(st->ksp);CHKERRQ(ierr); }
   ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->T);CHKERRQ(ierr);
   ierr = MatDestroyMatrices(PetscMax(2,st->nmat),&st->A);CHKERRQ(ierr);
   ierr = PetscFree(st->Astate);CHKERRQ(ierr);
+  ierr = MatDestroy(&st->Op);CHKERRQ(ierr);
   ierr = MatDestroy(&st->P);CHKERRQ(ierr);
   ierr = VecDestroyVecs(st->nwork,&st->work);CHKERRQ(ierr);
   st->nwork = 0;
   ierr = VecDestroy(&st->wb);CHKERRQ(ierr);
+  ierr = VecDestroy(&st->wht);CHKERRQ(ierr);
   ierr = VecDestroy(&st->D);CHKERRQ(ierr);
-  st->state = ST_STATE_INITIAL;
+  st->state   = ST_STATE_INITIAL;
+  st->opready = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
 /*@
    STDestroy - Destroys ST context that was created with STCreate().
 
-   Collective on ST
+   Collective on st
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -150,7 +153,7 @@ PetscErrorCode STDestroy(ST *st)
 /*@
    STCreate - Creates a spectral transformation context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
@@ -174,33 +177,76 @@ PetscErrorCode STCreate(MPI_Comm comm,ST *newst)
   ierr = SlepcHeaderCreate(st,ST_CLASSID,"ST","Spectral Transformation","ST",comm,STDestroy,STView);CHKERRQ(ierr);
 
   st->A            = NULL;
-  st->Astate       = NULL;
-  st->T            = NULL;
-  st->P            = NULL;
   st->nmat         = 0;
   st->sigma        = 0.0;
-  st->sigma_set    = PETSC_FALSE;
   st->defsigma     = 0.0;
-  st->shift_matrix = ST_MATMODE_COPY;
+  st->matmode      = ST_MATMODE_COPY;
   st->str          = DIFFERENT_NONZERO_PATTERN;
   st->transform    = PETSC_FALSE;
+  st->D            = NULL;
 
   st->ksp          = NULL;
+  st->usesksp      = PETSC_FALSE;
   st->nwork        = 0;
   st->work         = NULL;
-  st->D            = NULL;
   st->wb           = NULL;
-  st->data         = NULL;
+  st->wht          = NULL;
   st->state        = ST_STATE_INITIAL;
+  st->Astate       = NULL;
+  st->T            = NULL;
+  st->Op           = NULL;
+  st->opseized     = PETSC_FALSE;
+  st->opready      = PETSC_FALSE;
+  st->P            = NULL;
+  st->M            = NULL;
+  st->sigma_set    = PETSC_FALSE;
+  st->asymm        = PETSC_FALSE;
+  st->aherm        = PETSC_FALSE;
+  st->data         = NULL;
 
   *newst = st;
   PetscFunctionReturn(0);
 }
 
+/*
+   Checks whether the ST matrices are all symmetric or hermitian.
+*/
+PETSC_STATIC_INLINE PetscErrorCode STMatIsSymmetricKnown(ST st,PetscBool *symm,PetscBool *herm)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscBool      sbaij=PETSC_FALSE,set,flg=PETSC_FALSE;
+
+  PetscFunctionBegin;
+  /* check if problem matrices are all sbaij */
+  for (i=0;i<st->nmat;i++) {
+    ierr = PetscObjectTypeCompareAny((PetscObject)st->A[i],&sbaij,MATSEQSBAIJ,MATMPISBAIJ,"");CHKERRQ(ierr);
+    if (!sbaij) break;
+  }
+  /* check if user has set the symmetric flag */
+  *symm = PETSC_TRUE;
+  for (i=0;i<st->nmat;i++) {
+    ierr = MatIsSymmetricKnown(st->A[i],&set,&flg);CHKERRQ(ierr);
+    if (!set || !flg) { *symm = PETSC_FALSE; break; }
+  }
+  if (sbaij) *symm = PETSC_TRUE;
+#if defined(PETSC_USE_COMPLEX)
+  /* check if user has set the hermitian flag */
+  *herm = PETSC_TRUE;
+  for (i=0;i<st->nmat;i++) {
+    ierr = MatIsHermitianKnown(st->A[i],&set,&flg);CHKERRQ(ierr);
+    if (!set || !flg) { *herm = PETSC_FALSE; break; }
+  }
+#else
+  *herm = *symm;
+#endif
+  PetscFunctionReturn(0);
+}
+
 /*@
    STSetMatrices - Sets the matrices associated with the eigenvalue problem.
 
-   Collective on ST and Mat
+   Collective on st
 
    Input Parameters:
 +  st - the spectral transformation context
@@ -227,6 +273,8 @@ PetscErrorCode STSetMatrices(ST st,PetscInt n,Mat A[])
   if (n <= 0) SETERRQ1(PetscObjectComm((PetscObject)st),PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more matrices, you have %D",n);
   PetscValidPointer(A,3);
   PetscCheckSameComm(st,1,*A,3);
+  STCheckNotSeized(st,1);
+
   if (st->state) {
     if (n!=st->nmat) same = PETSC_FALSE;
     for (i=0;same&&i<n;i++) {
@@ -256,6 +304,10 @@ PetscErrorCode STSetMatrices(ST st,PetscInt n,Mat A[])
   st->nmat = n;
   if (same) st->state = ST_STATE_UPDATED;
   else st->state = ST_STATE_INITIAL;
+  st->opready = PETSC_FALSE;
+  if (!same) {
+    ierr = STMatIsSymmetricKnown(st,&st->asymm,&st->aherm);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -336,7 +388,7 @@ PetscErrorCode STGetNumMatrices(ST st,PetscInt *n)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidPointer(n,2);
+  PetscValidIntPointer(n,2);
   *n = st->nmat;
   PetscFunctionReturn(0);
 }
@@ -344,7 +396,7 @@ PetscErrorCode STGetNumMatrices(ST st,PetscInt *n)
 /*@
    STResetMatrixState - Resets the stored state of the matrices in the ST.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -372,7 +424,7 @@ PetscErrorCode STResetMatrixState(ST st)
 /*@
    STSetShift - Sets the shift associated with the spectral transformation.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st - the spectral transformation context
@@ -397,12 +449,13 @@ PetscErrorCode STSetShift(ST st,PetscScalar shift)
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidType(st,1);
   PetscValidLogicalCollectiveScalar(st,shift,2);
-  if (st->state==ST_STATE_SETUP && st->sigma != shift) {
-    if (st->ops->setshift) {
+  if (st->sigma != shift) {
+    STCheckNotSeized(st,1);
+    if (st->state==ST_STATE_SETUP && st->ops->setshift) {
       ierr = (*st->ops->setshift)(st,shift);CHKERRQ(ierr);
     }
+    st->sigma = shift;
   }
-  st->sigma = shift;
   st->sigma_set = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
@@ -435,7 +488,7 @@ PetscErrorCode STGetShift(ST st,PetscScalar* shift)
    STSetDefaultShift - Sets the value of the shift that should be employed if
    the user did not specify one.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st - the spectral transformation context
@@ -450,14 +503,18 @@ PetscErrorCode STSetDefaultShift(ST st,PetscScalar defaultshift)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
   PetscValidLogicalCollectiveScalar(st,defaultshift,2);
-  st->defsigma = defaultshift;
+  if (st->defsigma != defaultshift) {
+    st->defsigma = defaultshift;
+    st->state    = ST_STATE_INITIAL;
+    st->opready  = PETSC_FALSE;
+  }
   PetscFunctionReturn(0);
 }
 
 /*@
    STScaleShift - Multiply the shift with a given factor.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st     - the spectral transformation context
@@ -483,14 +540,15 @@ PetscErrorCode STScaleShift(ST st,PetscScalar factor)
 /*@
    STSetBalanceMatrix - Sets the diagonal matrix to be used for balancing.
 
-   Collective on ST and Vec
+   Collective on st
 
    Input Parameters:
 +  st - the spectral transformation context
 -  D  - the diagonal matrix (represented as a vector)
 
    Notes:
-   If this matrix is set, STApply will effectively apply D*OP*D^{-1}.
+   If this matrix is set, STApply will effectively apply D*OP*D^{-1}. Use NULL
+   to reset a previously passed D.
 
    Balancing is usually set via EPSSetBalance, but the advanced user may use
    this function to bypass the usual balancing methods.
@@ -505,12 +563,17 @@ PetscErrorCode STSetBalanceMatrix(ST st,Vec D)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidHeaderSpecific(D,VEC_CLASSID,2);
-  PetscCheckSameComm(st,1,D,2);
-  ierr = PetscObjectReference((PetscObject)D);CHKERRQ(ierr);
+  if (st->D == D) PetscFunctionReturn(0);
+  STCheckNotSeized(st,1);
+  if (D) {
+    PetscValidHeaderSpecific(D,VEC_CLASSID,2);
+    PetscCheckSameComm(st,1,D,2);
+    ierr = PetscObjectReference((PetscObject)D);CHKERRQ(ierr);
+  }
   ierr = VecDestroy(&st->D);CHKERRQ(ierr);
   st->D = D;
-  st->state = ST_STATE_INITIAL;
+  st->state   = ST_STATE_INITIAL;
+  st->opready = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
@@ -544,7 +607,7 @@ PetscErrorCode STGetBalanceMatrix(ST st,Vec *D)
 /*@C
    STMatCreateVecs - Get vector(s) compatible with the ST matrices.
 
-   Collective on ST
+   Collective on st
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -569,7 +632,7 @@ PetscErrorCode STMatCreateVecs(ST st,Vec *right,Vec *left)
    STMatCreateVecsEmpty - Get vector(s) compatible with the ST matrices, i.e. with the same
    parallel layout, but without internal array.
 
-   Collective on ST
+   Collective on st
 
    Input Parameter:
 .  st - the spectral transformation context
@@ -644,7 +707,7 @@ PetscErrorCode STMatGetLocalSize(ST st,PetscInt *m,PetscInt *n)
    STSetOptionsPrefix - Sets the prefix used for searching for all
    ST options in the database.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st     - the spectral transformation context
@@ -676,7 +739,7 @@ PetscErrorCode STSetOptionsPrefix(ST st,const char *prefix)
    STAppendOptionsPrefix - Appends to the prefix used for searching for all
    ST options in the database.
 
-   Logically Collective on ST
+   Logically Collective on st
 
    Input Parameters:
 +  st     - the spectral transformation context
@@ -738,7 +801,7 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
 /*@C
    STView - Prints the ST data structure.
 
-   Collective on ST
+   Collective on st
 
    Input Parameters:
 +  st - the ST context
@@ -757,7 +820,7 @@ PetscErrorCode STGetOptionsPrefix(ST st,const char *prefix[])
 
    Level: beginner
 
-.seealso: EPSView(), PetscViewerASCIIOpen()
+.seealso: EPSView()
 @*/
 PetscErrorCode STView(ST st,PetscViewer viewer)
 {
@@ -765,20 +828,19 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
   STType         cstr;
   const char*    pat=NULL;
   char           str[50];
-  PetscBool      isascii,isstring,flg;
-  PetscInt       tabs;
+  PetscBool      isascii,isstring;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)st));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)st),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(st,1,viewer,2);
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)st)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)st,viewer);CHKERRQ(ierr);
     if (st->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -788,7 +850,7 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
     ierr = SlepcSNPrintfScalar(str,50,st->sigma,PETSC_FALSE);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  shift: %s\n",str);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of matrices: %D\n",st->nmat);CHKERRQ(ierr);
-    switch (st->shift_matrix) {
+    switch (st->matmode) {
     case ST_MATMODE_COPY:
       break;
     case ST_MATMODE_INPLACE:
@@ -798,7 +860,7 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
       ierr = PetscViewerASCIIPrintf(viewer,"  using a shell matrix\n");CHKERRQ(ierr);
       break;
     }
-    if (st->nmat>1 && st->shift_matrix != ST_MATMODE_SHELL) {
+    if (st->nmat>1 && st->matmode != ST_MATMODE_SHELL) {
       switch (st->str) {
         case SAME_NONZERO_PATTERN:      pat = "same nonzero pattern";break;
         case DIFFERENT_NONZERO_PATTERN: pat = "different nonzero pattern";break;
@@ -809,20 +871,44 @@ PetscErrorCode STView(ST st,PetscViewer viewer)
     if (st->transform && st->nmat>2) {
       ierr = PetscViewerASCIIPrintf(viewer,"  computing transformed matrices\n");CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   } else if (isstring) {
     ierr = STGetType(st,&cstr);CHKERRQ(ierr);
     ierr = PetscViewerStringSPrintf(viewer," %-7.7s",cstr);CHKERRQ(ierr);
     if (st->ops->view) { ierr = (*st->ops->view)(st,viewer);CHKERRQ(ierr); }
   }
-  ierr = PetscObjectTypeCompare((PetscObject)st,STSHIFT,&flg);CHKERRQ(ierr);
-  if (st->nmat>1 || !flg) {
+  if (st->usesksp) {
     if (!st->ksp) { ierr = STGetKSP(st,&st->ksp);CHKERRQ(ierr); }
+    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = KSPView(st->ksp,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
+/*@C
+   STViewFromOptions - View from options
+
+   Collective on ST
+
+   Input Parameters:
++  st   - the spectral transformation context
+.  obj  - optional object
+-  name - command line option
+
+   Level: intermediate
+
+.seealso: STView(), STCreate()
+@*/
+PetscErrorCode STViewFromOptions(ST st,PetscObject obj,const char name[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(st,ST_CLASSID,1);
+  ierr = PetscObjectViewFromOptions((PetscObject)st,obj,name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    STRegister - Adds a method to the spectral transformation package.
 
@@ -855,6 +941,7 @@ PetscErrorCode STRegister(const char *name,PetscErrorCode (*function)(ST))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = STInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&STList,name,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
